@@ -15,9 +15,12 @@
     You should have received a copy of the GNU Library General Public
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+
+    cswstest - This is the main test application for the csws windowing
+    system.
 */
 
-#define CS_SYSDEF_PROVIDE_PATH
 #include "cssysdef.h"
 #include "csws/csws.h"
 #include "csver.h"
@@ -52,7 +55,8 @@ public:
 
   virtual bool Initialize ();
 
-  iFont *LucidiaFont;
+  csRef<iFont> VeraFont8;
+  csRef<iFont> VeraFont12;
 };
 
 //csWsTest *cswstest_app;                        // The main Windowing System object
@@ -138,10 +142,10 @@ public:
   virtual void Draw ()
   {
     csWindow::Draw();
-    SetFont (((csWsTest *)app)->LucidiaFont, 12);
+    SetFont (((csWsTest *)app)->VeraFont12);
     Text (BorderWidth+8, BorderHeight+100, CSPAL_WINDOW_LIGHT3D, -1,
       "This is a font 12 test");
-    SetFont (((csWsTest *)app)->LucidiaFont, 8);
+    SetFont (((csWsTest *)app)->VeraFont8);
     Text (BorderWidth+8, BorderHeight+140, CSPAL_WINDOW_LIGHT3D, -1,
       "This is a font 8 test");
   }
@@ -163,13 +167,11 @@ csWsTest::csWsTest (iObjectRegistry *object_reg, csSkin &Skin)
 {
   int pal = csRegisterPalette (palette_csWsTest, sizeof (palette_csWsTest) / sizeof (int));
   SetPalette (pal);
-  LucidiaFont = NULL;
 }
 
 csWsTest::~csWsTest ()
 {
-  if (LucidiaFont)
-    LucidiaFont->DecRef ();
+  csResetPalette();
 }
 
 bool csWsTest::Initialize ()
@@ -177,7 +179,8 @@ bool csWsTest::Initialize ()
   if (!csApp::Initialize ())
     return false;
 
-  LucidiaFont = LoadFont ("*courier");
+  VeraFont8 = LoadFont ("VeraSans", 8);
+  VeraFont12 = LoadFont ("VeraSans", 12);
   DefaultFontSize=14;
   //  LucidiaFont = LoadFont ("/fonts/LucidiaTypewriterRegular.ttf");
   
@@ -216,25 +219,25 @@ bool csWsTest::Initialize ()
 
   // Create a menu for all test dialogs we implement
   csMenu *menu = new csMenu (this, csmfs3D, 0);
-  csMenu *submenu = new csMenu (NULL);
+  csMenu *submenu = new csMenu (0);
   (void)new csMenuItem (menu, "~Standard dialogs", submenu);
     (void)new csMenuItem (submenu, "~File open", 66600);
     (void)new csMenuItem (submenu, "~Color choice", 66601);
 
-  submenu = new csMenu (NULL);
+  submenu = new csMenu (0);
   (void)new csMenuItem (menu, "~Windows", submenu);
     (void)new csMenuItem (submenu, "No top when selected", 66610);
     (void)new csMenuItem (submenu, "Complex dialog", 66611);
     (void)new csMenuItem (submenu, "Semi-transparent window", 66612);
 
-  submenu = new csMenu (NULL);
+  submenu = new csMenu (0);
   (void)new csMenuItem (menu, "~Tests", submenu);
     (void)new csMenuItem (submenu, "Notebook", 66620);
     (void)new csMenuItem (submenu, "Grid", 66621);
     (void)new csMenuItem (submenu, "Tree", 66622);
     (void)new csMenuItem (submenu, "Layouts", 66623);
 
-  submenu = new csMenu (NULL);
+  submenu = new csMenu (0);
   (void)new csMenuItem (menu, "~Color scheme", submenu);
     (void)new csMenuItem (submenu, "Green", 66630);
     (void)new csMenuItem (submenu, "Red", 66631);
@@ -269,7 +272,7 @@ void csWsTest::MiscDialog ()
   {
     csMenu *submenu;
 
-    submenu = new csMenu (NULL);
+    submenu = new csMenu (0);
     (void)new csMenuItem (menu, "~File", submenu);
       (void)new csMenuItem (submenu, "~Open\tCtrl+O", cscmdNothing);
       (void)new csMenuItem (submenu, "~Save\tCtrl+S", cscmdNothing);
@@ -277,7 +280,7 @@ void csWsTest::MiscDialog ()
       (void)new csMenuItem (submenu);
       (void)new csMenuItem (submenu, "~Quit\tCtrl+Q", cscmdQuit);
 
-    submenu = new csMenu (NULL);
+    submenu = new csMenu (0);
     (void)new csMenuItem (menu, "~Edit", submenu);
       (void)new csMenuItem (submenu, "~Undo\tAlt+BackSpace", cscmdNothing);
       (void)new csMenuItem (submenu);
@@ -288,23 +291,23 @@ void csWsTest::MiscDialog ()
       (void)new csMenuItem (submenu, "~Select all\tCtrl+/", cscmdNothing);
       (void)new csMenuItem (submenu, "~Deselect all\tCtrl+\\", cscmdNothing);
 
-    submenu = new csMenu (NULL);
+    submenu = new csMenu (0);
     (void)new csMenuItem (menu, "~Modify", submenu);
      (void)new csMenuItem (submenu, "~Scale", cscmdNothing);
       (void)new csMenuItem (submenu, "~Rotate", cscmdNothing);
       (void)new csMenuItem (submenu, "~Move", cscmdNothing);
 
-    submenu = new csMenu (NULL);
+    submenu = new csMenu (0);
     (void)new csMenuItem (menu, "~Test submenu", submenu);
       (void)new csMenuItem (submenu, "~One", cscmdNothing, CSMIS_DEFAULTVALUE | CSMIS_CHECKED);
       (void)new csMenuItem (submenu, "~Two", cscmdNothing);
       (void)new csMenuItem (submenu, "T~hree", cscmdNothing);
-      csMenu *subsubmenu = new csMenu (NULL);
+      csMenu *subsubmenu = new csMenu (0);
       (void)new csMenuItem (submenu, "~Four", subsubmenu);
         (void)new csMenuItem (subsubmenu, "Four / ~One", cscmdNothing);
         (void)new csMenuItem (subsubmenu, "Four / ~Two", cscmdNothing);
         (void)new csMenuItem (subsubmenu, "Four / T~hree", cscmdNothing);
-      subsubmenu = new csMenu (NULL);
+      subsubmenu = new csMenu (0);
       (void)new csMenuItem (submenu, "Fi~ve", subsubmenu, CSMIS_NEWCOLUMN);
         (void)new csMenuItem (subsubmenu, "Five / ~One", cscmdNothing);
         (void)new csMenuItem (subsubmenu, "Five / ~Two", cscmdNothing);
@@ -324,7 +327,7 @@ void csWsTest::MiscDialog ()
 
     (void)new csMenuItem (menu, CSMIS_NEWCOLUMN);
 
-    submenu = new csMenu (NULL);
+    submenu = new csMenu (0);
     (void)new csMenuItem (menu, "~Help", submenu);
       (void)new csMenuItem (submenu, "~Index", cscmdNothing);
       (void)new csMenuItem (submenu, "~General", cscmdNothing);
@@ -814,7 +817,8 @@ struct ModalData : public iBase
   uint code;
   csWindow* d;
   SCF_DECLARE_IBASE;
-  ModalData () { SCF_CONSTRUCT_IBASE (NULL); }
+  ModalData () { SCF_CONSTRUCT_IBASE (0); }
+  virtual ~ModalData () { SCF_DESTRUCT_IBASE (); }
 };
 
 SCF_IMPLEMENT_IBASE (ModalData)
@@ -845,19 +849,19 @@ bool csWsTest::HandleEvent (iEvent &Event)
 	  csComponent* d = GetTopModalComponent ();
 	  int rc = (int)Event.Command.Info;
 	  if (rc == 0 || rc == cscmdCancel) { delete d; return true; }
-	  iMessageBoxData* mbd = SCF_QUERY_INTERFACE (GetTopModalUserdata (),
-	  	iMessageBoxData);
-	  if (mbd) { mbd->DecRef (); delete d; return true; }
+	  csRef<iMessageBoxData> mbd (
+	  	SCF_QUERY_INTERFACE (GetTopModalUserdata (), iMessageBoxData));
+	  if (mbd) { delete d; return true; }
 
 	  ModalData* data = (ModalData*)GetTopModalUserdata ();
-	  CS_ASSERT (data != NULL);
+	  CS_ASSERT (data != 0);
 	    switch (data->code)
 	    {
 	      case 66600:
 	      {
                 char filename [CS_MAXPATHLEN + 1];
                 csQueryFileDialog ((csWindow*)d, filename, sizeof (filename));
-                csMessageBox (app, "Result", filename, NULL);
+                csMessageBox (app, "Result", filename, 0);
 	      }
 	      break;
               case 66601:
@@ -866,7 +870,7 @@ bool csWsTest::HandleEvent (iEvent &Event)
                 csQueryColorDialog ((csWindow*)d, color);
                 char buff [100];
                 sprintf (buff, "color value: %08X\n", color);
-                csMessageBox (app, "Result", buff, NULL);
+                csMessageBox (app, "Result", buff, 0);
 	      }
 	      break;
 	    }
@@ -935,7 +939,7 @@ bool csWsTest::HandleEvent (iEvent &Event)
           return true;
 	}
         case 66639:
-          csSetColorScheme (this, *(csColorScheme *)NULL);
+          csSetColorScheme (this, *(csColorScheme *)0);
           return true;
         case 66699:
           WindowList ();
@@ -1042,16 +1046,14 @@ int main (int argc, char* argv[])
     return -1;
   }
 
-  iCommandLineParser* cmdline = CS_QUERY_REGISTRY (object_reg,
-  	iCommandLineParser);
-  iConfigManager* config = CS_QUERY_REGISTRY (object_reg, iConfigManager);
+  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
+  	iCommandLineParser));
+  csRef<iConfigManager> config (CS_QUERY_REGISTRY (object_reg, iConfigManager));
 
   // Look for skin variant from config file
   DefaultSkin.Prefix = cmdline->GetOption ("skin");
   if (!DefaultSkin.Prefix)
-    DefaultSkin.Prefix = config->GetStr ("CSWS.Skin.Variant", NULL);
-  cmdline->DecRef ();
-  config->DecRef ();
+    DefaultSkin.Prefix = config->GetStr ("CSWS.Skin.Variant", 0);
 
   // Create our application object
   csWsTest *app = new csWsTest (object_reg, DefaultSkin);
@@ -1059,6 +1061,10 @@ int main (int argc, char* argv[])
   if (app->Initialize ())
     csDefaultRunLoop(object_reg);
 
+  // Explicit release before DestroyApplication().
+  cmdline = 0;
+  config = 0;
+
   delete app;
   csInitializer::DestroyApplication (object_reg);
   return 0;
