@@ -19,13 +19,15 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef CTHING_H
-#define CTHING_H
+#ifndef __CTHING_H__
+#define __CTHING_H__
 
 #include "ithing.h"
+#include "csutil/ref.h"
 
 class CCSWorld;
 class CCSSector;
+struct iDocumentNode;
 
 /**
   * This class will encapsulate a Thing to show up in a Crystal Space
@@ -41,10 +43,11 @@ public:
   virtual ~CCSThing();
 
   /// Write this thing as part of an sector
-  virtual bool Write(CIWorld* pIWorld, CISector* pISector);
+  virtual bool Write(csRef<iDocumentNode> node, CIWorld* pIWorld, CISector* pISector);
 
   /// Write only polygons without headings
-  virtual bool WriteAsPart(CIWorld* pIWorld, CISector* pISector);
+  virtual bool WriteAsPart(csRef<iDocumentNode> node, CIWorld* pIWorld, CISector* pISector,
+			   bool &Sky);
 };
 
-#endif
+#endif // __CTHING_H__
