@@ -19,13 +19,12 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef MAPSTD_H
-#define MAPSTD_H
+#ifndef __MAPSTD_H__
+#define __MAPSTD_H__
 
 #include <stdio.h>
 
-#include "csutil/csvector.h"
 #include "csutil/csstring.h"
 #include "dmath3d.h"
 
-#endif
+#endif // __MAPSTD_H__
