@@ -29,7 +29,7 @@
 CTextureFile* CPk3File::CreateTexture(const char* texturename)
 {
   char texfilename[256];
-  CTextureFile* pTexture = NULL;
+  CTextureFile* pTexture = 0;
 
   sprintf (texfilename, "textures/%s.tga", texturename);
   pTexture = ExtractTexture(texturename, texfilename);
@@ -39,7 +39,11 @@ CTextureFile* CPk3File::CreateTexture(const char* texturename)
   pTexture = ExtractTexture(texturename, texfilename);
   if (pTexture) return pTexture;
 
-  return NULL;
+  sprintf (texfilename, "textures/%s.png", texturename);
+  pTexture = ExtractTexture(texturename, texfilename);
+  if (pTexture) return pTexture;
+
+  return CZipArchive::CreateTexture (texturename);
 }
 
 
