@@ -20,6 +20,7 @@
 #define __SIMPMAP_H__
 
 #include <stdarg.h>
+#include "csutil/ref.h"
 
 struct iEngine;
 struct iLoader;
@@ -35,13 +36,13 @@ class Simple
 {
 private:
   iObjectRegistry* object_reg;
-  iEngine* engine;
-  iLoader* loader;
-  iGraphics3D* g3d;
-  iKeyboardDriver* kbd;
-  iVirtualClock* vc;
+  csRef<iEngine> engine;
+  csRef<iLoader> loader;
+  csRef<iGraphics3D> g3d;
+  csRef<iKeyboardDriver> kbd;
+  csRef<iVirtualClock> vc;
   iSector* room;
-  iView* view;
+  csRef<iView> view;
 
   static bool SimpleEventHandler (iEvent& ev);
   bool HandleEvent (iEvent& ev);
@@ -50,10 +51,10 @@ private:
   bool LoadMap ();
 
 public:
-  Simple ();
+  Simple (iObjectRegistry* object_reg);
   ~Simple ();
 
-  bool Initialize (int argc, const char* const argv[]);
+  bool Initialize ();
   void Start ();
 };
 
