@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1998,1999,2000 by Jorrit Tyberghein
+    Copyright (C) 1998-2002 by Jorrit Tyberghein
     Largely rewritten by Ivan Avramovic <ivan@avramovic.com>
 
     This library is free software; you can redistribute it and/or
@@ -20,12 +20,24 @@
 #ifndef __CS_BOX_H__
 #define __CS_BOX_H__
 
+/**\file 
+ * Bounding boxes for 2D and 3D space.
+ */
+/**
+ * \addtogroup geom_utils
+ * @{ */
+
+#include "csextern.h"
+
 #include "cstypes.h"	// for bool
+#include "csrect.h"
 #include "vector2.h"
 #include "vector3.h"
 #include "segment.h"
 
 class csPlane3;
+class csTransform;
+class csPoly2D;
 
 /**
  * The maximum value that a coordinate in the bounding box can use.
@@ -33,33 +45,51 @@ class csPlane3;
  */
 #define CS_BOUNDINGBOX_MAXVALUE 1000000000.
 
-/// For csBox2::GetCorner().
+/**\name Corner indices
+ * For csBox2::GetCorner().
+ * @{ */
+/// min X, min Y
 #define CS_BOX_CORNER_xy 0
+/// min X, max Y
 #define CS_BOX_CORNER_xY 1
+/// max X, min Y
 #define CS_BOX_CORNER_Xy 2
+/// max X, max Y
 #define CS_BOX_CORNER_XY 3
+/// center
+#define CS_BOX_CENTER2 4
+/** @} */
 
 /**
- * Indices of edges for cxBox2.
+ * \name Indices of edges for cxBox2.
  * Index e+1 is opposite edge of e (with e even).
- */
+ * @{ */
+/// from min X, min Y to max X, min Y
 #define CS_BOX_EDGE_xy_Xy 0
+/// from max X, min Y to min X, min Y
 #define CS_BOX_EDGE_Xy_xy 1
+/// from max X, min Y to max X, max Y
 #define CS_BOX_EDGE_Xy_XY 2
+/// from max X, max Y to max X, min Y
 #define CS_BOX_EDGE_XY_Xy 3
+/// from max X, max Y to min X, max Y
 #define CS_BOX_EDGE_XY_xY 4
+/// from min X, max Y to max X, max Y
 #define CS_BOX_EDGE_xY_XY 5
+/// from min X, max Y to min X, min Y
 #define CS_BOX_EDGE_xY_xy 6
+/// from min X, min Y to min X, max Y
 #define CS_BOX_EDGE_xy_xY 7
+/** @} */
 
 /**
  * A bounding box in 2D space.
  * In order to operate correctly, this bounding box assumes that all values
  * entered or compared against lie within the range
- * (-CS_BOUNDINGBOX_MAXVALUE, CS_BOUNDINGBOX_MAXVALUE).  It is not
+ * (-#CS_BOUNDINGBOX_MAXVALUE, #CS_BOUNDINGBOX_MAXVALUE).  It is not
  * recommended to use points outside of this range.
  */
-class csBox2
+class CS_CSGEOM_EXPORT csBox2
 {
 private:
   struct bEdge
@@ -99,7 +129,8 @@ public:
    * to 3. This contrasts with Min() and Max() because
    * those are only the min and max corners.
    * Corner 0 = xy, 1 = xY, 2 = Xy, 3 = XY.
-   * Use CS_BOX_CORNER_??? defines.
+   * Use #CS_BOX_CORNER_xy etc. defines.
+   * #CS_BOX_CENTER2 also works.
    */
   csVector2 GetCorner (int corner) const;
 
@@ -120,8 +151,8 @@ public:
   void SetSize (const csVector2& s);
 
   /**
-   * Given an edge index (CS_BOX_EDGE_???) return the two vertices
-   * (index CS_BOX_CORNER_???).
+   * Given an edge index (#CS_BOX_EDGE_xy_Xy etc.) return the two vertices
+   * (index #CS_BOX_CORNER_xy etc.).
    */
   void GetEdgeInfo (int edge, int& v1, int& v2) const
   {
@@ -131,7 +162,7 @@ public:
 
   /**
    * Return every edge (segment) of this bounding box
-   * from 0 to 7 (CS_BOX_EDGE_???).
+   * from 0 to 7 (#CS_BOX_EDGE_xy_Xy etc.).
    */
   csSegment2 GetEdge (int edge) const
   {
@@ -140,7 +171,7 @@ public:
 
   /**
    * Return every edge (segment) of this bounding box
-   * from 0 to 7 (CS_BOX_EDGE_???).
+   * from 0 to 7 (#CS_BOX_EDGE_xy_Xy etc.).
    */
   void GetEdge (int edge, csSegment2& e) const
   {
@@ -285,6 +316,56 @@ public:
     AddBoundingVertexSmart (v.x, v.y);
   }
 
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * Return true if the box was modified.
+   */
+  bool AddBoundingVertexTest (float x, float y)
+  {
+    bool rc = false;
+    if (x < minbox.x) { minbox.x = x; rc = true; }
+    if (x > maxbox.x) { maxbox.x = x; rc = true; }
+    if (y < minbox.y) { minbox.y = y; rc = true; }
+    if (y > maxbox.y) { maxbox.y = y; rc = true; }
+    return rc;
+  }
+
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * Return true if the box was modified.
+   */
+  bool AddBoundingVertexTest (const csVector2& v)
+  {
+    return AddBoundingVertexTest (v.x, v.y);
+  }
+
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * This version is a little more optimal. It assumes however
+   * that at least one point has been added to the bounding box.
+   * Return true if the box was modified.
+   */
+  bool AddBoundingVertexSmartTest (float x, float y)
+  {
+    bool rc = false;
+    if (x < minbox.x) { minbox.x = x; rc = true; }
+    else if (x > maxbox.x) { maxbox.x = x; rc = true; }
+    if (y < minbox.y) { minbox.y = y; rc = true; }
+    else if (y > maxbox.y) { maxbox.y = y; rc = true; }
+    return rc;
+  }
+
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * This version is a little more optimal. It assumes however
+   * that at least one point has been added to the bounding box.
+   * Return true if the box was modified.
+   */
+  bool AddBoundingVertexSmartTest (const csVector2& v)
+  {
+    return AddBoundingVertexSmartTest (v.x, v.y);
+  }
+
   //-----
   // Maintenance Note: The csBox2 constructors and Set() appear at this point
   // in the file, rather than earlier, in order to appease the OpenStep 4.2
@@ -311,6 +392,10 @@ public:
     minbox (x1, y1), maxbox (x2, y2)
   { if (Empty ()) StartBoundingBox (); }
 
+  /// Initialize this box from the given csRect.
+  csBox2 (const csRect& r) : minbox (r.xmin, r.ymin), maxbox (r.xmax, r.ymax)
+  { }
+  
   /// Sets the bounds of the box with the given values.
   void Set (const csVector2& bmin, const csVector2& bmax)
   {
@@ -325,12 +410,28 @@ public:
     else { minbox.x = x1;  minbox.y = y1;  maxbox.x = x2;  maxbox.y = y2; }
   }
 
+  /// Set Min component for 0 (x) or 1 (y).
+  void SetMin (int idx, float val)
+  {
+    if (idx == 1) minbox.y = val;
+    else minbox.x = val;
+  }
+
+  /// Set Max component for 0 (x) or 1 (y).
+  void SetMax (int idx, float val)
+  {
+    if (idx == 1) maxbox.y = val;
+    else maxbox.x = val;
+  }
+
   /// Compute the union of two bounding boxes.
   csBox2& operator+= (const csBox2& box);
   /// Compute the union of a point with this bounding box.
   csBox2& operator+= (const csVector2& point);
   /// Compute the intersection of two bounding boxes.
   csBox2& operator*= (const csBox2& box);
+  /// Test if the two boxes have an intersection.
+  bool TestIntersect (const csBox2& box) const;
 
   /// Compute the union of two bounding boxes.
   friend csBox2 operator+ (const csBox2& box1, const csBox2& box2);
@@ -352,84 +453,131 @@ public:
 };
 
 /**
- * Indices of corner vertices for csBox3.
+ * \name Indices of corner vertices for csBox3.
  * Used by csBox3::GetCorner().
- */
+ * @{ */
+/// min X, min Y, min Z
 #define CS_BOX_CORNER_xyz 0
+/// min X, min Y, max Z
 #define CS_BOX_CORNER_xyZ 1
+/// min X, max Y, min Z
 #define CS_BOX_CORNER_xYz 2
+/// min X, max Y, max Z
 #define CS_BOX_CORNER_xYZ 3
+/// min X, min Y, min Z
 #define CS_BOX_CORNER_Xyz 4
+/// max X, min Y, max Z
 #define CS_BOX_CORNER_XyZ 5
+/// max X, max Y, min Z
 #define CS_BOX_CORNER_XYz 6
+/// max X, max Y, max Z
 #define CS_BOX_CORNER_XYZ 7
+/// center
+#define CS_BOX_CENTER3 8
+/** @} */
 
 /**
- * Indices of faces for csBox3.
+ * \name Indices of faces for csBox3.
  * Used by csBox3::GetSide().
- */
+ * @{ */
+/// min X
 #define CS_BOX_SIDE_x 0
+/// max X
 #define CS_BOX_SIDE_X 1
+/// min Y
 #define CS_BOX_SIDE_y 2
+/// max Y
 #define CS_BOX_SIDE_Y 3
+/// min Z
 #define CS_BOX_SIDE_z 4
+/// max Z
 #define CS_BOX_SIDE_Z 5
+/// inside
 #define CS_BOX_INSIDE 6
+/** @} */
 
 /**
- * Indices of edges for cxBox3.
+ * \name Indices of edges for cxBox3.
  * Index e+1 is opposite edge of e (with e even).
- */
+ * @{ */
+/// from max X, min Y, min Z to min X, min Y, min Z
 #define CS_BOX_EDGE_Xyz_xyz 0
+/// from min X, min Y, min Z to max X, min Y, min Z
 #define CS_BOX_EDGE_xyz_Xyz 1
+/// from min X, min Y, min Z to min X, max Y, min Z
 #define CS_BOX_EDGE_xyz_xYz 2
+/// from min X, max Y, min Z to min X, min Y, min Z
 #define CS_BOX_EDGE_xYz_xyz 3
+/// from min X, max Y, min Z to max X, max Y, min Z
 #define CS_BOX_EDGE_xYz_XYz 4
+/// from max X, max Y, min Z to min X, max Y, min Z
 #define CS_BOX_EDGE_XYz_xYz 5
+/// from max X, max Y, min Z to max X, min Y, min Z
 #define CS_BOX_EDGE_XYz_Xyz 6
+/// from max X, min Y, min Z to max X, max Y, min Z
 #define CS_BOX_EDGE_Xyz_XYz 7
+/// from max X, min Y, min Z to max X, min Y, max Z
 #define CS_BOX_EDGE_Xyz_XyZ 8
+/// from max X, min Y, max Z to max X, min Y, min Z
 #define CS_BOX_EDGE_XyZ_Xyz 9
+/// from max X, min Y, max Z to max X, max Y, max Z
 #define CS_BOX_EDGE_XyZ_XYZ 10
+/// from max X, max Y, max Z to max X, min Y, max Z
 #define CS_BOX_EDGE_XYZ_XyZ 11
+/// from max X, max Y, max Z to max X, max Y, min Z
 #define CS_BOX_EDGE_XYZ_XYz 12
+/// from max X, max Y, min Z to max X, max Y, max Z
 #define CS_BOX_EDGE_XYz_XYZ 13
+/// from max X, max Y, max Z to min X, max Y, max Z
 #define CS_BOX_EDGE_XYZ_xYZ 14
+/// from min X, max Y, max Z to max X, max Y, max Z
 #define CS_BOX_EDGE_xYZ_XYZ 15
+/// from min X, max Y, max Z to min X, max Y, min Z
 #define CS_BOX_EDGE_xYZ_xYz 16
+/// from min X, max Y, min Z to min X, max Y, max Z
 #define CS_BOX_EDGE_xYz_xYZ 17
+/// from min X, max Y, max Z to min X, min Y, max Z
 #define CS_BOX_EDGE_xYZ_xyZ 18
+/// from min X, min Y, max Z to min X, max Y, max Z
 #define CS_BOX_EDGE_xyZ_xYZ 19
+/// from min X, min Y, max Z to min X, min Y, min Z
 #define CS_BOX_EDGE_xyZ_xyz 20
+/// from min X, min Y, min Z to min X, min Y, max Z
 #define CS_BOX_EDGE_xyz_xyZ 21
+/// from min X, min Y, max Z to max X, min Y, max Z
 #define CS_BOX_EDGE_xyZ_XyZ 22
+/// from max X, min Y, max Z to min X, min Y, max Z
 #define CS_BOX_EDGE_XyZ_xyZ 23
+/** @} */
 
 /**
  * A bounding box in 3D space.
  * In order to operate correctly, this bounding box assumes that all values
  * entered or compared against lie within the range
- * (-CS_BOUNDINGBOX_MAXVALUE, CS_BOUNDINGBOX_MAXVALUE).  It is not
+ * (-#CS_BOUNDINGBOX_MAXVALUE, #CS_BOUNDINGBOX_MAXVALUE).  It is not
  * recommended to use points outside of this range.
  */
-class csBox3
+class CS_CSGEOM_EXPORT csBox3
 {
 protected:
   /// The top-left of this bounding box.
   csVector3 minbox;
   /// The bottom-right.
   csVector3 maxbox;
-
+  /// \internal A csBox3 edge.
   struct bEdge
   {
     uint8 v1, v2; // Indices of vertex in bounding box (CS_BOX_CORNER_...)
     uint8 fl, fr; // Indices of left/right faces sharing edge (CS_BOX_SIDE_...)
   };
-  typedef uint8 bFace[4];	// Indices of four clock-wise edges (0..23)
-  // Index by edge number. Edge e and e+1 with e even are opposite edges.
-  // (CS_BOX_EDGE_...)
+  /// Indices of four clock-wise edges (0..23)
+  typedef uint8 bFace[4];	
+  /**
+   * Index by edge number. Edge e and e+1 with e even are opposite edges.
+   * (CS_BOX_EDGE_...) 
+   */
   static bEdge edges[24];
-  // Index by CS_BOX_SIDE_? number.
+  /// Index by CS_BOX_SIDE_? number.
   static bFace faces[6];
 public:
   /// Get the minimum X value of the box
@@ -461,13 +609,14 @@ public:
    * those are only the min and max corners.
    * Corner 0 = xyz, 1 = xyZ, 2 = xYz, 3 = xYZ,
    *        4 = Xyz, 5 = XyZ, 6 = XYz, 7 = XYZ.
-   * Use CS_BOX_CORNER_??? defines.
+   * Use #CS_BOX_CORNER_xyz etc. defines.
+   * #CS_BOX_CENTER3 also works.
    */
   csVector3 GetCorner (int corner) const;
 
   /**
-   * Given an edge index (CS_BOX_EDGE_???) return the two vertices
-   * (index CS_BOX_CORNER_???) and left/right faces (CS_BOX_SIDE_???).
+   * Given an edge index (#CS_BOX_EDGE_Xyz_xyz etc.) return the two vertices
+   * (index #CS_BOX_CORNER_xyz etc.) and left/right faces (#CS_BOX_SIDE_x etc.).
    */
   void GetEdgeInfo (int edge, int& v1, int& v2, int& fleft, int& fright) const
   {
@@ -478,8 +627,8 @@ public:
   }
 
   /**
-   * Given a face index (CS_BOX_SIDE_???) return the four edges oriented
-   * clockwise around this face (CS_BOX_EDGE_???).
+   * Given a face index (#CS_BOX_SIDE_x etc.) return the four edges oriented
+   * clockwise around this face (#CS_BOX_EDGE_Xyz_xyz etc.).
    */
   uint8* GetFaceEdges (int face) const
   {
@@ -504,13 +653,13 @@ public:
 
   /**
    * Get a side of this box as a 2D box.
-   * Use CS_BOX_SIDE_??? defines.
+   * Use #CS_BOX_SIDE_x etc. defines.
    */
   csBox2 GetSide (int side) const;
 
   /**
    * Fill the array (which should be three long at least)
-   * with all visible sides (CS_BOX_SIDE_??? defines) as seen
+   * with all visible sides (#CS_BOX_SIDE_x etc. defines) as seen
    * from the given point.
    * Returns the number of visible sides.
    */
@@ -527,7 +676,7 @@ public:
 
   /**
    * Return every edge (segment) of this bounding box
-   * from 0 to 23 (use one of the CS_BOX_EDGE_??? indices).
+   * from 0 to 23 (use one of the #CS_BOX_EDGE_Xyz_xyz etc. indices).
    * The returned edge is undefined for any other index.
    */
   csSegment3 GetEdge (int edge) const
@@ -537,7 +686,7 @@ public:
 
   /**
    * Return every edge (segment) of this bounding box
-   * from 0 to 23 (use one of the CS_BOX_EDGE_??? indices).
+   * from 0 to 23 (use one of the #CS_BOX_EDGE_Xyz_xyz etc. indices).
    * The returned edge is undefined for any other index.
    */
   void GetEdge (int edge, csSegment3& e) const
@@ -640,6 +789,60 @@ public:
     AddBoundingVertexSmart (v.x, v.y, v.z);
   }
 
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * Returns true if box was modified.
+   */
+  bool AddBoundingVertexTest (float x, float y, float z)
+  {
+    bool rc = false;
+    if (x < minbox.x) { minbox.x = x; rc = true; }
+    if (x > maxbox.x) { maxbox.x = x; rc = true; }
+    if (y < minbox.y) { minbox.y = y; rc = true; }
+    if (y > maxbox.y) { maxbox.y = y; rc = true; }
+    if (z < minbox.z) { minbox.z = z; rc = true; }
+    if (z > maxbox.z) { maxbox.z = z; rc = true; }
+    return rc;
+  }
+
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * Returns true if box was modified.
+   */
+  bool AddBoundingVertexTest (const csVector3& v)
+  {
+    return AddBoundingVertexTest (v.x, v.y, v.z);
+  }
+
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * This version is a little more optimal. It assumes however
+   * that at least one point has been added to the bounding box.
+   * Returns true if box was modified.
+   */
+  bool AddBoundingVertexSmartTest (float x, float y, float z)
+  {
+    bool rc = false;
+    if (x < minbox.x) { minbox.x = x; rc = true; }
+    else if (x > maxbox.x) { maxbox.x = x; rc = true; }
+    if (y < minbox.y) { minbox.y = y; rc = true; }
+    else if (y > maxbox.y) { maxbox.y = y; rc = true; }
+    if (z < minbox.z) { minbox.z = z; rc = true; }
+    else if (z > maxbox.z) { maxbox.z = z; rc = true; }
+    return rc;
+  }
+
+  /**
+   * Add a new vertex and recalculate the bounding box.
+   * This version is a little more optimal. It assumes however
+   * that at least one point has been added to the bounding box.
+   * Returns true if box was modified.
+   */
+  bool AddBoundingVertexSmartTest (const csVector3& v)
+  {
+    return AddBoundingVertexSmartTest (v.x, v.y, v.z);
+  }
+
   //-----
   // Maintenance Note: The csBox3 constructors and Set() appear at this point
   // in the file, rather than earlier, in order to appease the OpenStep 4.2
@@ -694,6 +897,22 @@ public:
     }
   }
 
+  /// Set Min component for 0 (x), 1 (y), or 2 (z).
+  void SetMin (int idx, float val)
+  {
+    if (idx == 1) minbox.y = val;
+    else if (idx == 0) minbox.x = val;
+    else minbox.z = val;
+  }
+
+  /// Set Max component for 0 (x), 1 (y), or 2 (z).
+  void SetMax (int idx, float val)
+  {
+    if (idx == 1) maxbox.y = val;
+    else if (idx == 0) maxbox.x = val;
+    else maxbox.z = val;
+  }
+
   /**
    * Test if this box is adjacent to the other on the X side.
    */
@@ -711,12 +930,20 @@ public:
 
   /**
    * Test if this box is adjacent to the other one.
-   * Return -1 if not adjacent or else any of the CS_BOX_SIDE_???
+   * Return -1 if not adjacent or else any of the #CS_BOX_SIDE_x etc.
    * flags to indicate the side of this box that the other
    * box is adjacent with.
    */
   int Adjacent (const csBox3& other) const;
 
+  /**
+   * Assume that 3D space is divided into 27 areas. One is inside
+   * the box. The other 26 are rectangular segments around the box.
+   * This function will calculate the right segment for a given point
+   * and return that.
+   */
+  int CalculatePointSegment (const csVector3& pos) const;
+
   /**
    * Get a convex outline (not a polygon unless projected to 2D)
    * for for this box as seen from the given position.
@@ -752,30 +979,84 @@ public:
    */
   float SquaredOriginMaxDist () const;
 
+  /**
+   * Project this box to a 2D bounding box given the view point
+   * transformation and also the field-of-view and shift values (for
+   * perspective projection). The transform should transform from world
+   * to camera space (using Other2This). The minimum and maximum z
+   * are also calculated. If the bounding box is behind the camera
+   * then the 'sbox' will not be calculated (min_z and max_z are
+   * still calculated) and the function will return false.
+   * If the camera is inside the transformed box then this function will
+   * return true and a conservative screen space bounding box is returned.
+   */
+  bool ProjectBox (const csTransform& trans, float fov, float sx, float sy,
+  	csBox2& sbox, float& min_z, float& max_z) const;
+
+  /**
+   * Project this box to the 2D outline given the view point
+   * transformation and also the field-of-view and shift values (for
+   * perspective correction). The minimum and maximum z are also
+   * calculated. If the box is fully behind the camera
+   * then false is returned and this function will not do anything.
+   * If the box is partially behind the camera you will get an outline
+   * that is conservatively correct (i.e. it will overestimate the box).
+   */
+  bool ProjectOutline (const csTransform& trans, float fov, float sx, float sy,
+  	csPoly2D& poly, float& min_z, float& max_z) const;
+
+  /**
+   * Project this box to the 2D outline given the view point
+   * transformation and also the field-of-view and shift values (for
+   * perspective correction). The minimum and maximum z are also
+   * calculated. If the box is fully behind the camera
+   * then false is returned and this function will not do anything.
+   * If the box is partially behind the camera you will get an outline
+   * that is conservatively correct (i.e. it will overestimate the box).
+   * In addition to the outline this function also returns the projected
+   * screen-space box. So it is a combination of ProjectBox() and
+   * ProjectOutline().
+   */
+  bool ProjectBoxAndOutline (const csTransform& trans, float fov,
+  	float sx, float sy, csBox2& sbox, csPoly2D& poly,
+	float& min_z, float& max_z) const;
+
   /// Compute the union of two bounding boxes.
   csBox3& operator+= (const csBox3& box);
   /// Compute the union of a point with this bounding box.
   csBox3& operator+= (const csVector3& point);
   /// Compute the intersection of two bounding boxes.
   csBox3& operator*= (const csBox3& box);
+  /// Test if the two boxes have an intersection.
+  bool TestIntersect (const csBox3& box) const;
 
   /// Compute the union of two bounding boxes.
-  friend csBox3 operator+ (const csBox3& box1, const csBox3& box2);
+  friend CS_CSGEOM_EXPORT csBox3 operator+ (const csBox3& box1, 
+    const csBox3& box2);
   /// Compute the union of a bounding box and a point.
-  friend csBox3 operator+ (const csBox3& box, const csVector3& point);
+  friend CS_CSGEOM_EXPORT csBox3 operator+ (const csBox3& box, 
+    const csVector3& point);
   /// Compute the intersection of two bounding boxes.
-  friend csBox3 operator* (const csBox3& box1, const csBox3& box2);
+  friend CS_CSGEOM_EXPORT csBox3 operator* (const csBox3& box1, 
+    const csBox3& box2);
 
   /// Tests if two bounding boxes are equal.
-  friend bool operator== (const csBox3& box1, const csBox3& box2);
+  friend CS_CSGEOM_EXPORT bool operator== (const csBox3& box1, 
+    const csBox3& box2);
   /// Tests if two bounding boxes are unequal.
-  friend bool operator!= (const csBox3& box1, const csBox3& box2);
+  friend CS_CSGEOM_EXPORT bool operator!= (const csBox3& box1, 
+    const csBox3& box2);
   /// Tests if box1 is a subset of box2.
-  friend bool operator< (const csBox3& box1, const csBox3& box2);
+  friend CS_CSGEOM_EXPORT bool operator< (const csBox3& box1, 
+    const csBox3& box2);
   /// Tests if box1 is a superset of box2.
-  friend bool operator> (const csBox3& box1, const csBox3& box2);
+  friend CS_CSGEOM_EXPORT bool operator> (const csBox3& box1, 
+    const csBox3& box2);
   /// Tests if a point is contained in a box.
-  friend bool operator< (const csVector3& point, const csBox3& box);
+  friend CS_CSGEOM_EXPORT bool operator< (const csVector3& point, 
+    const csBox3& box);
 };
 
+/** @} */
+
 #endif // __CS_BOX_H__
