@@ -16,8 +16,17 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __CSPOINT_H__
-#define __CSPOINT_H__
+#ifndef __CS_CSPOINT_H__
+#define __CS_CSPOINT_H__
+
+/**\file 
+ * 2D point object.
+ */
+/**
+ * \addtogroup geom_utils
+ * @{ */
+
+#include "csextern.h"
 
 /**
  * A 2D point object
@@ -44,5 +53,7 @@ struct csPoint
   { x = v.x; y = v.y; }
 };
 
-#endif
+/** @} */
+
+#endif // __CS_CSPOINT_H__
 
