@@ -21,8 +21,16 @@
 #ifndef __CS_MATH3D_D_H__
 #define __CS_MATH3D_D_H__
 
+#include "csextern.h"
+
 #include "cstypes.h"
 
+/**\file 
+ */
+/**
+ * \addtogroup geom_utils
+ * @{ */
+
 #ifndef ABS
 #define ABS(x) ((x)<0?-(x):(x))
 #endif
@@ -39,7 +47,7 @@ inline double dSqr (double d)
 /**
  * A 3D vector.
  */
-class csDVector3
+class CS_CSGEOM_EXPORT csDVector3
 {
 public:
   /// The X component of the vector
@@ -187,6 +195,9 @@ public:
   /// Returns the norm of this vector.
   double Norm () const;
 
+  /// Returns the norm of this vector.
+  double SquaredNorm () const;
+
   /**
    * Returns the unit vector in the direction of this vector.
    * Attempting to normalize a zero-vector will result in a divide by
@@ -208,7 +219,7 @@ public:
 /**
  * A 3x3 matrix.
  */
-class csDMatrix3
+class CS_CSGEOM_EXPORT csDMatrix3
 {
 public:
   double m11, m12, m13;
@@ -343,7 +354,7 @@ public:
  * The plane is given by the equation AAx + BBy + CCz + DD = 0,
  * Where (AA,BB,CC) is given by the vector 'norm'.
  */
-class csDPlane
+class CS_CSGEOM_EXPORT csDPlane
 {
 public:
   /// The normal vector (or the (A,B,C) components).
@@ -421,7 +432,7 @@ public:
  * Various assorted 3D mathematical functions.
  * This is a static class and contains only static member functions.
  */
-class csDMath3
+class CS_CSGEOM_EXPORT csDMath3
 {
 public:
   /**
@@ -485,17 +496,16 @@ public:
   }
 
   /**
-   * Compute twice the signed area of triangle composed by three points.
+   * Compute twice the area of triangle composed by three points.
    * This function returns 2 x the area of the triangle formed by the points
    * a, b, and c.
    */
-  inline static double Area3 (const csDVector3 &a, const csDVector3 &b,
+  inline static double DoubleArea3 (const csDVector3 &a, const csDVector3 &b,
                              const csDVector3 &c)
   {
     csDVector3 v1 = b - a;
     csDVector3 v2 = c - a;
-    return ((v1.y * v2.z + v1.z * v2.x + v1.x * v2.y) -
-            (v1.y * v2.x + v1.x * v2.z + v1.z * v2.y));
+    return (v1 % v2).Norm ();
   }
 
   /**
@@ -555,7 +565,7 @@ public:
  * Some functions to perform squared distance calculations.
  * This is a static class and contains only static member functions.
  */
-class csDSquaredDist
+class CS_CSGEOM_EXPORT csDSquaredDist
 {
 public:
   /// Returns the squared distance between two points.
@@ -585,7 +595,7 @@ public:
  * line segments.  This is a static class and contains only static member
  * functions.
  */
-class csDIntersect3
+class CS_CSGEOM_EXPORT csDIntersect3
 {
 public:
   /**
@@ -668,4 +678,6 @@ public:
     double B, const csDVector3& u, const csDVector3& v, csDVector3& isect);
 };
 
+/** @} */
+
 #endif // __CS_MATH3D_D_H__
