@@ -17,14 +17,33 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __XORPAT_H__
-#define __XORPAT_H__
+/**\file
+ */
+ 
+/**\addtogroup gfx
+ * @{ 
+ */
+
+#ifndef __CS_XORPAT_H__
+#define __CS_XORPAT_H__
+
+#include "csextern.h"
 
 /**
- * Create an iImage with a nice XOR pattern.
- * recdepth - "recursion depth", clamped to range 1-8
+ * Create an iImage with a nice XOR pattern with 2^\p recdepth
+ * shades of from white to black.
+ * \param width Width of the image to create
+ * \param height Height of the image to create
+ * \param recdepth "recursion depth", clamped to range 1-8.
+ * \param red Maximum value of red component
+ * \param green Maximum value of green component
+ * \param blue Maximum value of blue component
  */
-extern iImage* csCreateXORPatternImage(int width, int height, int recdepth);
+extern CS_CSGFX_EXPORT csPtr<iImage> csCreateXORPatternImage(int width, 
+  int height, int recdepth, float red = 1.0f, float green = 1.0f,
+  float blue = 1.0f);
 
-#endif
+/** @} */
+
+#endif // __CS_XORPAT_H__
 
