@@ -26,20 +26,14 @@
 #include "csdef.h"
 #undef CSDEF_FRIEND
 
-/*
+/** \file
+
     This include file should be included from every source file.
     Just before #include directive it can contain several #define's
     that specify what the source file requires.
 
     The following variables can be defined:
 
-    #define CS_SYSDEF_PROVIDE_CASE
-      Define the UPPERCASE() and LOWERCASE() macros.
-
-    #define CS_SYSDEF_PROVIDE_PATH
-      Include definition of PATH_SEPARATOR character, MAXPATHLEN and
-      APPEND_SLASH macros.
-
     #define CS_SYSDEF_PROVIDE_MKDIR
       Include definition for MKDIR()
 
@@ -59,25 +53,9 @@
     #define CS_SYSDEF_PROVIDE_ACCESS
       Include definitions required for access()
 
-    #define CS_SYSDEF_PROVIDE_ALLOCA
-      Include definition for alloca() and ALLOC_STACK_ARRAY()
-
     #define CS_SYSDEF_PROVIDE_GETOPT
       For getopt() and GNU getopt_long()
 
-    #define CS_SYSDEF_PROVIDE_SOCKETS
-      For TCP/IP sockets definitions.  Specifically, should define the
-      following macros, constants, typedefs, and prototypes:
-	inet_addr(), gethostbyname(), ntohl(), etc.
-	socket(), listen(), bind(), etc. -- the standard socket functions
-	csNetworkSocket -- typedef or macro for socket descriptor type
-	struct sockaddr -- standard socket address type (and cousins)
-	socklen_t -- typedef or macro
-	CS_NET_SOCKET_INVALID -- value representing invalid socket
-	CS_CLOSESOCKET -- name of function to close a socket
-	CS_IOCTLSOCKET -- name of "ioctl" function for sockets
-	CS_GETSOCKETERROR -- name of function or variable for socket error code
-	
     #define CS_SYSDEF_PROVIDE_SELECT
       Includes definitions required for select(), FD_* macros, and
       struct timeval.
@@ -89,62 +67,38 @@
     avoid including util/gnu/getopt.h at the bottom of this file.
 */
 
-#if defined (CS_SYSDEF_PROVIDE_DIR) && !defined (CS_SYSDEF_PROVIDE_PATH)
-#  define CS_SYSDEF_PROVIDE_PATH
-#endif
-
 /*
  * Pull in platform-specific overrides of the requested functionality.
  */
-#include "cssys/csosdefs.h"
+#include "csutil/csosdefs.h"
+
+#include "csextern.h"
 
 /*
  * Default definitions for requested functionality.  Platform-specific
  * configuration files may override these.
  */
 
-#ifdef CS_SYSDEF_PROVIDE_CASE
-// Convert a character to upper case
-#  ifndef UPPERCASE
-#    define UPPERCASE(c) ((c >= 'a' && c <= 'z') ? c - ('a' - 'A') : c)
-#  endif
-// Convert a character to lower case
-#  ifndef LOWERCASE
-#    define LOWERCASE(c) ((c >= 'A' && c <= 'Z') ? c + ('a' - 'A') : c)
-#  endif
-#endif // CS_SYSDEF_PROVIDE_CASE
-
-#ifdef CS_SYSDEF_PROVIDE_PATH
-// Path separator character
-#  ifndef PATH_SEPARATOR
-#    if defined(__CYGWIN32__)
-#      define PATH_SEPARATOR '/'
-#    elif defined(OS_OS2) || defined(OS_DOS) || defined(OS_WIN32)
-#      define PATH_SEPARATOR '\\'
-#    else
-#      define PATH_SEPARATOR '/'
-#    endif
-#  endif
-// Maximal path length
-#  ifndef CS_MAXPATHLEN
-#    ifdef _MAX_FNAME
-#      define CS_MAXPATHLEN _MAX_FNAME
-#    else
-#      define CS_MAXPATHLEN 256
-#    endif
-#  endif
-#  define APPEND_SLASH(str,len)			\
-     if ((len)					\
-      && (str[len - 1] != '/')			\
-      && (str[len - 1] != PATH_SEPARATOR))	\
-     {						\
-       str[len++] = PATH_SEPARATOR;		\
-       str[len] = 0;				\
-     } /* endif */
-#endif // CS_SYSDEF_PROVIDE_PATH
+/**\def CS_ALLOC_STACK_ARRAY(type, var, size)
+ * Dynamic stack memory allocation.
+ * \param type Type of the array elements.
+ * \param var Name of the array to be allocated.
+ * \param size Number of elements to be allocated.
+ */
+#ifdef COMP_GCC
+// In GCC we are able to declare stack vars of dynamic size directly
+#  define CS_ALLOC_STACK_ARRAY(type, var, size) \
+      type var [size]
+#else
+#  include <malloc.h>
+#  define CS_ALLOC_STACK_ARRAY(type, var, size) \
+      type *var = (type *)alloca ((size) * sizeof (type))
+#endif
 
+/**\def TEMP_DIR
+ * Directory for temporary files
+ */
 #ifdef CS_SYSDEF_PROVIDE_TEMP
-// Directory for temporary files
 #  ifndef TEMP_DIR
 #    if defined(OS_UNIX)
 #      define TEMP_DIR "/tmp/"
@@ -152,19 +106,22 @@
 #      define TEMP_DIR ""
 #    endif
 #  endif
-// Name for temporary file
 #  ifndef TEMP_FILE
 #    if defined(OS_UNIX)
 #      include <unistd.h>
+/// Name for temporary file
 #      define TEMP_FILE "cs%lud.tmp", (unsigned long)getpid()
 #    else
+/// Name for temporary file
 #      define TEMP_FILE "$cs$.tmp"
 #    endif
 #  endif
 #endif // CS_SYSDEF_PROVIDE_TEMP
 
+/**\def MKDIR(path)
+ * How to make a directory (not entire path, only the last on the path)
+ */
 #ifdef CS_SYSDEF_PROVIDE_MKDIR
-// How to make a directory (not entire path, only the last on the path)
 #  ifndef MKDIR
 #    if defined(OS_WIN32) || (defined(OS_DOS) && !defined(COMP_GCC))
 #      define MKDIR(path) _mkdir (path)
@@ -182,20 +139,18 @@
 
 #ifdef CS_SYSDEF_PROVIDE_DIR
 // For systems without opendir()
-// COMP_GCC has opendir, readdir 
-# if !defined(COMP_GCC)
-#  if defined(__NEED_OPENDIR_PROTOTYPE)
+// Although COMP_GCC has opendir, readdir, CS' versions are preferred.
+# if defined(__NEED_OPENDIR_PROTOTYPE)
      struct DIR;
      struct dirent;
-     extern "C" DIR *opendir (const char *name);
-     extern "C" dirent *readdir (DIR *dirp);
-     extern "C" int closedir (DIR *dirp);
+     extern "C" CS_CSUTIL_EXPORT DIR *opendir (const char *name);
+     extern "C" CS_CSUTIL_EXPORT dirent *readdir (DIR *dirp);
+     extern "C" CS_CSUTIL_EXPORT int closedir (DIR *dirp);
      //extern "C" void seekdir (DIR *dirp, long off);
      //extern "C" long telldir (DIR *dirp);
      //extern "C" void rewinddir (DIR *dirp);
-#  endif
 # endif
-// Generic ISDIR needed for COMP_GCC
+// Generic ISDIR where needed
 #  ifdef __NEED_GENERIC_ISDIR
 #    if defined (OS_WIN32) || defined (OS_DOS)
 #      include <io.h>
@@ -204,16 +159,19 @@
 #    if !defined(OS_WIN32)
 #      include <dirent.h>
 #    endif
-#    if defined(__CYGWIN32__)
-#      include <sys/dirent.h>
-#    endif
 #    include <sys/stat.h>
      static inline bool isdir (const char *path, struct dirent *de)
      {
-       char fullname [CS_MAXPATHLEN];
        int pathlen = strlen (path);
+       CS_ALLOC_STACK_ARRAY(char, fullname, pathlen + 2 + strlen (de->d_name));
        memcpy (fullname, path, pathlen + 1);
-       APPEND_SLASH (fullname, pathlen);
+       
+       if ((pathlen) && (fullname[pathlen-1] != PATH_SEPARATOR))
+       {
+	 fullname[pathlen++] = PATH_SEPARATOR;
+	 fullname[pathlen] = 0;
+       }
+              
        strcat (&fullname [pathlen], de->d_name);
        struct stat st;
        stat (fullname, &st);
@@ -228,28 +186,6 @@
 #  endif
 #endif
 
-#ifdef CS_SYSDEF_PROVIDE_ALLOCA
-// Prototypes for dynamic stack memory allocation
-#  if defined (COMP_VC) || defined(COMP_BC) || \
-     (defined(COMP_GCC) && defined(OS_WIN32))
-#    include <malloc.h>
-#  elif defined(COMP_GCC) && defined(OS_DOS)
-#    include <stdlib.h>
-#  elif defined(OS_BSD)
-#    include <stdlib.h>
-#  else
-#    include <alloca.h>
-#  endif
-#  if defined (COMP_GCC)
-    // In GCC we are able to declare stack vars of dynamic size directly
-#    define ALLOC_STACK_ARRAY(var,type,size) \
-       type var [size]
-#  else
-#    define ALLOC_STACK_ARRAY(var,type,size) \
-       type *var = (type *)alloca (size * sizeof (type))
-#  endif
-#endif
-
 #ifdef CS_SYSDEF_PROVIDE_ACCESS
 #  if !defined(COMP_VC) && !defined(COMP_BC)
 #    include <unistd.h>
@@ -269,39 +205,14 @@
 #  ifndef __STDC__
 #    define __STDC__ 1
 #  endif
-#  include "cssys/getopt.h"
+#  include "csutil/getopt.h"
 #endif
 
 #ifdef CS_SYSDEF_PROVIDE_SOCKETS
-#  include <sys/types.h>
-#  include <sys/socket.h>
-#  if defined (OS_UNIX)
-#    include <unistd.h>
-#    define BSD_COMP 1
-#    include <sys/ioctl.h>
-#    if !defined (OS_SOLARIS) && !defined (OS_BE)
-#      include <arpa/inet.h>
-#      include <sys/time.h>
-#    endif
-#  endif
-#  include <netinet/in.h>
-#  include <netdb.h>
-#  if defined (CS_USE_FAKE_SOCKLEN_TYPE)
-     typedef int socklen_t;
-#  endif
-#  if !defined (CS_IOCTLSOCKET)
-#    define CS_IOCTLSOCKET ioctl
-#  endif
-#  if !defined (CS_CLOSESOCKET)
-#    define CS_CLOSESOCKET close
-#  endif
-#  if !defined (CS_GETSOCKETERROR)
-#    define CS_GETSOCKETERROR errno
-#  endif
-   typedef unsigned int csNetworkSocket;
-#  if !defined (CS_NET_SOCKET_INVALID)
-#    define CS_NET_SOCKET_INVALID ((csNetworkSocket)~0)
-#  endif
+# ifdef __GNUC__
+#  warning CS_SYSDEF_PROVIDE_SOCKETS is deprecated, include cssys/sockets.h instead
+# endif
+#include "csutil/sockets.h"
 #endif
 
 #ifdef CS_SYSDEF_PROVIDE_SELECT
@@ -324,7 +235,7 @@
  * </pre>
  */
 #define CS_HEADER_GLOBAL(X,Y) CS_HEADER_GLOBAL_COMPOSE(X,Y)
-#define CS_HEADER_GLOBAL_COMPOSE(X,Y) < ## X ## / ## Y ## >
+#define CS_HEADER_GLOBAL_COMPOSE(X,Y) <X/Y>
 
 /**
  * The CS_HEADER_LOCAL() macro composes a pathname from two components and
@@ -339,12 +250,12 @@
  * </pre>
  */
 #define CS_HEADER_LOCAL(X,Y) CS_HEADER_LOCAL_COMPOSE1(X,Y)
-#define CS_HEADER_LOCAL_COMPOSE1(X,Y) CS_HEADER_LOCAL_COMPOSE2(X ## / ## Y)
+#define CS_HEADER_LOCAL_COMPOSE1(X,Y) CS_HEADER_LOCAL_COMPOSE2(X/Y)
 #define CS_HEADER_LOCAL_COMPOSE2(X) #X
 
 
-/*
- * A macro to export a function from a shared library.
+/**\def CS_EXPORTED_FUNCTION
+ * \internal A macro to export a function from a shared library.
  * Some platforms may need to override this.  For instance, Windows requires
  * extra `__declspec' goop when exporting a function from a plug-in module.
  */
@@ -352,8 +263,8 @@
 #  define CS_EXPORTED_FUNCTION extern "C"
 #endif
 
-/*
- * A macro used to build exported function names.
+/**\def CS_EXPORTED_NAME(Prefix, Suffix)
+ * \internal A macro used to build exported function names.
  * Usually "Prefix" is derived from shared library name, thus for each library
  * we'll have different exported names.  This prevents naming collisions when
  * static linking is used, and on platforms where plug-in symbols are exported
@@ -375,13 +286,34 @@
 #  define CS_IMPLEMENT_PLATFORM_APPLICATION
 #endif
 
+/**\def CS_INITIALIZE_PLATFORM_APPLICATION
+ * Perform platform-specific application initializations.
+ * This macro should be invoked very near to the "beginning" of the 
+ * application.
+ * \remark NB: It is invoked in csInitializer::CreateEnvironment().
+ */
+#ifndef CS_INITIALIZE_PLATFORM_APPLICATION
+#  define CS_INITIALIZE_PLATFORM_APPLICATION	/* */
+/*
+  This definition may seem odd, but it's here for doxygen's sake, which
+  apparently fails to document empty macro definitions.
+ */
+#endif
+
+#if !defined(CS_BUILD_SHARED_LIBS)
+# define CS_STATIC_VAR_EXTERN
+#else
+# define CS_STATIC_VAR_EXTERN	CS_CSUTIL_EXPORT
+#endif
+
 #ifndef CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION
 #  define CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION cs_static_var_cleanup
 #endif
 
 #ifndef CS_DECLARE_STATIC_VARIABLE_REGISTRATION
 #  define CS_DECLARE_STATIC_VARIABLE_REGISTRATION \
-void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)());
+    CS_STATIC_VAR_EXTERN void \
+    CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)());
 #endif
 
 #ifndef CS_DECLARE_STATIC_VARIABLE_CLEANUP
@@ -391,29 +323,35 @@ void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)());
 
 #ifndef CS_IMPLEMENT_STATIC_VARIABLE_REGISTRATION
 #  define CS_IMPLEMENT_STATIC_VARIABLE_REGISTRATION                    \
-void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())        \
+CS_STATIC_VAR_EXTERN void 					       \
+CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())             \
 {                                                                      \
-  static void (**a)()=0;                                               \
-  static int lastEntry=0;                                              \
-  static int maxEntries=0;                                             \
+  static void (**a)() = 0;                                             \
+  static int lastEntry = 0;                                            \
+  static int maxEntries = 0;                                           \
                                                                        \
-  if (p)                                                               \
+  if (p != 0)                                                          \
   {                                                                    \
     if (lastEntry >= maxEntries)                                       \
     {                                                                  \
-      maxEntries+=10;                                                  \
-      a = (void (**)())realloc (a, maxEntries*sizeof(void*));          \
+      maxEntries += 10;                                                \
+      if (a == 0)                                                      \
+        a = (void (**)())malloc(maxEntries * sizeof(void*));           \
+      else                                                             \
+        a = (void (**)())realloc(a, maxEntries * sizeof(void*));       \
     }                                                                  \
     a[lastEntry++] = p;                                                \
   }                                                                    \
-  else                                                                 \
+  else if (a != 0)                                                     \
   {                                                                    \
-    int i;                                                             \
-    for (i=lastEntry-1; i >= 0; i--)                                   \
+    for (int i = lastEntry - 1; i >= 0; i--)                           \
       a[i] ();                                                         \
     free (a);                                                          \
-   }                                                                   \
-}                                                                      
+    a = 0;                                                             \
+    lastEntry = 0;                                                     \
+    maxEntries = 0;                                                    \
+  }                                                                    \
+}
 #endif
 
 #ifndef CS_IMPLEMENT_STATIC_VARIABLE_CLEANUP
@@ -421,7 +359,7 @@ void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())        \
    CS_IMPLEMENT_STATIC_VARIABLE_REGISTRATION   
 #endif
 
-/**
+/**\def CS_IMPLEMENT_PLUGIN
  * The CS_IMPLEMENT_PLUGIN macro should be placed at the global scope in
  * exactly one compilation unit comprising a plugin module.  For maximum
  * portability, each plugin module must employ this macro.  Platforms may
@@ -429,7 +367,7 @@ void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())        \
  * of the plugin module with any special implementation details required by the
  * platform.
  */
-#if defined(CS_STATIC_LINKED)
+#if defined(CS_STATIC_LINKED) || defined(CS_BUILD_SHARED_LIBS)
 
 #  ifndef CS_IMPLEMENT_PLUGIN
 #  define CS_IMPLEMENT_PLUGIN        \
@@ -445,7 +383,8 @@ void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())        \
 #  endif
 
 #endif
-/**
+
+/**\def CS_IMPLEMENT_APPLICATION
  * The CS_IMPLEMENT_APPLICATION macro should be placed at the global scope in
  * exactly one compilation unit comprising an application.  For maximum
  * portability, each application should employ this macro.  Platforms may
@@ -454,12 +393,17 @@ void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())        \
  * platform.
  */
 #ifndef CS_IMPLEMENT_APPLICATION
+# if !defined(CS_BUILD_SHARED_LIBS)
 #  define CS_IMPLEMENT_APPLICATION       \
    CS_IMPLEMENT_STATIC_VARIABLE_CLEANUP  \
    CS_IMPLEMENT_PLATFORM_APPLICATION 
+# else
+#  define CS_IMPLEMENT_APPLICATION       \
+   CS_IMPLEMENT_PLATFORM_APPLICATION 
+# endif
 #endif
 
-/**
+/**\def CS_REGISTER_STATIC_FOR_DESTRUCTION
  * Register a method that will destruct one static variable.
  */
 #ifndef CS_REGISTER_STATIC_FOR_DESTRUCTION
@@ -467,7 +411,7 @@ void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())        \
         CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (getterFunc);
 #endif
 
-/**
+/**\def CS_STATIC_VARIABLE_CLEANUP
  * Invoke the function that will call all destruction functions
  */
 #ifndef CS_STATIC_VARIABLE_CLEANUP
@@ -475,7 +419,7 @@ void CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (void (*p)())        \
         CS_STATIC_VAR_DESTRUCTION_REGISTRAR_FUNCTION (0);
 #endif
 
-/**
+/**\def CS_IMPLEMENT_STATIC_VAR(getterFunc,Type,initParam,kill_how)
  * Create a global variable thats created on demand. Create a Getter function to access 
  * the variable and a destruction function. The Getter function will register
  * the destruction function on first invocation.
@@ -491,13 +435,15 @@ CS_DECLARE_STATIC_VARIABLE_REGISTRATION                                 \
 extern "C" {                                                            \
 static Type* getterFunc ();                                             \
 static void getterFunc ## _kill ();                                     \
+static void getterFunc ## _kill_array ();                               \
 void getterFunc ## _kill ()                                             \
 {                                                                       \
+  (void)getterFunc ## _kill_array;                                      \
   delete getterFunc ();                                                 \
 }                                                                       \
-static void getterFunc ## _kill_array ();                               \
 void getterFunc ## _kill_array ()                                       \
 {                                                                       \
+  (void)getterFunc ## _kill;                                            \
   delete [] getterFunc ();                                              \
 }                                                                       \
 Type* getterFunc ()                                                     \
@@ -523,7 +469,7 @@ Type* getterFunc ()                                                     \
  CS_IMPLEMENT_STATIC_VAR_EXT(getterFunc,Type,initParam,_kill_array)    
 #endif
 
-/**
+/**\def CS_DECLARE_STATIC_CLASSVAR(var,getterFunc,Type)
  * Declare a static variable inside a class. This will also declare a Getter function.
  * Example:
  * CS_DECLARE_STATIC_CLASSVAR (pool, GetVertexPool, csVertexPool)
@@ -540,7 +486,7 @@ static Type *var;                                             \
 static Type &getterFunc ();                                   
 #endif
 
-/**
+/**\def CS_IMPLEMENT_STATIC_CLASSVAR(Class,var,getterFunc,Type,initParam)
  * Create the static class variable that has been declared with 
  * CS_DECLARE_STATIC_CLASSVAR.
  * This will also create the Getter function and the destruction function. 
@@ -554,13 +500,15 @@ static Type &getterFunc ();
 Type *Class::var = 0;                                                                    \
 extern "C" {                                                                             \
 static void Class ## _ ## getterFunc ## _kill ();                                        \
+static void Class ## _ ## getterFunc ## _kill_array ();                                  \
 void Class ## _ ## getterFunc ## _kill ()                                                \
 {                                                                                        \
+  (void) Class ## _ ## getterFunc ## _kill_array;                                        \
   delete Class::getterFunc ();                                                           \
 }                                                                                        \
-static void Class ## _ ## getterFunc ## _kill_array ();                                  \
 void Class ## _ ## getterFunc ## _kill_array ()                                          \
 {                                                                                        \
+  (void) Class ## _ ## getterFunc ## _kill;                                              \
   delete [] Class::getterFunc ();                                                        \
 }                                                                                        \
 }                                                                                        \
@@ -591,13 +539,15 @@ Type* Class::getterFunc ()
 Type *Class::var = 0;                                                                        \
 extern "C" {                                                                                 \
 static void Class ## _ ## getterFunc ## _kill ();                                            \
+static void Class ## _ ## getterFunc ## _kill_array ();                                      \
 void Class ## _ ## getterFunc ## _kill ()                                                    \
 {                                                                                            \
+  (void) Class ## _ ## getterFunc ## _kill_array;                                            \
   delete &Class::getterFunc ();                                                              \
 }                                                                                            \
-static void Class ## _ ## getterFunc ## _kill_array ();                                      \
 void Class ## _ ## getterFunc ## _kill_array ()                                              \
 {                                                                                            \
+  (void) Class ## _ ## getterFunc ## _kill;                                                  \
   delete [] &Class::getterFunc ();                                                           \
 }                                                                                            \
 }                                                                                            \
@@ -635,26 +585,34 @@ Type &Class::getterFunc ()
 // Do not enable CS_EXTENSIVE_MEMDEBUG if your platform or your own code
 // defines its own 'new' operator, since this version will interfere with your
 // own.
+// CS_MEMORY_TRACKER is treated like CS_EXTENSIVE_MEMDEBUG here.
+// Same for CS_REF_TRACKER.
 #ifndef CS_DEBUG
 #  undef CS_EXTENSIVE_MEMDEBUG
+#  undef CS_MEMORY_TRACKER
+#  undef CS_REF_TRACKER
+#else
+#  if defined(CS_EXTENSIVE_MEMDEBUG) && defined(CS_MEMORY_TRACKER)
+#    error Do not use CS_EXTENSIVE_MEMDEBUG and CS_MEMORY_TRACKER at the same time!
+#  endif
 #endif
-#ifdef CS_EXTENSIVE_MEMDEBUG
+#if defined(CS_EXTENSIVE_MEMDEBUG) || defined(CS_MEMORY_TRACKER)
 extern void* operator new (size_t s, void* filename, int line);
 extern void* operator new[] (size_t s, void* filename, int line);
-#define NEW new ((void*)__FILE__, __LINE__)
-#define new NEW
+#define CS_EXTENSIVE_MEMDEBUG_NEW new ((void*)__FILE__, __LINE__)
+#define new CS_EXTENSIVE_MEMDEBUG_NEW
 #endif
 
 #ifdef CS_DEBUG
 #  if !defined (DEBUG_BREAK)
 #    if defined (PROC_X86)
 #      if defined (COMP_GCC)
-#        define DEBUG_BREAK	asm ("int $3")
+#        define DEBUG_BREAK asm ("int $3")
 #      else
-#        define DEBUG_BREAK	_asm int 3
+#        define DEBUG_BREAK _asm int 3
 #      endif
 #    else
-#      define DEBUG_BREAK	{ static int x = 0; x /= x; }
+#      define DEBUG_BREAK { static int x = 0; x /= x; }
 #    endif
 #  endif
 #  if !defined (CS_ASSERT)
@@ -662,20 +620,25 @@ extern void* operator new[] (size_t s, void* filename, int line);
 #      define  CS_ASSERT(x) assert(x)
 #    else
 #      include <stdio.h>
-#      define CS_ASSERT(x)						\
-         if (!(x))							\
-         {								\
+#      define CS_ASSERT(x)            \
+         if (!(x))              \
+         {                \
            fprintf (stderr, __FILE__ ":%d: failed assertion '%s'\n",\
-             int(__LINE__), #x );					\
-           DEBUG_BREAK;							\
+             int(__LINE__), #x );         \
+           DEBUG_BREAK;             \
          }
 #    endif
 #  endif
+#  if !defined (CS_ASSERT_MSG)
+#      define  CS_ASSERT_MSG(msg, x) CS_ASSERT(((msg) && (x)))
+#  endif
 #else
 #  undef DEBUG_BREAK
 #  define DEBUG_BREAK
 #  undef CS_ASSERT
 #  define CS_ASSERT(x)
+#  undef CS_ASSERT_MSG
+#  define CS_ASSERT_MSG(m, x)
 #endif
 
 // Check if the csosdefs.h defined either CS_LITTLE_ENDIAN or CS_BIG_ENDIAN
@@ -683,6 +646,18 @@ extern void* operator new[] (size_t s, void* filename, int line);
 #  error No CS_XXX_ENDIAN macro defined in your OS-specific csosdefs.h!
 #endif
 
+/*
+ * This is a bit of overkill but if you're sure your CPU doesn't require
+ * strict alignment add your CPU to the !defined below to get slightly
+ * smaller and faster code in some cases.
+ *
+ * @@@ In the future, this should be moved to volatile.h and determined as
+ * part of the configuration process.
+ */
+#if !defined (PROC_X86)
+#  define CS_STRICT_ALIGNMENT
+#endif
+
 // Adjust some definitions contained in volatile.h
 #if defined (PROC_X86) && !defined (DO_NASM)
 #  undef NO_ASSEMBLER
@@ -706,15 +681,75 @@ extern void* operator new[] (size_t s, void* filename, int line);
 // gcc can perform usefull checking for printf/scanf format strings, just add
 // this define at the end of the function declaration
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
-#  define CS_GNUC_PRINTF( format_idx, arg_idx)		\
-     __attribute__((format (printf, format_idx, arg_idx)))
-#  define CS_GNUC_SCANF( format_idx, arg_idx )				\
-     __attribute__((format (scanf, format_idx, arg_idx)))
+#  define CS_GNUC_PRINTF( format_idx, arg_idx)    \
+     __attribute__((format (__printf__, format_idx, arg_idx)))
+#  define CS_GNUC_SCANF( format_idx, arg_idx )        \
+     __attribute__((format (__scanf__, format_idx, arg_idx)))
 #else
 #  define CS_GNUC_PRINTF( format_idx, arg_idx )
 #  define CS_GNUC_SCANF( format_idx, arg_idx )
 #endif
 
+// remove __attribute__ on non GNUC compilers
+#ifndef __GNUC__
+#define __attribute__(x)
+#endif
+
+// Support for alignment and packing of structures.
+#if !defined(CS_STRUCT_ALIGN_4BYTE_BEGIN)
+#  if defined(__GNUC__) && defined(CS_STRICT_ALIGNMENT)
+#    define CS_STRUCT_ALIGN_4BYTE_BEGIN
+#    define CS_STRUCT_ALIGN_4BYTE_END __attribute__ ((aligned(4)))
+#  else
+#    define CS_STRUCT_ALIGN_4BYTE_BEGIN
+#    define CS_STRUCT_ALIGN_4BYTE_END
+#  endif
+#endif
+
+// Macro used to define static implicit pointer conversion function.
+// Only use within a class declaration.
+#ifndef _CS_IMPLICITPTRCAST_NAME
+#  define _CS_IMPLICITPTRCAST_NAME __ImplicitPtrCast
+#endif
+/**
+ * Implements a static member function for a class which can be used to
+ * perform implicit pointer casts.
+ * \param classname Name of the class that the macro is being used in.
+ * \remarks
+ * This macro is intended to support typecasting within macros, allowing the
+ * compiler to provide a more descriptive error message. Use
+ * CS_IMPLEMENT_IMPLICIT_PTR_CAST in the declaration of the class and
+ * CS_IMPLICIT_PTR_CAST in the macro declaration.
+ * \par Example:
+ * \code
+ * struct iObjectRegistry : public iBase
+ * {
+ *   // Allow implicit casts through static function.
+ *   CS_IMPLEMENT_IMPLICIT_PTR_CAST(iObjectRegistry);
+ *   ...
+ * }
+ *
+ * #define CS_QUERY_REGISTRY_TAG(Reg, Tag) \
+ *  csPtr<iBase> (CS_IMPLICIT_PTR_CAST(iObjectRegistry, Reg)->Get (Tag))
+ * \endcode
+ */
+#define CS_IMPLEMENT_IMPLICIT_PTR_CAST(classname) \
+  inline static classname* _CS_IMPLICITPTRCAST_NAME (classname* ptr) \
+  { \
+    return ptr;\
+  }
+
+/**
+ * Perform a compiler implicit cast of a pointer to another pointer type
+ * using a static member function declared with the
+ * \c CS_IMPLEMENT_IMPLICIT_PTR_CAST macro.
+ * \param classname Name of the class to convert to
+ * \param ptr Pointer to be convereted into 
+ * \see CS_IMPLEMENT_IMPLICIT_PTR_CAST
+ */
+#define CS_IMPLICIT_PTR_CAST(classname, ptr) \
+  (classname::_CS_IMPLICITPTRCAST_NAME(ptr))
+
 /// Fatal exit routine (which can be replaced if neccessary)
 extern void (*fatal_exit) (int errorcode, bool canreturn);
 
