@@ -16,22 +16,28 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __PROCDOTSTEX_H__
-#define __PROCDOTSTEX_H__
+#ifndef __CS_PROCDOTSTEX_H__
+#define __CS_PROCDOTSTEX_H__
+
+#include "csextern.h"
 
 #include "csutil/cscolor.h"
 #include "cstool/proctex.h"
 
 /**
- *  Random dots (demonstration procedural texture).
+ * Random dots (demonstration procedural texture).
  */
-class csProcDots : public csProcTexture
+class CS_CSTOOL_EXPORT csProcDots : public csProcTexture
 {
 private:
   /// palette
   int *palette;
   /// number of colours in palette
   int palsize;
+  /// Accumulated elapsed time.
+  csTicks elapsed;
+  /// State.
+  int state;
 
   /// make my palette, max nr of colours
   void MakePalette (int max);
@@ -48,5 +54,5 @@ public:
   virtual void Animate (csTicks current_time);
 };
 
-#endif // __PROCDOTSTEX_H__
+#endif // __CS_PROCDOTSTEX_H__
 
