@@ -18,17 +18,20 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __SNDWRAP_H__
-#define __SNDWRAP_H__
+#ifndef __CS_SNDWRAP_H__
+#define __CS_SNDWRAP_H__
+
+#include "csextern.h"
 
 #include "csutil/csobject.h"
 #include "isound/wrapper.h"
 
-class csSoundWrapper : public csObject
+/// Document me and possible move me to some sound manager! @@@
+class CS_CSTOOL_EXPORT csSoundWrapper : public csObject
 {
 protected:
   ///
-  iSoundHandle* SoundHandle;
+  csRef<iSoundHandle> SoundHandle;
 
 public:
 
@@ -50,4 +53,4 @@ public:
   } scfiSoundWrapper;
 };
 
-#endif // __SNDWRAP_H__
+#endif // __CS_SNDWRAP_H__
