@@ -16,9 +16,12 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
+
 #include <math.h>
 #include "cssysdef.h"
 #include "csgeom/box.h"
+#include "csgeom/transfrm.h"
+#include "csgeom/poly2d.h"
 
 //---------------------------------------------------------------------------
 csBox2::bEdge csBox2:: edges[8] =
@@ -41,6 +44,7 @@ csVector2 csBox2::GetCorner (int corner) const
     case CS_BOX_CORNER_xY:  return csVector2 (MinX (), MaxY ());
     case CS_BOX_CORNER_Xy:  return csVector2 (MaxX (), MinY ());
     case CS_BOX_CORNER_XY:  return Max ();
+    case CS_BOX_CENTER2:    return GetCenter ();
   }
 
   return csVector2 (0, 0);
@@ -87,6 +91,15 @@ csBox2 &csBox2::operator*= (const csBox2 &box)
   return *this;
 }
 
+bool csBox2::TestIntersect (const csBox2& box) const
+{
+  if (box.minbox.x >= maxbox.x) return false;
+  if (box.minbox.y >= maxbox.y) return false;
+  if (box.maxbox.x <= minbox.x) return false;
+  if (box.maxbox.y <= minbox.y) return false;
+  return true;
+}
+
 csBox2 operator+ (const csBox2 &box1, const csBox2 &box2)
 {
   return csBox2 (
@@ -269,13 +282,20 @@ float csBox2::SquaredOriginMaxDist () const
  *	5: right/down/back
  *	6: right/up/front
  *	7: right/up/back
+ * Note that every entry in this table contains 8 vertices.
+ * The first are for the outline but the remaining non-outline
+ * indexes are also included.
+ * Note that even though 'num' may be equal to 7 there are always
+ * at most 6 vertices valid for the real outline. If there is a
+ * seventh element then that's one additional vertex that is visible
+ * but not part of the outline (i.e. the center point).
  */
 
 // This table also contains an array of sides visible from that region.
 struct Outline
 {
   int num;
-  int vertices[7];
+  int vertices[8];
   int num_sides;
   int sides[3];
 };
@@ -283,93 +303,60 @@ struct Outline
 /// Outline lookup table.
 static Outline outlines[27] =
 {
-  { 7, { 3, 2, 6, 4, 5, 1, 0 }, 3, { CS_BOX_SIDE_x, CS_BOX_SIDE_y,
-        CS_BOX_SIDE_z } },
-
-  //000
-  { 6, { 3, 2, 0, 4, 5, 1, -1 }, 2, { CS_BOX_SIDE_x, CS_BOX_SIDE_y, -1 } },
-
-  //001
-  { 7, { 7, 3, 2, 0, 4, 5, 1 }, 3, { CS_BOX_SIDE_x, CS_BOX_SIDE_y,
-        CS_BOX_SIDE_Z } },
-
-  //002
-  { 6, { 3, 2, 6, 4, 0, 1, -1 }, 2, { CS_BOX_SIDE_x, CS_BOX_SIDE_z, -1 } },
-
-  //010
-  { 4, { 3, 2, 0, 1, -1, -1, -1 }, 1, { CS_BOX_SIDE_x, -1, -1 } },
-
-  //011
-  { 6, { 7, 3, 2, 0, 1, 5, -1 }, 2, { CS_BOX_SIDE_x, CS_BOX_SIDE_Z, -1 } },
-
-  //012
-  { 7, { 3, 7, 6, 4, 0, 1, 2 }, 3, { CS_BOX_SIDE_x, CS_BOX_SIDE_Y,
-        CS_BOX_SIDE_z } },
-
-  //020
-  { 6, { 3, 7, 6, 2, 0, 1, -1 }, 2, { CS_BOX_SIDE_x, CS_BOX_SIDE_Y, -1 } },
-
-  //021
-  { 7, { 7, 6, 2, 0, 1, 5, 3 }, 3, { CS_BOX_SIDE_x, CS_BOX_SIDE_Y,
-        CS_BOX_SIDE_Z } },
-
-  //022
-  { 6, { 2, 6, 4, 5, 1, 0, -1 }, 2, { CS_BOX_SIDE_y, CS_BOX_SIDE_z, -1 } },
-
-  //100
-  { 4, { 0, 4, 5, 1, -1, -1, -1 }, 1, { CS_BOX_SIDE_y, -1, -1 } },
-
-  //101
-  { 6, { 3, 1, 0, 4, 5, 7, -1 }, 2, { CS_BOX_SIDE_y, CS_BOX_SIDE_Z, -1 } },
-
-  //102
-  { 4, { 2, 6, 4, 0, -1, -1, -1 }, 1, { CS_BOX_SIDE_z, -1, -1 } },
-
-  //110
-  { 0, { -1, -1, -1, -1, -1, -1, -1 }, 0, { -1, -1, -1 } },
-
-  //111
-  { 4, { 7, 3, 1, 5, -1, -1, -1 }, 1, { CS_BOX_SIDE_Z, -1, -1 } },
-
-  //112
-  { 6, { 3, 7, 6, 4, 0, 2, -1 }, 2, { CS_BOX_SIDE_Y, CS_BOX_SIDE_z, -1 } },
-
-  //120
-  { 4, { 3, 7, 6, 2, -1, -1, -1 }, 1, { CS_BOX_SIDE_Y, -1, -1 } },
-
-  //121
-  { 6, { 2, 3, 1, 5, 7, 6, -1 }, 2, { CS_BOX_SIDE_Y, CS_BOX_SIDE_Z, -1 } },
-
-  //122
-  { 7, { 2, 6, 7, 5, 1, 0, 4 }, 3, { CS_BOX_SIDE_X, CS_BOX_SIDE_y,
-        CS_BOX_SIDE_z } },
-
-  //200
-  { 6, { 6, 7, 5, 1, 0, 4, -1 }, 2, { CS_BOX_SIDE_X, CS_BOX_SIDE_y, -1 } },
-
-  //201
-  { 7, { 6, 7, 3, 1, 0, 4, 5 }, 3, { CS_BOX_SIDE_X, CS_BOX_SIDE_y,
-        CS_BOX_SIDE_Z } },
-
-  //202
-  { 6, { 2, 6, 7, 5, 4, 0, -1 }, 2, { CS_BOX_SIDE_X, CS_BOX_SIDE_z, -1 } },
-
-  //210
-  { 4, { 6, 7, 5, 4, -1, -1, -1 }, 1, { CS_BOX_SIDE_X, -1, -1 } },
-
-  //211
-  { 6, { 6, 7, 3, 1, 5, 4, -1 }, 2, { CS_BOX_SIDE_X, CS_BOX_SIDE_Z, -1 } },
-
-  //212
-  { 7, { 2, 3, 7, 5, 4, 0, 6 }, 3, { CS_BOX_SIDE_X, CS_BOX_SIDE_Y,
-        CS_BOX_SIDE_z } },
-
-  //220
-  { 6, { 2, 3, 7, 5, 4, 6, -1 }, 2, { CS_BOX_SIDE_X, CS_BOX_SIDE_Y, -1 } },
-
-  //221
-  { 7, { 6, 2, 3, 1, 5, 4, 7 }, 3, { CS_BOX_SIDE_X, CS_BOX_SIDE_Y,
-        CS_BOX_SIDE_Z } } //222
+  { 7, { 3, 2, 6, 4, 5, 1, 0, /**/ 7 },
+    3, { CS_BOX_SIDE_x, CS_BOX_SIDE_y, CS_BOX_SIDE_z } }, //000
+  { 6, { 3, 2, 0, 4, 5, 1, /**/ 6, 7 },
+    2, { CS_BOX_SIDE_x, CS_BOX_SIDE_y, -1 } }, //001
+  { 7, { 7, 3, 2, 0, 4, 5, 1, /**/ 6 },
+    3, { CS_BOX_SIDE_x, CS_BOX_SIDE_y, CS_BOX_SIDE_Z } }, //002
+  { 6, { 3, 2, 6, 4, 0, 1, /**/ 5, 7 },
+    2, { CS_BOX_SIDE_x, CS_BOX_SIDE_z, -1 } }, //010
+  { 4, { 3, 2, 0, 1, /**/ 4, 5, 6, 7 },
+    1, { CS_BOX_SIDE_x, -1, -1 } }, //011
+  { 6, { 7, 3, 2, 0, 1, 5, /**/ 4, 6 },
+    2, { CS_BOX_SIDE_x, CS_BOX_SIDE_Z, -1 } }, //012
+  { 7, { 3, 7, 6, 4, 0, 1, 2, /**/ 5 },
+    3, { CS_BOX_SIDE_x, CS_BOX_SIDE_Y, CS_BOX_SIDE_z } }, //020
+  { 6, { 3, 7, 6, 2, 0, 1, /**/ 4, 5 },
+    2, { CS_BOX_SIDE_x, CS_BOX_SIDE_Y, -1 } }, //021
+  { 7, { 7, 6, 2, 0, 1, 5, 3, /**/ 4 },
+    3, { CS_BOX_SIDE_x, CS_BOX_SIDE_Y, CS_BOX_SIDE_Z } }, //022
+  { 6, { 2, 6, 4, 5, 1, 0, /**/ 3, 7 },
+    2, { CS_BOX_SIDE_y, CS_BOX_SIDE_z, -1 } }, //100
+  { 4, { 0, 4, 5, 1, /**/ 2, 3, 6, 7 },
+    1, { CS_BOX_SIDE_y, -1, -1 } }, //101
+  { 6, { 3, 1, 0, 4, 5, 7, /**/ 2, 6 },
+    2, { CS_BOX_SIDE_y, CS_BOX_SIDE_Z, -1 } }, //102
+  { 4, { 2, 6, 4, 0, /**/ 1, 3, 5, 7 },
+    1, { CS_BOX_SIDE_z, -1, -1 } }, //110
+  { 0, { /**/ 0, 1, 2, 3, 4, 5, 6, 7 },
+    0, { -1, -1, -1 } }, //111
+  { 4, { 7, 3, 1, 5, /**/ 0, 2, 4, 6 },
+    1, { CS_BOX_SIDE_Z, -1, -1 } }, //112
+  { 6, { 3, 7, 6, 4, 0, 2, /**/ 1, 5 },
+    2, { CS_BOX_SIDE_Y, CS_BOX_SIDE_z, -1 } }, //120
+  { 4, { 3, 7, 6, 2, /**/ 0, 1, 4, 5 },
+    1, { CS_BOX_SIDE_Y, -1, -1 } }, //121
+  { 6, { 2, 3, 1, 5, 7, 6, /**/ 0, 4 },
+    2, { CS_BOX_SIDE_Y, CS_BOX_SIDE_Z, -1 } }, //122
+  { 7, { 2, 6, 7, 5, 1, 0, 4, /**/ 3 },
+    3, { CS_BOX_SIDE_X, CS_BOX_SIDE_y, CS_BOX_SIDE_z } }, //200
+  { 6, { 6, 7, 5, 1, 0, 4, /**/ 2, 3 },
+    2, { CS_BOX_SIDE_X, CS_BOX_SIDE_y, -1 } }, //201
+  { 7, { 6, 7, 3, 1, 0, 4, 5, /**/ 2 },
+    3, { CS_BOX_SIDE_X, CS_BOX_SIDE_y, CS_BOX_SIDE_Z } }, //202
+  { 6, { 2, 6, 7, 5, 4, 0, /**/ 1, 3 },
+    2, { CS_BOX_SIDE_X, CS_BOX_SIDE_z, -1 } }, //210
+  { 4, { 6, 7, 5, 4, /**/ 0, 1, 2, 3 },
+    1, { CS_BOX_SIDE_X, -1, -1 } }, //211
+  { 6, { 6, 7, 3, 1, 5, 4, /**/ 0, 2 },
+    2, { CS_BOX_SIDE_X, CS_BOX_SIDE_Z, -1 } }, //212
+  { 7, { 2, 3, 7, 5, 4, 0, 6, /**/ 1 },
+    3, { CS_BOX_SIDE_X, CS_BOX_SIDE_Y, CS_BOX_SIDE_z } }, //220
+  { 6, { 2, 3, 7, 5, 4, 6, /**/ 0, 1 },
+    2, { CS_BOX_SIDE_X, CS_BOX_SIDE_Y, -1 } }, //221
+  { 7, { 6, 2, 3, 1, 5, 4, 7, /**/ 0 },
+    3, { CS_BOX_SIDE_X, CS_BOX_SIDE_Y, CS_BOX_SIDE_Z } } //222
 };
 
 csBox3::bEdge csBox3:: edges[24] =
@@ -429,6 +416,7 @@ csVector3 csBox3::GetCorner (int corner) const
     case CS_BOX_CORNER_XyZ: return csVector3 (MaxX (), MinY (), MaxZ ());
     case CS_BOX_CORNER_XYz: return csVector3 (MaxX (), MaxY (), MinZ ());
     case CS_BOX_CORNER_XYZ: return Max ();
+    case CS_BOX_CENTER3:    return GetCenter ();
   }
 
   return csVector3 (0, 0, 0);
@@ -537,7 +525,7 @@ int csBox3::Adjacent (const csBox3 &other) const
   return -1;
 }
 
-int csBox3::GetVisibleSides (const csVector3 &pos, int *visible_sides) const
+int csBox3::CalculatePointSegment (const csVector3& pos) const
 {
   const csVector3 &bmin = Min ();
   const csVector3 &bmax = Max ();
@@ -567,6 +555,12 @@ int csBox3::GetVisibleSides (const csVector3 &pos, int *visible_sides) const
   else
     idx += 1;
 
+  return idx;
+}
+
+int csBox3::GetVisibleSides (const csVector3 &pos, int *visible_sides) const
+{
+  int idx = CalculatePointSegment (pos);
   const Outline &ol = outlines[idx];
   int num_array = ol.num_sides;
   int i;
@@ -582,31 +576,7 @@ void csBox3::GetConvexOutline (
 {
   const csVector3 &bmin = Min ();
   const csVector3 &bmax = Max ();
-  int idx;
-
-  // First select x part of coordinate.
-  if (pos.x < bmin.x)
-    idx = 0 * 9;
-  else if (pos.x > bmax.x)
-    idx = 2 * 9;
-  else
-    idx = 1 * 9;
-
-  // Then y part.
-  if (pos.y < bmin.y)
-    idx += 0 * 3;
-  else if (pos.y > bmax.y)
-    idx += 2 * 3;
-  else
-    idx += 1 * 3;
-
-  // Then z part.
-  if (pos.z < bmin.z)
-    idx += 0;
-  else if (pos.z > bmax.z)
-    idx += 2;
-  else
-    idx += 1;
+  int idx = CalculatePointSegment (pos);
 
   const Outline &ol = outlines[idx];
   num_array = (bVisible ? ol.num : MIN (ol.num, 6));
@@ -692,7 +662,7 @@ float csBox3::SquaredOriginDist () const
   if (minbox.z > 0)
     res += minbox.z * minbox.z;
   else if (maxbox.z < 0)
-    res += maxbox.z * maxbox.z;;
+    res += maxbox.z * maxbox.z;
   return res;
 }
 
@@ -722,6 +692,158 @@ float csBox3::SquaredOriginMaxDist () const
   return res;
 }
 
+// Version to cope with z <= 0. This is wrong but it in the places where
+// it is used below the result is acceptable because it generates a
+// conservative result (i.e. a box or outline that is bigger then reality).
+static void PerspectiveWrong (const csVector3& v, csVector2& p, float fov,
+    	float sx, float sy)
+{
+  float iz = fov * 10;
+  p.x = v.x * iz + sx;
+  p.y = v.y * iz + sy;
+}
+
+static void Perspective (const csVector3& v, csVector2& p, float fov,
+    	float sx, float sy)
+{
+  float iz = fov / v.z;
+  p.x = v.x * iz + sx;
+  p.y = v.y * iz + sy;
+}
+
+bool csBox3::ProjectBox (const csTransform& trans, float fov,
+	float sx, float sy, csBox2& sbox, float& min_z, float& max_z) const
+{
+  const csVector3& origin = trans.GetOrigin ();
+  int idx = CalculatePointSegment (origin);
+  const Outline &ol = outlines[idx];
+  int num_array = MIN (ol.num, 6);
+
+  csBox3 cbox (trans * GetCorner (ol.vertices[0]));
+  int i;
+  // We go to 8 so that we can calculate the correct min_z/max_z.
+  // If we only go to num_array we will only calculate min_z/max_z
+  // for the outine vertices.
+  for (i = 1; i < 8; i++)
+  {
+    csVector3 v = trans * GetCorner (ol.vertices[i]);
+    if (i < num_array)
+    {
+      cbox.AddBoundingVertexSmart (v);
+      min_z = cbox.MinZ ();
+      max_z = cbox.MaxZ ();
+    }
+    else
+    {
+      if (v.z < min_z) min_z = v.z;
+      if (v.z > max_z) max_z = v.z;
+    }
+  }
+
+  if (max_z < 0.01) return false;
+
+// @@@ In theory we can optimize here again by calling CalculatePointSegment
+// again for the new box and the 0,0,0 point. By doing that we could
+// avoid doing four perspective projections.
+
+  // If z < .1 we do conservative clipping. Not correct but it will generate
+  // a box that is bigger then the real one which is ok for testing culling.
+  csVector2 oneCorner;
+  if (cbox.Max ().z < .1)
+    PerspectiveWrong (cbox.Max (), oneCorner, fov, sx, sy);
+  else
+    Perspective (cbox.Max (), oneCorner, fov, sx, sy);
+  sbox.StartBoundingBox (oneCorner);
+
+  csVector3 v (cbox.MinX (), cbox.MinY (), cbox.MaxZ ());
+  if (v.z < .1)
+    PerspectiveWrong (v, oneCorner, fov, sx, sy);
+  else
+    Perspective (v, oneCorner, fov, sx, sy);
+  sbox.AddBoundingVertexSmart (oneCorner);
+
+  if (cbox.Min ().z < .1)
+    PerspectiveWrong (cbox.Min (), oneCorner, fov, sx, sy);
+  else
+    Perspective (cbox.Min (), oneCorner, fov, sx, sy);
+  sbox.AddBoundingVertexSmart (oneCorner);
+
+  v.Set (cbox.MaxX (), cbox.MaxY (), cbox.MinZ ());
+  if (v.z < .1)
+    PerspectiveWrong (v, oneCorner, fov, sx, sy);
+  else
+    Perspective (v, oneCorner, fov, sx, sy);
+  sbox.AddBoundingVertexSmart (oneCorner);
+
+  return true;
+}
+
+bool csBox3::ProjectOutline (const csTransform& trans, float fov,
+	float sx, float sy,
+  	csPoly2D& poly, float& min_z, float& max_z) const
+{
+  const csVector3& origin = trans.GetOrigin ();
+  int idx = CalculatePointSegment (origin);
+  const Outline &ol = outlines[idx];
+  int num_array = MIN (ol.num, 6);
+  poly.SetVertexCount (num_array);
+
+  min_z = 100000000.0;
+  max_z = 0;
+  int i;
+  // We go to 8 so that we can calculate the correct min_z/max_z.
+  // If we only go to num_array we will only calculate min_z/max_z
+  // for the outine vertices.
+  for (i = 0 ; i < 8 ; i++)
+  {
+    csVector3 v = trans * GetCorner (ol.vertices[i]);
+    if (v.z > max_z) max_z = v.z;
+    if (v.z < min_z) min_z = v.z;
+    if (i < num_array)
+    {
+      if (v.z < .1)
+        PerspectiveWrong (v, poly[i], fov, sx, sy);
+      else
+        Perspective (v, poly[i], fov, sx, sy);
+    }
+  }
+  return max_z >= .1;
+}
+
+bool csBox3::ProjectBoxAndOutline (const csTransform& trans, float fov,
+	float sx, float sy, csBox2& sbox,
+  	csPoly2D& poly, float& min_z, float& max_z) const
+{
+  const csVector3& origin = trans.GetOrigin ();
+  int idx = CalculatePointSegment (origin);
+  const Outline &ol = outlines[idx];
+  int num_array = MIN (ol.num, 6);
+  poly.SetVertexCount (num_array);
+
+  min_z = 100000000.0;
+  max_z = 0;
+  sbox.StartBoundingBox ();
+  int i;
+  // We go to 8 so that we can calculate the correct min_z/max_z.
+  // If we only go to num_array we will only calculate min_z/max_z
+  // for the outine vertices.
+  for (i = 0 ; i < 8 ; i++)
+  {
+    csVector3 v = trans * GetCorner (ol.vertices[i]);
+    if (v.z > max_z) max_z = v.z;
+    if (v.z < min_z) min_z = v.z;
+    if (i < num_array)
+    {
+      if (v.z < .1)
+        PerspectiveWrong (v, poly[i], fov, sx, sy);
+      else
+        Perspective (v, poly[i], fov, sx, sy);
+      sbox.AddBoundingVertex (poly[i]);
+    }
+  }
+  return max_z >= .1;
+}
+
 csBox3 &csBox3::operator+= (const csBox3 &box)
 {
   if (box.minbox.x < minbox.x) minbox.x = box.minbox.x;
@@ -755,6 +877,17 @@ csBox3 &csBox3::operator*= (const csBox3 &box)
   return *this;
 }
 
+bool csBox3::TestIntersect (const csBox3& box) const
+{
+  if (box.minbox.x >= maxbox.x) return false;
+  if (box.minbox.y >= maxbox.y) return false;
+  if (box.minbox.z >= maxbox.z) return false;
+  if (box.maxbox.x <= minbox.x) return false;
+  if (box.maxbox.y <= minbox.y) return false;
+  if (box.maxbox.z <= minbox.z) return false;
+  return true;
+}
+
 csBox3 operator+ (const csBox3 &box1, const csBox3 &box2)
 {
   return csBox3 (
