@@ -30,11 +30,11 @@
 class csConfigNode
 {
 public:
-  // create a new config node. Set name to NULL to create the initial node.
+  // create a new config node. Set name to 0 to create the initial node.
   csConfigNode(const char *Name);
   // delete this node
   ~csConfigNode();
-  // delete all nodes will non-NULL name
+  // delete all nodes will non-0 name
   void DeleteDataNodes();
   // insert this node after the given node
   void InsertAfter(csConfigNode *Where);
@@ -71,17 +71,17 @@ private:
 
 csConfigNode::csConfigNode(const char *Keyname)
 {
-  Prev = Next = NULL;
+  Prev = Next = 0;
   Name = csStrNew(Keyname);
-  Data = Comment = NULL;
+  Data = Comment = 0;
 }
 
 csConfigNode::~csConfigNode()
 {
   Remove();
-  if (Name) delete[] Name;
-  if (Data) delete[] Data;
-  if (Comment) delete[] Comment;
+  delete[] Name;
+  delete[] Data;
+  delete[] Comment;
 }
 
 const char *csConfigNode::GetName() const
@@ -118,12 +118,12 @@ void csConfigNode::Remove()
 {
   if (Next) Next->Prev = Prev;
   if (Prev) Prev->Next = Next;
-  Prev = Next = NULL;
+  Prev = Next = 0;
 }
 
 void csConfigNode::SetStr(const char *s)
 {
-  if (Data) delete[] Data;
+  delete[] Data;
   Data = csStrNew(s);
 }
 
@@ -150,7 +150,7 @@ void csConfigNode::SetBool(bool b)
 
 void csConfigNode::SetComment(const char *s)
 {
-  if (Comment) delete[] Comment;
+  delete[] Comment;
   Comment = csStrNew(s);
 }
 
@@ -215,7 +215,7 @@ public:
   virtual const char *GetStr() const;
   /// Get a boolean value from the configuration.
   virtual bool GetBool() const;
-  /// Get the comment of the given key, or NULL if no comment exists.
+  /// Get the comment of the given key, or 0 if no comment exists.
   virtual const char *GetComment() const;
 
 private:
@@ -223,7 +223,7 @@ private:
   csConfigFile *Config;
   csConfigNode *Node;
   char *Subsection;
-  int SubsectionLength;
+  size_t SubsectionLength;
 
   // Create a new iterator.
   csConfigIterator(csConfigFile *Config, const char *Subsection);
@@ -245,7 +245,7 @@ SCF_IMPLEMENT_IBASE_END;
 
 csConfigIterator::csConfigIterator(csConfigFile *c, const char *sub)
 {
-  SCF_CONSTRUCT_IBASE(NULL);
+  SCF_CONSTRUCT_IBASE(0);
   Config = c;
   Node = Config->FirstNode;
   Subsection = csStrNew(sub);
@@ -256,8 +256,9 @@ csConfigIterator::csConfigIterator(csConfigFile *c, const char *sub)
 csConfigIterator::~csConfigIterator()
 {
   Config->RemoveIterator(this);
-  if (Subsection) delete[] Subsection;
+  delete[] Subsection;
   Config->DecRef();
+  SCF_DESTRUCT_IBASE ();
 }
 
 iConfigFile *csConfigIterator::GetConfigFile() const
@@ -279,14 +280,14 @@ bool csConfigIterator::DoPrev()
 {
   if (!Node->GetPrev()) return false;
   Node = Node->GetPrev();
-  return (Node->GetName() != NULL);
+  return (Node->GetName() != 0);
 }
 
 bool csConfigIterator::DoNext()
 {
   if (!Node->GetNext()) return false;
   Node = Node->GetNext();
-  return (Node->GetName() != NULL);
+  return (Node->GetName() != 0);
 }
 
 bool csConfigIterator::CheckSubsection(const char *Key) const
@@ -355,15 +356,14 @@ SCF_IMPLEMENT_IBASE_END;
 
 void csConfigFile::InitializeObject ()
 {
-  FirstNode = new csConfigNode(NULL);
-  LastNode = new csConfigNode(NULL);
+  FirstNode = new csConfigNode(0);
+  LastNode = new csConfigNode(0);
   LastNode->InsertAfter(FirstNode);
 
-  Iterators = new csVector();
-  Filename = NULL;
-  VFS = NULL;
+  Iterators = new csArray<csConfigIterator*>();
+  Filename = 0;
   Dirty = false;
-  EOFComment = NULL;
+  EOFComment = 0;
 }
 
 csConfigFile::csConfigFile(iBase *pBase)
@@ -374,7 +374,7 @@ csConfigFile::csConfigFile(iBase *pBase)
 
 csConfigFile::csConfigFile(const char *file, iVFS *vfs)
 {
-  SCF_CONSTRUCT_IBASE(NULL);
+  SCF_CONSTRUCT_IBASE(0);
   InitializeObject ();
   if (file)
     Load(file, vfs);
@@ -389,8 +389,8 @@ csConfigFile::~csConfigFile()
   // deleted there shouldn't be any iterators left.
   CS_ASSERT(Iterators->Length() == 0);
   delete Iterators;
-  if (Filename) delete[] Filename;
-  if (VFS) VFS->DecRef();
+  delete[] Filename;
+  SCF_DESTRUCT_IBASE ();
 }
 
 const char* csConfigFile::GetFileName() const
@@ -405,12 +405,10 @@ iVFS* csConfigFile::GetVFS() const
 
 void csConfigFile::SetFileName(const char *fName, iVFS *vfs)
 {
-  if (Filename) delete[] Filename;
-  if (VFS) VFS->DecRef();
+  delete[] Filename;
 
   Filename = csStrNew(fName);
   VFS = vfs;
-  if (VFS) VFS->IncRef();
   Dirty = true;
 }
 
@@ -444,9 +442,14 @@ bool csConfigFile::Load(const char* fName, iVFS *vfs, bool Merge, bool NewWins)
 
 bool csConfigFile::Save()
 {
-  if (!Dirty) return true;
+  if (!Dirty)
+    return true;
+  
+  if (!SaveNow(Filename, VFS))
+    return false;
+  
   Dirty = false;
-  return SaveNow(Filename, VFS);
+  return true;
 }
 
 bool csConfigFile::Save(const char *file, iVFS *vfs)
@@ -459,17 +462,20 @@ bool csConfigFile::Save(const char *file, iVFS *vfs)
   if (Filename && strcmp(Filename, file)==0 && VFS==vfs)
   {
     if (!Dirty) return true;
-    Dirty = false;
   }
 
-  return SaveNow(file, vfs);
+  if (!SaveNow(file, vfs))
+    return false;
+
+  Dirty = false;
+  return true;
 }
 
 void WriteComment(csString &Filedata, const char *s)
 {
   if (s != 0)
   {
-    for (const char* b = strchr(s,'\n'); b != NULL; b = strchr(s,'\n'))
+    for (const char* b = strchr(s,'\n'); b != 0; b = strchr(s,'\n'))
     {
       if (*s != '\n' && *s != ';') // Prepend comment character if absent.
         Filedata << "; ";
@@ -494,10 +500,10 @@ bool csConfigFile::SaveNow(const char *file, iVFS *vfs) const
   csString Filedata;
   csConfigNode *n;
 
-  for (n = FirstNode; n != NULL; n = n->GetNext())
+  for (n = FirstNode; n != 0; n = n->GetNext())
   {
     // don't write first and last nodes
-    if (n->GetName() == NULL) continue;
+    if (n->GetName() == 0) continue;
 
     // write comment
     WriteComment(Filedata, n->GetComment());
@@ -529,22 +535,22 @@ void csConfigFile::Clear()
   // rewind all iterators
   for (long i = 0; i < Iterators->Length(); i++)
   {
-    csConfigIterator *it = (csConfigIterator*)Iterators->Get(i);
+    csConfigIterator *it = Iterators->Get(i);
     it->Rewind();
   }
   if (EOFComment)
   {
     delete[] EOFComment;
-    EOFComment = NULL;
+    EOFComment = 0;
   }
   Dirty = true;
 }
 
-iConfigIterator *csConfigFile::Enumerate(const char *Subsection)
+csPtr<iConfigIterator> csConfigFile::Enumerate(const char *Subsection)
 {
-  iConfigIterator *it = new csConfigIterator(this, Subsection);
+  csConfigIterator *it = new csConfigIterator(this, Subsection);
   Iterators->Push(it);
-  return it;
+  return csPtr<iConfigIterator> (it);
 }
 
 bool csConfigFile::KeyExists(const char *Key) const
@@ -584,7 +590,7 @@ bool csConfigFile::GetBool(const char *Key, bool Def) const
 const char *csConfigFile::GetComment(const char *Key) const
 {
   csConfigNode *Node = FindNode(Key);
-  return Node ? Node->GetComment() : NULL;
+  return Node ? Node->GetComment() : 0;
 }
 
 void csConfigFile::SetStr (const char *Key, const char *Val)
@@ -672,7 +678,7 @@ void csConfigFile::DeleteKey(const char *Name)
 bool csConfigFile::LoadNow(const char *fName, iVFS *vfs, bool overwrite)
 {
   // load the file buffer
-  iDataBuffer *Filedata;
+  csRef<iDataBuffer> Filedata;
   if (vfs)
   {
     Filedata = vfs->ReadFile(fName);
@@ -685,7 +691,7 @@ bool csConfigFile::LoadNow(const char *fName, iVFS *vfs, bool overwrite)
     fseek(fp, 0, SEEK_END);
     size_t Size = ftell(fp);
     fseek(fp, 0, SEEK_SET);
-    Filedata = new csDataBuffer(Size + 1);
+    Filedata = csPtr<iDataBuffer> (new csDataBuffer(Size + 1));
     fread(Filedata->GetData(), sizeof(char), Size, fp);
     fclose(fp);
     Filedata->GetInt8()[Size] = 0;
@@ -693,7 +699,6 @@ bool csConfigFile::LoadNow(const char *fName, iVFS *vfs, bool overwrite)
 
   // parse the data
   LoadFromBuffer((char*)Filedata->GetInt8(), overwrite);
-  Filedata->DecRef();
 
   return true;
 }
@@ -779,10 +784,10 @@ void csConfigFile::LoadFromBuffer(char *Filedata, bool overwrite)
 
 csConfigNode *csConfigFile::FindNode(const char *Name, bool isSubsection) const
 {
-  if (!Name) return NULL;
+  if (!Name) return 0;
 
   csConfigNode *n = FirstNode;
-  const int sz = (isSubsection ? strlen(Name) : 0);
+  const size_t sz = (isSubsection ? strlen(Name) : 0);
   while (n)
   {
     const char* s = n->GetName();
@@ -791,12 +796,12 @@ csConfigNode *csConfigFile::FindNode(const char *Name, bool isSubsection) const
       return n;
     n = n->GetNext();
   }
-  return NULL;
+  return 0;
 }
 
 csConfigNode *csConfigFile::CreateNode(const char *Name)
 {
-  if (!Name) return NULL;
+  if (!Name) return 0;
 
   csConfigNode *n = new csConfigNode(Name);
   n->InsertAfter(LastNode->GetPrev());
@@ -808,13 +813,13 @@ void csConfigFile::RemoveIterator(csConfigIterator *it) const
 {
   int n = Iterators->Find(it);
   CS_ASSERT(n != -1);
-  Iterators->Delete(n);
+  Iterators->DeleteIndex(n);
 }
 
 void csConfigFile::SetEOFComment(const char *text)
 {
-  if (EOFComment) delete[] EOFComment;
-  EOFComment = (text ? csStrNew(text) : NULL);
+  delete[] EOFComment;
+  EOFComment = (text ? csStrNew(text) : 0);
   Dirty = true;
 }
 
