@@ -17,14 +17,10 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+
 #include "cssysdef.h"
 #include "csutil/databuf.h"
 
 SCF_IMPLEMENT_IBASE (csDataBuffer)
   SCF_IMPLEMENTS_INTERFACE (iDataBuffer);
 SCF_IMPLEMENT_IBASE_END
-
-csDataBuffer::~csDataBuffer ()
-{
-  delete [] Data;
-}
