@@ -19,31 +19,54 @@
 #include "cssysdef.h"
 #include "csutil/objiter.h"
 
-csTypedObjectIterator::csTypedObjectIterator (iObject *Parent)
+csTypedObjectIterator::csTypedObjectIterator (iObject *Parent) : fetched(false)
 {
   iter = Parent->GetIterator ();
-  CurrentTypedObject = NULL;
 }
 
 csTypedObjectIterator::~csTypedObjectIterator ()
 {
-  iter->DecRef ();
+}
+
+iBase* csTypedObjectIterator::GetCurrentObject()
+{
+  if (!fetched)
+    FetchObject();
+  return CurrentTypedObject;
 }
 
 void csTypedObjectIterator::FetchObject ()
 {
-  if (CurrentTypedObject)
-  {
-    CurrentTypedObject->DecRef ();
-    CurrentTypedObject = NULL;
-  }
-  if (iter->IsFinished ())
+  fetched = true;
+  CurrentTypedObject = 0;
+  if (!iter->HasNext ())
     return;
 
   scfInterfaceID id;
   int ver;
   GetRequestedInterface (id, ver);
 
-  CurrentTypedObject = (iBase*)(iter->GetObject ()->QueryInterface (id, ver));
-  if (!CurrentTypedObject) Next ();
+  while (iter->HasNext ())
+  {
+    CurrentTypedObject = csPtr<iBase> (
+  	(iBase*)(iter->Next ()->QueryInterface (id, ver)));
+    if (CurrentTypedObject) return;
+  }
 }
+
+iBase* csTypedObjectIterator::FindName (const char* name)
+{
+  iObject* obj = iter->FindName (name);
+  if (obj)
+  {
+    scfInterfaceID id;
+    int ver;
+    GetRequestedInterface (id, ver);
+    CurrentTypedObject = csPtr<iBase> (
+  	(iBase*)(obj->QueryInterface (id, ver)));
+  }
+  else
+    CurrentTypedObject = 0;
+  return CurrentTypedObject;
+}
+
