@@ -62,13 +62,15 @@
  * <panos@alumni.cs.colorado.edu> for xinetd.
  */
 
+#include "cssysdef.h"
+#include "csutil/snprintf.h"
 // This MUST go before the ctype includes!
 #ifdef WIN32
 #define _CTYPE_DISABLE_MACROS
 #endif
 
 #include <stdio.h>
-#include "csutil/csctype.h"
+#include <ctype.h>
 #ifndef NETWARE
 #include <sys/types.h>
 #endif
@@ -135,8 +137,12 @@ typedef int bool_int;
 
 #define	NDIG	80
 
+#ifndef ULONG_MAX
 #define ULONG_MAX (unsigned long)0xffffffffl
+#endif
+#ifndef LONG_MAX
 #define LONG_MAX (signed long)0xefffffffl
+#endif
 
 
 // Moved here by CN from apache header
@@ -321,7 +327,7 @@ static char *ap_gcvt(double number, int ndigit, char *buf, boolean_e altform)
 
 #define STR_TO_DEC( str, num )		\
     num = NUM( *str++ ) ;		\
-    while ( cs_isdigit( *str ) )		\
+    while ( isdigit( *str ) )		\
     {					\
 	num *= 10 ;			\
 	num += NUM( *str++ ) ;		\
@@ -543,7 +549,7 @@ static char *conv_fp(register char format, register double num,
     /*
      * Check for Infinity and NaN
      */
-    if (cs_isalpha(*p)) {
+    if (isalpha(*p)) {
 	*len = strlen(strcpy(buf, p));
 	*is_negative = FALSE;
 	return (buf);
@@ -677,7 +683,7 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
     register int cc = 0;
     register int i;
 
-    register char *s = NULL;
+    register char *s = 0;
     char *q;
     int s_len = 0;
 
@@ -734,7 +740,7 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
 	    /*
 	     * Try to avoid checking for flags, width or precision
 	     */
-	    if (!cs_islower(*fmt)) {
+	    if (!islower(*fmt)) {
 		/*
 		 * Recognize flags: -, #, BLANK, +
 		 */
@@ -756,7 +762,7 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
 		/*
 		 * Check if a width was specified
 		 */
-		if (cs_isdigit(*fmt)) {
+		if (isdigit(*fmt)) {
 		    STR_TO_DEC(fmt, min_width);
 		    adjust_width = YES;
 		}
@@ -782,7 +788,7 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
 		if (*fmt == '.') {
 		    adjust_precision = YES;
 		    fmt++;
-		    if (cs_isdigit(*fmt)) {
+		    if (isdigit(*fmt)) {
 			STR_TO_DEC(fmt, precision);
 		    }
 		    else if (*fmt == '*') {
@@ -930,7 +936,7 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
 
 	    case 's':
 		s = va_arg(ap, char *);
-		if (s != NULL) {
+		if (s != 0) {
 		    s_len = strlen(s);
 		    if (adjust_precision && precision < s_len)
 			s_len = precision;
@@ -998,11 +1004,11 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
 
 		s_len = strlen(s);
 
-		if (alternate_form && (q = strchr(s, '.')) == NULL) {
+		if (alternate_form && (q = strchr(s, '.')) == 0) {
 		    s[s_len++] = '.';
 		    s[s_len] = '\0'; /* delimit for following strchr() */
 		}
-		if (*fmt == 'G' && (q = strchr(s, 'e')) != NULL)
+		if (*fmt == 'G' && (q = strchr(s, 'e')) != 0)
 		    *q = 'E';
 		break;
 
@@ -1074,7 +1080,7 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
 			struct sockaddr_in *si;
 
 			si = va_arg(ap, struct sockaddr_in *);
-			if (si != NULL) {
+			if (si != 0) {
 			    s = conv_sockaddr_in(si, &num_buf[NUM_BUF_SIZE], &s_len);
 			    if (adjust_precision && precision < s_len)
 				s_len = precision;
@@ -1093,7 +1099,7 @@ int ap_vformatter(int (*flush_func)(ap_vformatter_buff *),
 			struct in_addr *ia;
 
 			ia = va_arg(ap, struct in_addr *);
-			if (ia != NULL) {
+			if (ia != 0) {
 			    s = conv_in_addr(ia, &num_buf[NUM_BUF_SIZE], &s_len);
 			    if (adjust_precision && precision < s_len)
 				s_len = precision;
