@@ -22,6 +22,7 @@
 #include "csws/cswindow.h"
 #include "csws/csgrid.h"
 #include "csws/cssplit.h"
+#include "csutil/event.h"
 
 // The minimal width of the new grid view when we split a view horiz. or vert.
 #define MIN_GRIDVIEW_SIZE	8
@@ -34,11 +35,11 @@
 csRegionTree2D::csRegionTree2D ()
 {
   region.MakeEmpty ();
-  data = NULL;
+  data = 0;
   memset (children, 0, 5 * sizeof (csRegionTree2D *));
 }
 
-csRegionTree2D::csRegionTree2D (csRect area, csSome data)
+csRegionTree2D::csRegionTree2D (csRect area, void* data)
 {
   region.Set (area);
   csRegionTree2D::data = data;
@@ -55,7 +56,7 @@ csRegionTree2D::~csRegionTree2D ()
 /**
  * Tiles this rect into the tree and creates new children if needed.
  */
-void csRegionTree2D::Insert (csRect &area, csSome data)
+void csRegionTree2D::Insert (csRect &area, void* data)
 {
   if (children [0])
   {
@@ -120,7 +121,8 @@ void csRegionTree2D::Insert (csRect &area, csSome data)
 /**
  * Returns a list of leaves that do all contain parts of area.
  */
-void csRegionTree2D::FindRegion (const csRect &area, csVector &vLeafList)
+void csRegionTree2D::FindRegion (const csRect &area,
+	csArray<csRegionTree2D*> &vLeafList)
 {
   if (children [0])
   {
@@ -135,7 +137,7 @@ void csRegionTree2D::FindRegion (const csRect &area, csVector &vLeafList)
 /**
  * Traverse the tree and call user supplied function for every node.
  */
-void csRegionTree2D::Traverse (csRegionTreeFunc userFunc, csSome databag)
+void csRegionTree2D::Traverse (csRegionTreeFunc userFunc, void* databag)
 {
   if (userFunc (this, databag))
   {
@@ -154,54 +156,35 @@ csSparseGrid::csGridRow::csGridRow (int theCol)
   col = theCol;
 }
 
-csSparseGrid::csGridRow::~csGridRow ()
+void csSparseGrid::csGridRow::SetAt (int col, void* data)
 {
-  DeleteAll ();
-}
-
-void csSparseGrid::csGridRow::SetAt (int col, csSome data)
-{
-  int key = FindSortedKey ((csConstSome)col);
+  int key = FindSortedKey (KeyCmp(col));
   if (key == -1 && data)
-    key = InsertSorted (new csGridRowEntry (col, data));
+    key = InsertSorted (new csGridRowEntry(col, data), Compare);
   else
     if (data)
       Get (key)->data = data;
     else
-      Delete (key);
+      DeleteIndex (key);
 }
 
-csSparseGrid::csGridRowEntry* csSparseGrid::csGridRow::Get (int index)
+int csSparseGrid::csGridRow::Compare (csGridRowEntry* const& Item1,
+				      csGridRowEntry* const& Item2)
 {
-  return (csSparseGrid::csGridRowEntry*)csVector::Get (index);
+  return (Item1->col < Item2->col ? -1 : Item1->col > Item2->col ? 1 : 0);
 }
 
-int csSparseGrid::csGridRow::Compare (csSome Item1, csSome Item2, int Mode) const
+int csSparseGrid::csGridRow::CompareKey (csGridRowEntry* const& Item1,
+					 int const& Key)
 {
-  (void)Mode;
-  csSparseGrid::csGridRowEntry *e1 = (csSparseGrid::csGridRowEntry*)Item1;
-  csSparseGrid::csGridRowEntry *e2 = (csSparseGrid::csGridRowEntry*)Item2;
-  return (e1->col < e2->col ? -1 : e1->col > e2->col ? 1 : 0);
-}
-
-int csSparseGrid::csGridRow::CompareKey (csSome Item1, csConstSome Key, int Mode) const
-{
-  (void)Mode;
-  csSparseGrid::csGridRowEntry *e1 = (csSparseGrid::csGridRowEntry*)Item1;
-  return (e1->col < (int)Key ? -1 : e1->col > (int)Key ? 1 : 0);
-}
-
-bool csSparseGrid::csGridRow::FreeItem (csSome Item)
-{
-  delete (csSparseGrid::csGridRowEntry*)Item;
-  return true;
+  return (Item1->col < Key ? -1 : Item1->col > Key ? 1 : 0);
 }
 
 /******************************************************************************
  * csGridCell
  ******************************************************************************/
 
-csGridCell::csGridCell () : csComponent (NULL), inUse (false)
+csGridCell::csGridCell () : csComponent (0), inUse (false)
 {
   state |= CSS_SELECTABLE;
   valuePattern = "%s";
@@ -319,12 +302,12 @@ csGridView::csGridView (csGrid *pParent, const csRect& region, int iStyle)
   if (Style & CSGVS_HSCROLL)
     hscroll = new csScrollBar (this, cssfsThinRect);
   else
-    hscroll = NULL;
+    hscroll = 0;
 
   if (Style & CSGVS_VSCROLL)
     vscroll = new csScrollBar (this, cssfsThinRect);
   else
-    vscroll = NULL;
+    vscroll = 0;
   SetPalette (CSPAL_GRIDVIEW);
   col = area.xmin;
   row = area.ymin;
@@ -340,12 +323,12 @@ csGridView::csGridView (const csGridView& view, int iStyle)
   if (Style & CSGVS_HSCROLL)
     hscroll = new csScrollBar (this, cssfsThinRect);
   else
-    hscroll = NULL;
+    hscroll = 0;
 
   if (Style & CSGVS_VSCROLL)
     vscroll = new csScrollBar (this, cssfsThinRect);
   else
-    vscroll = NULL;
+    vscroll = 0;
   SetPalette (view.palette, view.palettesize);
   row = view.row;
   col = view.col;
@@ -377,7 +360,7 @@ void csGridView::PlaceItems ()
 
   // count the number of cells visible in the first row
   // (exact would be the minimum of cells in a row in the visible area)
-  csVector vRegionList;
+  csArray<csRegionTree2D*> vRegionList;
   csRect rc;
   int i = 0, w1 = 0, w2 = 0;
   int nRowCells = 0, nColCells = 0;
@@ -416,7 +399,7 @@ void csGridView::PlaceItems ()
     hsbstatus.pagestep = MAX (nRowCells, 1);
     hscroll->SendCommand (cscmdScrollBarSet, &hsbstatus);
 
-    vRegionList.SetLength (0);
+    vRegionList.DeleteAll ();
     i = 0; w1 = 0; w2 = 0;
   }
 
@@ -472,14 +455,14 @@ void csGridView::CooAt (int theX, int theY, int &theRow, int &theCol)
   int actCol = col;
   csRect rc;
   csRegionTree2D *r;
-  csVector vRegions;
-  csGridCell *cell = NULL;
+  csArray<csRegionTree2D*> vRegions;
+  csGridCell *cell = 0;
 
   theCol = area.xmin -1;
   theRow = area.ymin -1;
 
   rc.Set (actCol, actRow, actCol+1, area.ymax);
-  vRegions.SetLength (0);
+  vRegions.DeleteAll ();
   pGrid->regions->FindRegion (rc, vRegions);
   n = 0;
   c = actRow;
@@ -498,7 +481,7 @@ void csGridView::CooAt (int theX, int theY, int &theRow, int &theCol)
   {
     actRow--;
     rc.Set (actCol, actRow, area.xmax, actRow+1);
-    vRegions.SetLength (0);
+    vRegions.DeleteAll ();
     pGrid->regions->FindRegion (rc, vRegions);
     x = 0;
     n = 0;
@@ -532,8 +515,8 @@ void csGridView::Draw ()
   int c, actRow = row;
   csRect rc;
   csRegionTree2D *r;
-  csVector vRegions;
-  csGridCell *cell = NULL;
+  csArray<csRegionTree2D*> vRegions;
+  csGridCell *cell = 0;
   int cs = pGrid->GetCursorStyle ();
   int cr, cc;
   bool sel;
@@ -543,7 +526,7 @@ void csGridView::Draw ()
   while (y < bound.Height ()-GRIDVIEW_BORDER_SIZE && actRow < area.ymax)
   {
     rc.Set (col, actRow, area.xmax, actRow + 1);
-    vRegions.SetLength (0);
+    vRegions.DeleteAll ();
     pGrid->regions->FindRegion (rc, vRegions);
     if (vRegions.Length () == 0)
       break; // no more rows to draw
@@ -566,7 +549,7 @@ void csGridView::Draw ()
 	  (cs == CSGCS_COLUMN && c == cc);
 	cell->SetState (CSS_GRIDCELL_SELECTED, sel);
         cell->Draw ();
-        cell->ForEach (DrawCellComponents, NULL, true);
+        cell->ForEach (DrawCellComponents, 0, true);
         x += cell->bound.Width ();
       }
       Delete (cell);
@@ -675,7 +658,7 @@ csGridView *csGridView::CreateCopy (int iStyle)
 
 csGridView *csGridView::SplitX (int x, int iStyle)
 {
-  csGridView *sp = NULL;
+  csGridView *sp = 0;
   if (x > MIN_GRIDVIEW_SIZE && x < bound.Width () - MIN_GRIDVIEW_SIZE)
   {
     sp = CreateCopy (iStyle);
@@ -693,7 +676,7 @@ csGridView *csGridView::SplitX (int x, int iStyle)
 
 csGridView *csGridView::SplitY (int y, int iStyle)
 {
-  csGridView *sp = NULL;
+  csGridView *sp = 0;
   if (y > MIN_GRIDVIEW_SIZE && y < bound.Height () - MIN_GRIDVIEW_SIZE)
   {
     sp = CreateCopy (iStyle);
@@ -739,10 +722,10 @@ void csGrid::init (csComponent *pParent, csRect &rc, int iStyle, csGridCell *gc)
   SetState (CSS_SELECTABLE, true);
   vRegionStyles.Push (gc);
   vViews.Push (new csGridView (this, rc, (iStyle & ~(CSGS_HSPLIT|CSGS_VSPLIT))));
-  regions = new csRegionTree2D (rc, vRegionStyles.Get (0) );
+  regions = new csRegionTree2D (rc, vRegionStyles[0] );
   // rc below is a dummy and will be recalculated when SetRect is called
-  viewlayout = new csRegionTree2D (rc, vViews.Get (0) );
-  splitterX = splitterY = NULL;
+  viewlayout = new csRegionTree2D (rc, vViews[0] );
+  splitterX = splitterY = 0;
   if (iStyle & CSGS_HSPLIT)
     splitterX = new csSplitter (this);
   if (iStyle & CSGS_VSPLIT)
@@ -762,7 +745,7 @@ csGrid::~csGrid ()
     for (j = 0; j < r->Length (); j++)
     {
       csString *str = (csString*)r->Get (j)->data;
-      if (str) delete str;
+      delete str;
     }
     delete r;
   }
@@ -771,8 +754,10 @@ csGrid::~csGrid ()
   delete regions;
 
   for (i = 0; i < vRegionStyles.Length (); i++)
-    delete (csGridCell *)vRegionStyles.Get (i);
+    delete vRegionStyles[i];
   //for (i=0; i<vViews.Length (); i++) delete (csGridView*)vViews.Get (i);
+
+  delete viewlayout;
 }
 
 void csGrid::Draw ()
@@ -796,7 +781,7 @@ bool csGrid::HandleEvent (iEvent &Event)
           int x, y;
           sl->GetPos (x, y);
           csRect rc (x, y, x + 1, y + 1);
-          csVector vSpl;
+          csArray<csRegionTree2D*> vSpl;
           viewlayout->FindRegion (rc, vSpl);
           if (vSpl.Length () == 1)
           {
@@ -813,45 +798,48 @@ bool csGrid::HandleEvent (iEvent &Event)
         break;
       }
       break;
-  case csevKeyDown:
-    switch (Event.Key.Code)
+  case csevKeyboard:
+    if (csKeyEventHelper::GetEventType (&Event) == csKeyEventTypeDown)
     {
-    case CSKEY_DOWN:
-      if (GetCursorStyle () != CSGCS_NONE)
+      switch (csKeyEventHelper::GetCookedCode (&Event))
       {
-	const csRect &rc = GetRootView ()->GetArea ();
-	if (ycur < rc.ymax)
-	  SetCursorPos (ycur+1, xcur);
-	return true;
+      case CSKEY_DOWN:
+	if (GetCursorStyle () != CSGCS_NONE)
+	{
+	  const csRect &rc = GetRootView ()->GetArea ();
+	  if (ycur < rc.ymax)
+	    SetCursorPos (ycur+1, xcur);
+	  return true;
+	}
+	break;
+      case CSKEY_UP:
+	if (GetCursorStyle () != CSGCS_NONE)
+	{
+	  const csRect &rc = GetRootView ()->GetArea ();
+	  if (ycur > rc.ymin)
+	    SetCursorPos (ycur-1, xcur);
+	  return true;
+	}
+	break;
+      case CSKEY_LEFT:
+	if (GetCursorStyle () != CSGCS_NONE)
+	{
+	  const csRect &rc = GetRootView ()->GetArea ();
+	  if (xcur > rc.xmin)
+	    SetCursorPos (ycur, xcur-1);
+	  return true;
+	}
+	break;
+      case CSKEY_RIGHT:
+	if (GetCursorStyle () != CSGCS_NONE)
+	{
+	  const csRect &rc = GetRootView ()->GetArea ();
+	  if (xcur < rc.xmax)
+	    SetCursorPos (ycur, xcur+1);
+	  return true;
+	}
+	break;
       }
-      break;
-    case CSKEY_UP:
-      if (GetCursorStyle () != CSGCS_NONE)
-      {
-	const csRect &rc = GetRootView ()->GetArea ();
-	if (ycur > rc.ymin)
-	  SetCursorPos (ycur-1, xcur);
-	return true;
-      }
-      break;
-    case CSKEY_LEFT:
-      if (GetCursorStyle () != CSGCS_NONE)
-      {
-	const csRect &rc = GetRootView ()->GetArea ();
-	if (xcur > rc.xmin)
-	  SetCursorPos (ycur, xcur-1);
-	return true;
-      }
-      break;
-    case CSKEY_RIGHT:
-      if (GetCursorStyle () != CSGCS_NONE)
-      {
-	const csRect &rc = GetRootView ()->GetArea ();
-	if (xcur < rc.xmax)
-	  SetCursorPos (ycur, xcur+1);
-	return true;
-      }
-      break;
     }
   }
   return csComponent::HandleEvent (Event);
@@ -862,10 +850,10 @@ bool csGrid::HandleEvent (iEvent &Event)
  * Resize views proportional to the new size of csGrid.
  * @@@ TODO: TAKE MINIMUM SIZE INTO ACCOUNT
  */
-static bool ResizeViews (csSome node, csSome /*databag*/)
+static bool ResizeViews (void* node, void* /*databag*/)
 {
   csRegionTree2D *t = (csRegionTree2D*)node;
-  if (t->children[0] == NULL)
+  if (t->children[0] == 0)
   {
     // leaf - we find the new size in the region variable
     ((csGridView*)t->data)->SetRect (t->region.xmin, t->region.ymin,
@@ -903,7 +891,7 @@ static bool ResizeViews (csSome node, csSome /*databag*/)
  */
 void csGrid::CalcMinimalSize (csRegionTree2D *node, int &w, int &h)
 {
-  if (node->children [0] == NULL)
+  if (node->children [0] == 0)
   {
     // leaf
     ((csGridView*)node->data)->SuggestSize (w, h);
@@ -979,7 +967,7 @@ void csGrid::SetStringAt (int row, int col, const char *data)
     else if (!data)
     {
       delete str;
-      grid->SetAt (row, col, NULL);
+      grid->SetAt (row, col, 0);
     }
     else
       str->Truncate (0).Append (data);
@@ -1024,7 +1012,7 @@ void csGrid::SetCursorPos (int row, int col)
   {
     ycur = row;
     xcur = col;
-    if (parent) parent->SendCommand (cscmdGridCursorChanged, (csSome)this);
+    if (parent) parent->SendCommand (cscmdGridCursorChanged, (void*)this);
   }
 }
 
