@@ -16,14 +16,16 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-
-#define CS_SYSDEF_PROVIDE_CASE
 #include "cssysdef.h"
+
+#include <ctype.h>
+
 #include "csws/cslistbx.h"
 #include "csws/cstimer.h"
 #include "csws/csscrbar.h"
 #include "csws/csapp.h"
 #include "csws/csskin.h"
+#include "csutil/event.h"
 
 // Amount of space at left and at right of each listbox item
 #define LISTBOXITEM_XSPACE              2
@@ -38,7 +40,7 @@
 
 csListBoxItem::csListBoxItem (csComponent *iParent, const char *iText, int iID,
   csListBoxItemStyle iStyle) : csComponent (iParent),
-    ItemStyle(iStyle), deltax(0),  ItemBitmap(NULL),  DeleteBitmap(false),
+    ItemStyle(iStyle), deltax(0),  ItemBitmap(0),  DeleteBitmap(false),
     hOffset(0)
 {
   state |= CSS_SELECTABLE | CSS_TRANSPARENT;
@@ -190,7 +192,7 @@ csListBox::csListBox (csComponent *iParent, int iStyle,
   csListBoxFrameStyle iFrameStyle) : csComponent (iParent),
     ListBoxStyle(iStyle),  FrameStyle(iFrameStyle),
     deltax(0), fPlaceItems(false),
-    FrameBitmap(NULL), fDelFrameBitmap(false), FrameAlpha(0)
+    FrameBitmap(0), fDelFrameBitmap(false), FrameAlpha(0)
 
 {
   state |= CSS_SELECTABLE;
@@ -229,11 +231,11 @@ csListBox::csListBox (csComponent *iParent, int iStyle,
   if (iStyle & CSLBS_HSCROLL)
     hscroll = new csScrollBar (this, sbsty);
   else
-    hscroll = NULL;
+    hscroll = 0;
   if (iStyle & CSLBS_VSCROLL)
     vscroll = new csScrollBar (this, sbsty);
   else
-    vscroll = NULL;
+    vscroll = 0;
 
 
    ApplySkin (GetSkin ());
@@ -259,7 +261,7 @@ void csListBox::PlaceItems (bool setscrollbars)
     do
     {
       cur = cur->next;
-      if (unsigned (cur->SendCommand (cscmdListBoxItemCheck, NULL)) == CS_LISTBOXITEMCHECK_SELECTED)
+      if (unsigned (cur->SendCommand (cscmdListBoxItemCheck, 0)) == CS_LISTBOXITEMCHECK_SELECTED)
         break;
     } while (cur != focused);
     if (cur == focused)
@@ -420,142 +422,165 @@ bool csListBox::HandleEvent (iEvent &Event)
     case csevMouseUp:
       if ((Event.Mouse.Button == 1) && (app->MouseOwner == this))
       {
-        app->CaptureMouse (NULL);
+        app->CaptureMouse (0);
         return true;
       } /* endif */
       break;
-    case csevKeyDown:
-      switch (Event.Key.Code)
+    case csevKeyboard:
+      if (csKeyEventHelper::GetEventType (&Event) == csKeyEventTypeDown)
       {
-        case CSKEY_UP:
-          if ((Event.Key.Modifiers & (CSMASK_SHIFT | CSMASK_ALT)) == 0)
-          {
-            bool mc = (Event.Key.Modifiers & CSMASK_CTRL) && !app->MouseOwner;
-            if (mc) app->CaptureMouse (this);
-            csComponent *comp = focused->prev;
-            while (mc && comp->GetState (CSS_LISTBOXITEM_SELECTED))
-              comp = comp->prev;
-            SendCommand (cscmdListBoxTrack, (void *)comp);
-            if (mc) app->CaptureMouse (NULL);
-          } /* endif */
-          return true;
-        case CSKEY_DOWN:
-          if ((Event.Key.Modifiers & (CSMASK_SHIFT | CSMASK_ALT)) == 0)
-          {
-            bool mc = (Event.Key.Modifiers & CSMASK_CTRL) && !app->MouseOwner;
-            if (mc) app->CaptureMouse (this);
-            csComponent *comp = focused->next;
-            while (mc && comp->GetState (CSS_LISTBOXITEM_SELECTED))
-              comp = comp->next;
-            SendCommand (cscmdListBoxTrack, (void *)comp);
-            if (mc) app->CaptureMouse (NULL);
-          } /* endif */
-          return true;
-        case CSKEY_LEFT:
-          if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == CSMASK_CTRL)
-          {
-            if (deltax > LISTBOX_HORIZONTAL_PAGESTEP)
-              deltax -= LISTBOX_HORIZONTAL_PAGESTEP;
-            else
-              deltax = 0;
-            PlaceItems ();
-          }
-          else if (((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == 0) && (deltax > 0))
-          {
-            deltax--;
-            PlaceItems ();
-          } /* endif */
-          return true;
-        case CSKEY_RIGHT:
-          if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == CSMASK_CTRL)
-          {
-            if (deltax + LISTBOX_HORIZONTAL_PAGESTEP <= maxdeltax)
-              deltax += LISTBOX_HORIZONTAL_PAGESTEP;
-            else
-              deltax = maxdeltax;
-            PlaceItems ();
-          }
-          else if (((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == 0) && (deltax < maxdeltax))
-          {
-            deltax++;
-            PlaceItems ();
-          } /* endif */
-          return true;
-        case CSKEY_PGUP:
-          if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == 0)
-		  {
-			int i;
-            for (i = 0; i < vertcount; i++)
-              SendCommand (cscmdListBoxTrack, (void *)focused->prev);
-		  }
-          else if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == CSMASK_CTRL)
-            SendCommand (cscmdListBoxTrack, (void *)NextChild (first));
-          return true;
-        case CSKEY_PGDN:
-          if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == 0)
-		  {
-			int i;
-            for (i = 0; i < vertcount; i++)
-              SendCommand (cscmdListBoxTrack, (void *)focused->next);
-		  }
-          else if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == CSMASK_CTRL)
-            SendCommand (cscmdListBoxTrack, (void *)PrevChild (first));
-          return true;
-        case CSKEY_HOME:
-          if ((Event.Key.Modifiers & CSMASK_CTRL) && (deltax != 0))
-          {
-            deltax = 0;
-            PlaceItems ();
-          }
-          else if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == 0)
-            SendCommand (cscmdListBoxTrack, (void *)NextChild (first));
-          return true;
-        case CSKEY_END:
-          if ((Event.Key.Modifiers & CSMASK_CTRL) && (deltax != maxdeltax))
-          {
-            deltax = maxdeltax;
-            PlaceItems ();
-          }
-          else if ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == 0)
-            SendCommand (cscmdListBoxTrack, (void *)PrevChild (first));
-          return true;
-        case '/':
-          if ((ListBoxStyle & CSLBS_MULTIPLESEL)
-           && ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == CSMASK_CTRL))
-            ForEachItem (do_select, NULL, false);
-          return true;
-        case '\\':
-          if ((ListBoxStyle & CSLBS_MULTIPLESEL)
-           && ((Event.Key.Modifiers & CSMASK_ALLSHIFTS) == CSMASK_CTRL))
-            ForEachItem (do_deselect, NULL);
-          return true;
-        default:
-          if ((Event.Key.Char >= ' ')
-           && (Event.Key.Char <= 255)
-           && !(Event.Key.Modifiers & (CSMASK_CTRL | CSMASK_ALT)))
-          {
-            // Find first next item that starts with this letter
-            csComponent *cur = focused->next;
-            while (cur != focused)
-              if (cur->SendCommand (cscmdListBoxItemCheck, NULL)
-               && (UPPERCASE (cur->GetText () [0]) == UPPERCASE (Event.Key.Char)))
-              {
-                SendCommand (cscmdListBoxTrack, (void *)cur);
-                return true;
-              }
-              else
-                cur = cur->next;
-            return true;
-          }
-      } /* endswitch */
+	switch (csKeyEventHelper::GetCookedCode (&Event))
+	{
+	  case CSKEY_UP:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      (CSMASK_SHIFT | CSMASK_ALT)) == 0)
+	    {
+	      bool mc = (csKeyEventHelper::GetModifiersBits (&Event) & 
+		CSMASK_CTRL) && !app->MouseOwner;
+	      if (mc) app->CaptureMouse (this);
+	      csComponent *comp = focused->prev;
+	      while (mc && comp->GetState (CSS_LISTBOXITEM_SELECTED))
+		comp = comp->prev;
+	      SendCommand (cscmdListBoxTrack, (void *)comp);
+	      if (mc) app->CaptureMouse (0);
+	    } /* endif */
+	    return true;
+	  case CSKEY_DOWN:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      (CSMASK_SHIFT | CSMASK_ALT)) == 0)
+	    {
+	      bool mc = (csKeyEventHelper::GetModifiersBits (&Event) & 
+		CSMASK_CTRL) && !app->MouseOwner;
+	      if (mc) app->CaptureMouse (this);
+	      csComponent *comp = focused->next;
+	      while (mc && comp->GetState (CSS_LISTBOXITEM_SELECTED))
+		comp = comp->next;
+	      SendCommand (cscmdListBoxTrack, (void *)comp);
+	      if (mc) app->CaptureMouse (0);
+	    } /* endif */
+	    return true;
+	  case CSKEY_LEFT:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == CSMASK_CTRL)
+	    {
+	      if (deltax > LISTBOX_HORIZONTAL_PAGESTEP)
+		deltax -= LISTBOX_HORIZONTAL_PAGESTEP;
+	      else
+		deltax = 0;
+	      PlaceItems ();
+	    }
+	    else if (((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == 0) && (deltax > 0))
+	    {
+	      deltax--;
+	      PlaceItems ();
+	    } /* endif */
+	    return true;
+	  case CSKEY_RIGHT:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == CSMASK_CTRL)
+	    {
+	      if (deltax + LISTBOX_HORIZONTAL_PAGESTEP <= maxdeltax)
+		deltax += LISTBOX_HORIZONTAL_PAGESTEP;
+	      else
+		deltax = maxdeltax;
+	      PlaceItems ();
+	    }
+	    else if (((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == 0) && (deltax < maxdeltax))
+	    {
+	      deltax++;
+	      PlaceItems ();
+	    } /* endif */
+	    return true;
+	  case CSKEY_PGUP:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == 0)
+		    {
+			  int i;
+	      for (i = 0; i < vertcount; i++)
+		SendCommand (cscmdListBoxTrack, (void *)focused->prev);
+		    }
+	    else if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == CSMASK_CTRL)
+	      SendCommand (cscmdListBoxTrack, (void *)NextChild (first));
+	    return true;
+	  case CSKEY_PGDN:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == 0)
+		    {
+			  int i;
+	      for (i = 0; i < vertcount; i++)
+		SendCommand (cscmdListBoxTrack, (void *)focused->next);
+		    }
+	    else if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == CSMASK_CTRL)
+	      SendCommand (cscmdListBoxTrack, (void *)PrevChild (first));
+	    return true;
+	  case CSKEY_HOME:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_CTRL) && (deltax != 0))
+	    {
+	      deltax = 0;
+	      PlaceItems ();
+	    }
+	    else if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == 0)
+	      SendCommand (cscmdListBoxTrack, (void *)NextChild (first));
+	    return true;
+	  case CSKEY_END:
+	    if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_CTRL) && (deltax != maxdeltax))
+	    {
+	      deltax = maxdeltax;
+	      PlaceItems ();
+	    }
+	    else if ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	      CSMASK_ALLSHIFTS) == 0)
+	      SendCommand (cscmdListBoxTrack, (void *)PrevChild (first));
+	    return true;
+	  case '/':
+	    if ((ListBoxStyle & CSLBS_MULTIPLESEL)
+	    && ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	    CSMASK_ALLSHIFTS) == CSMASK_CTRL))
+	      ForEachItem (do_select, 0, false);
+	    return true;
+	  case '\\':
+	    if ((ListBoxStyle & CSLBS_MULTIPLESEL)
+	    && ((csKeyEventHelper::GetModifiersBits (&Event) & 
+	    CSMASK_ALLSHIFTS) == CSMASK_CTRL))
+	      ForEachItem (do_deselect, 0);
+	    return true;
+	  default:
+	    if ((csKeyEventHelper::GetCookedCode (&Event) >= ' ')
+	    && (csKeyEventHelper::GetCookedCode (&Event) <= 255)
+	    && !(csKeyEventHelper::GetModifiersBits (&Event) & 
+	    (CSMASK_CTRL | CSMASK_ALT)))
+	    {
+	      // Find first next item that starts with this letter
+	      csComponent *cur = focused->next;
+	      while (cur != focused)
+		if (cur->SendCommand (cscmdListBoxItemCheck, 0)
+		&& (toupper (cur->GetText () [0]) == 
+		toupper (csKeyEventHelper::GetCookedCode (&Event))))
+		{
+		  SendCommand (cscmdListBoxTrack, (void *)cur);
+		  return true;
+		}
+		else
+		  cur = cur->next;
+	      return true;
+	    }
+	} /* endswitch */
+      }
       break;
     case csevCommand:
       switch (Event.Command.Code)
       {
         case cscmdListBoxClear:
           if (app->MouseOwner == this)
-            app->CaptureMouse (NULL);
-          ForEachItem (do_deleteitem, NULL, false);
+            app->CaptureMouse (0);
+          ForEachItem (do_deleteitem, 0, false);
           firstvisible = first;
           return true;
         case cscmdListBoxItemSelected:
@@ -611,7 +636,7 @@ bool csListBox::HandleEvent (iEvent &Event)
           MakeItemVisible ((csComponent *)Event.Command.Info);
           return true;
         case cscmdListBoxQueryFirstSelected:
-          Event.Command.Info = ForEachItem (do_true, NULL, true);
+          Event.Command.Info = ForEachItem (do_true, 0, true);
           return true;
         case cscmdTimerPulse:
           if (app && app->MouseOwner == this)
@@ -649,7 +674,7 @@ bool csListBox::HandleEvent (iEvent &Event)
             csComponent *cur = first;
             do
             {
-              if (cur->SendCommand (cscmdListBoxItemCheck, NULL))
+              if (cur->SendCommand (cscmdListBoxItemCheck, 0))
               {
                 if (sbs.value == 0)
                 {
@@ -715,7 +740,7 @@ csComponent *csListBox::ForEachItem (bool (*func) (csComponent *child,
   void *param), void *param, bool iSelected)
 {
   if (!func)
-    return NULL;
+    return 0;
 
   csComponent *start = first;
   csComponent *cur = start;
@@ -723,7 +748,7 @@ csComponent *csListBox::ForEachItem (bool (*func) (csComponent *child,
   {
     csComponent *next = cur->next;
 
-    unsigned reply = (long)cur->SendCommand (cscmdListBoxItemCheck, NULL);
+    unsigned reply = (long)cur->SendCommand (cscmdListBoxItemCheck, 0);
     bool ok;
     if (iSelected)
       ok = (reply == CS_LISTBOXITEMCHECK_SELECTED);
@@ -734,7 +759,7 @@ csComponent *csListBox::ForEachItem (bool (*func) (csComponent *child,
     if ((cur == next) || ((cur = next) == start))
       break;
   } /* endwhile */
-  return NULL;
+  return 0;
 }
 
 void csListBox::SetState (int mask, bool enable)
@@ -758,7 +783,7 @@ bool csListBox::SetFocused (csComponent *comp)
   if (parent)
   {
     // Check if it is for real a list box item
-    unsigned rc = (unsigned)comp->SendCommand (cscmdListBoxItemCheck, NULL);
+    unsigned rc = (unsigned)comp->SendCommand (cscmdListBoxItemCheck, 0);
     if (rc == CS_LISTBOXITEMCHECK_SELECTED
      || rc == CS_LISTBOXITEMCHECK_UNSELECTED)
       parent->SendCommand (cscmdListBoxItemFocused, comp);
