@@ -4,6 +4,8 @@
 #include "awskcfct.h"
 #include "awsslot.h"
 #include "awsscrbr.h"
+#include "awsfparm.h"
+
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/fontserv.h"
@@ -27,66 +29,54 @@ const int awsNotebook::fsFlat = 4;
 const int awsNotebook::fsNone = 5;
 
 awsNotebook::awsNotebook () :
-  tex(NULL),
-  frame_style(0),
   bb_location(nbTop),
   bb_style(nbSlide),
   maxheight (0),
-  bb(NULL),
-  alpha_level(92)
+  sink()
 {
+  sink.SetParm (this);
   SetFlag (AWSF_CMP_ALWAYSERASE);
 }
 
 awsNotebook::~awsNotebook ()
-{
-  SCF_DEC_REF (sink);
-}
+{}
 
-char *awsNotebook::Type ()
+const char *awsNotebook::Type ()
 {
   return "Notebook";
 }
 
-bool awsNotebook::Setup (iAws *_wmgr, awsComponentNode *settings)
+bool awsNotebook::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
-  if (!awsComponent::Setup (_wmgr, settings)) return false;
+  if (!awsPanel::Setup (_wmgr, settings)) return false;
 
   iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
 
-  pm->LookupIntKey ("OverlayTextureAlpha", alpha_level);  // global get
-  pm->GetInt (settings, "Alpha", alpha_level);            // local overrides, if present.
-  pm->GetInt (settings, "Style", frame_style);
   pm->GetInt (settings, "Location", bb_location);
   pm->GetInt (settings, "Mode", bb_style);
 
-  tex = pm->GetTexture ("Texture");
-
-  sink = new awsSink (this);
-  bb = new awsNotebookButtonBar;
+  sink.RegisterTrigger("ActivateTab", &OnActivateTab);
+  sink.RegisterTrigger("DeactivateTab", &OnDeactivateTab);
 
   awsKeyFactory info;
 
-  info.Initialize (new scfString ("ButtonBar"), new scfString ("Notebook ButtonBar"));
+  info.Initialize ("ButtonBar", "Notebook ButtonBar");
 
   csRect r(0, 0, Frame().Width (), 20);
 
-  bb->SetWindow (Window ());
-  bb->SetParent (this);
-  bb->Frame () = r;
-  awsComponent::AddChild (bb);
-  bb->Setup (_wmgr, info.GetThisNode ());
-  bb->SetTopBottom (bb_location == awsNotebook::nbTop);
+  tab_ctrl.SetParent (this);
+  tab_ctrl.Setup (_wmgr, info.GetThisNode ());
+  tab_ctrl.ResizeTo(r);
+  tab_ctrl.SetFlag(AWSF_CMP_NON_CLIENT);
+  awsComponent::AddChild (&tab_ctrl);
+  tab_ctrl.SetTopBottom (bb_location == awsNotebook::nbTop);
   return true;
 }
 
-bool awsNotebook::GetProperty (char *name, void **parm)
+bool awsNotebook::GetProperty (const char *name, void **parm)
 {
-  if (awsComponent::GetProperty (name, parm)) return true;
+  if (awsPanel::GetProperty (name, parm)) return true;
 
-  if (strcmp ("Style", name) == 0)
-    *parm = (void *) &frame_style;
-  else
   if (strcmp ("Location", name) == 0)
     *parm = (void *) &bb_location;
   else
@@ -98,31 +88,11 @@ bool awsNotebook::GetProperty (char *name, void **parm)
   return true;
 }
 
-bool awsNotebook::SetProperty (char *name, void *parm)
+bool awsNotebook::SetProperty (const char *name, void *parm)
 {
-  if (awsComponent::SetProperty (name, parm)) return true;
+  if (awsPanel::SetProperty (name, parm)) return true;
 
-  if (strcmp ("Style", name) == 0)
-  {
-    int h = *(int *)parm;
-    switch (h)
-    {
-    case fsBump:
-    case fsSimple:
-    case fsRaised:
-    case fsSunken:
-    case fsFlat:
-    case fsNone:
-      if (h != frame_style)
-      {
-        frame_style = h;
-        Invalidate ();
-      }
-      return true;
-    }
-    return false;
-  }
-  else if (strcmp ("Location", name) == 0)
+  if (strcmp ("Location", name) == 0)
   {
     int h = *(int *)parm;
     if (h == awsNotebook::nbTop || h == awsNotebook::nbBottom)
@@ -130,7 +100,7 @@ bool awsNotebook::SetProperty (char *name, void *parm)
       if (bb_location != h)
       {
         bb_location = h;
-        bb->SetTopBottom (h == awsNotebook::nbTop);
+        tab_ctrl.SetTopBottom (h == awsNotebook::nbTop);
         Invalidate ();
       }
       return true;
@@ -145,7 +115,6 @@ bool awsNotebook::SetProperty (char *name, void *parm)
       if ( h != bb_style)
       {
         bb_style = h;
-        DoLayout ();
         Invalidate ();
       }
       return true;
@@ -156,28 +125,71 @@ bool awsNotebook::SetProperty (char *name, void *parm)
   return false;
 }
 
-void awsNotebook::DoLayout ()
+bool awsNotebook::Execute (const char* action, iAwsParmList* parmlist)
 {
+  if (strcmp (action, "ActivateTab") == 0) 
+  {
+    if (!parmlist) return false;
+
+    iString* comp_name = 0;
+    if (!parmlist->GetString ("Tab", &comp_name)) return false;
+    
+    iAwsComponent* comp = FindChild (comp_name->GetData ());
+    if (!comp) return false;
+
+    tab_ctrl.ActivateTab((void*) comp);
+
+    return true;
+  }
+
+  return awsComponent::Execute (action, parmlist);
 }
 
-void awsNotebook::OnDraw (csRect )
+void awsNotebook::OnDraw (csRect r)
 {
-  aws3DFrame frame3d;
-
   csRect f (Frame ());
   if (bb_location == nbTop)
-    f.ymin += bb->Frame ().Height ();
+    f.ymin += tab_ctrl.Frame ().Height ();
   else
-    f.ymax -= bb->Frame ().Height ();
-  frame3d.Draw(WindowManager(), Window(), f, frame_style, tex, alpha_level);
+    f.ymax -= tab_ctrl.Frame ().Height ();
+  
+  frame_drawer.Draw (
+      f,
+      style,
+      Window()->Frame());
 }
 
-void awsNotebook::AddChild (iAwsComponent *child, bool has_layout)
+void awsNotebook::AddChild (iAwsComponent *child)
 {
-  child->Frame ().Set (0, bb->Frame ().Height ()+2, Frame ().Width (), Frame ().Height ());
-  awsComponent::AddChild (child, has_layout);
+  child->ResizeTo (csRect(0, tab_ctrl.Frame ().Height ()+2, 
+	                      Frame ().Width (), Frame ().Height ()));
+  if (GetChildCount() > 1) // the first we keep active, others we hide
+    child->Hide();
+  awsComponent::AddChild (child);
 
-  bb->Add (child);
+  iString *str = 0;
+  child->GetProperty ("Caption", (void**)&str);
+  iAwsSource* src = tab_ctrl.AddTab (str, (void*)child);
+
+  // hook up the source so we receive the signals
+  slot.Connect(src, awsTab::signalActivateTab, &sink, sink.GetTriggerID("ActivateTab"));
+  slot.Connect(src, awsTab::signalDeactivateTab, &sink, sink.GetTriggerID("DeactivateTab"));
+}
+
+void awsNotebook::OnActivateTab(void* param, iAwsSource* src)
+{
+  iAwsComponent* child;
+  src->GetComponent()->GetProperty("User Param", (void**)&child);
+  child->Show();
+  ((awsNotebook*)param)->Invalidate();
+}
+
+void awsNotebook::OnDeactivateTab(void* param, iAwsSource* src)
+{
+  iAwsComponent* child;
+  src->GetComponent()->GetProperty("User Param", (void**)&child);
+  child->Hide();
+  ((awsNotebook*)param)->Invalidate();
 }
 
 awsNotebookFactory::awsNotebookFactory (
@@ -209,9 +221,9 @@ iAwsComponent *awsNotebookFactory::Create ()
 /********************************** Notebook Page ********************************/
 
 awsNotebookPage::awsNotebookPage ():
-  tex(NULL),
-  caption (NULL),
-  icon (NULL),
+  tex(0),
+  caption (0),
+  icon (0),
   iconalign(0)
 {
   SetFlag (AWSF_CMP_ALWAYSERASE);
@@ -219,10 +231,10 @@ awsNotebookPage::awsNotebookPage ():
 
 awsNotebookPage::~awsNotebookPage ()
 {
-  SCF_DEC_REF (caption);
+  if (caption) caption->DecRef ();
 }
 
-bool awsNotebookPage::Setup (iAws *_wmgr, awsComponentNode *settings)
+bool awsNotebookPage::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
   if (!awsComponent::Setup (_wmgr, settings)) return false;
 
@@ -236,13 +248,13 @@ bool awsNotebookPage::Setup (iAws *_wmgr, awsComponentNode *settings)
   return true;
 }
 
-bool awsNotebookPage::GetProperty (char *name, void **parm)
+bool awsNotebookPage::GetProperty (const char *name, void **parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = NULL;
+    char *st = 0;
 
     if (caption) st = caption->GetData ();
 
@@ -253,7 +265,7 @@ bool awsNotebookPage::GetProperty (char *name, void **parm)
   else
   if (strcmp ("Icon", name) == 0)
   {
-    char *st = NULL;
+    char *st = 0;
 
     if (icon) st = icon->GetData ();
 
@@ -272,7 +284,7 @@ bool awsNotebookPage::GetProperty (char *name, void **parm)
   return false;
 }
 
-bool awsNotebookPage::SetProperty (char *name, void *parm)
+bool awsNotebookPage::SetProperty (const char *name, void *parm)
 {
   if (awsComponent::SetProperty (name, parm)) return true;
   
@@ -280,12 +292,12 @@ bool awsNotebookPage::SetProperty (char *name, void *parm)
   {
     iString *s = (iString *) (parm);
 
-    SCF_DEC_REF (caption);
+    if (caption) caption->DecRef ();
 
     if (s && s->Length ())
       (caption = s)->IncRef ();
     else
-      caption = NULL;
+      caption = 0;
     Invalidate ();
 
     return true;
@@ -295,12 +307,12 @@ bool awsNotebookPage::SetProperty (char *name, void *parm)
   {
     iString *s = (iString *) (parm);
 
-    SCF_DEC_REF (icon);
+    if (icon) icon->DecRef ();
 
     if (s && s->Length ())
       (icon = s)->IncRef ();
     else
-      icon = NULL;
+      icon = 0;
 
     return true;
   }
@@ -313,7 +325,7 @@ bool awsNotebookPage::SetProperty (char *name, void *parm)
   return false;
 }
 
-char *awsNotebookPage::Type ()
+const char *awsNotebookPage::Type ()
 {
   return "Notebook Page";
 }
@@ -337,29 +349,23 @@ iAwsComponent *awsNotebookPageFactory::Create ()
 
 /*********************** awsNotebookButton ***********************/
 
-const int awsNotebookButton::signalActivateTab = 1;
-const int awsNotebookButton::iconLeft = 0;
-const int awsNotebookButton::iconRight = 1;
-const int awsNotebookButton::iconTop = 2;
-const int awsNotebookButton::iconBottom = 3;
-
 awsNotebookButton::awsNotebookButton ():
   is_active(false),
   is_first(false),
   is_top(true),
-  caption (NULL),
+  caption (0),
   captured(false),
   icon_align(0),
   alpha_level(92)
 {
-  tex[0]=tex[1]=tex[2]=NULL;
+  tex[0]=tex[1]=tex[2]=0;
 }
 
 awsNotebookButton::~awsNotebookButton ()
 {
 }
 
-bool awsNotebookButton::Setup (iAws *_wmgr, awsComponentNode *settings)
+bool awsNotebookButton::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
   if (!awsComponent::Setup (_wmgr, settings)) return false;
 
@@ -370,14 +376,14 @@ bool awsNotebookButton::Setup (iAws *_wmgr, awsComponentNode *settings)
   pm->GetInt (settings, "IconAlign", icon_align);
   pm->GetString (settings, "Caption", caption);
 
-  iString *tn = NULL;
+  iString *tn = 0;
 
   tex[0] = pm->GetTexture ("Texture");
   pm->GetString (settings, "Image", tn);
 
   if (tn) tex[1] = pm->GetTexture (tn->GetData (), tn->GetData ());
 
-  iString *in = NULL;
+  iString *in = 0;
   pm->GetString (settings, "Icon", in);
   if (in) tex[2] = pm->GetTexture (in->GetData (), in->GetData ());
 
@@ -387,12 +393,9 @@ bool awsNotebookButton::Setup (iAws *_wmgr, awsComponentNode *settings)
 void awsNotebookButton::GetClientRect (csRect &pf)
 {
   awsComponent *t = (awsComponent *)Parent ();
-  iAwsClientRect *cr = SCF_QUERY_INTERFACE (t, iAwsClientRect);
+  csRef<iAwsClientRect> cr (SCF_QUERY_INTERFACE (t, iAwsClientRect));
   if (cr)
-  {
     pf = cr->GetClientRect ();
-    cr->DecRef ();
-  }
   else
     pf = Parent ()->Frame ();
 }
@@ -400,7 +403,7 @@ void awsNotebookButton::GetClientRect (csRect &pf)
 void awsNotebookButton::OnDraw (csRect)
 {
   int tw=0, th=0, tx, ty, itx=0, ity=0;
-  int c_xmin, c_ymin, c_xmax, c_ymax;
+  csRect oldClip;
 
   iGraphics2D *g2d = WindowManager ()->G2D ();
   iGraphics3D *g3d = WindowManager ()->G3D ();
@@ -408,7 +411,8 @@ void awsNotebookButton::OnDraw (csRect)
   csRect pf;
   GetClientRect (pf);
 
-  g2d->GetClipRect (c_xmin, c_ymin, c_xmax, c_ymax);
+  g2d->GetClipRect(oldClip.xmin, oldClip.ymin, oldClip.xmax, oldClip.ymax);
+  pf.Intersect(oldClip);
   g2d->SetClipRect (pf.xmin, pf.ymin, pf.xmax, pf.ymax);
 
   iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
@@ -514,17 +518,15 @@ void awsNotebookButton::OnDraw (csRect)
 
   }
 
-  g2d->SetClipRect (c_xmin, c_ymin, c_xmax, c_ymax);
-
 }
 
-bool awsNotebookButton::GetProperty (char *name, void **parm)
+bool awsNotebookButton::GetProperty (const char *name, void **parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = NULL;
+    char *st = 0;
 
     if (caption) st = caption->GetData ();
 
@@ -535,7 +537,7 @@ bool awsNotebookButton::GetProperty (char *name, void **parm)
   return false;
 }
 
-bool awsNotebookButton::SetProperty (char *name, void *parm)
+bool awsNotebookButton::SetProperty (const char *name, void *parm)
 {
   if (awsComponent::SetProperty (name, parm)) return true;
   
@@ -543,12 +545,12 @@ bool awsNotebookButton::SetProperty (char *name, void *parm)
   {
     iString *s = (iString *) (parm);
 
-    SCF_DEC_REF (caption);
+    if (caption) caption->DecRef ();
 
     if (s && s->Length ())
       (caption = s)->IncRef ();
     else
-      caption = NULL;
+      caption = 0;
     Invalidate ();
 
     return true;
@@ -599,11 +601,6 @@ bool awsNotebookButton::HandleClick (int x, int y)
   return false;
 }
 
-csRect awsNotebookButton::getPreferredSize ()
-{
-  return getMinimumSize ();
-}
-
 csRect awsNotebookButton::getMinimumSize ()
 {
   int tw = 0, th = 0;
@@ -661,16 +658,16 @@ SCF_IMPLEMENT_EMBEDDED_IBASE_END
 const int awsNotebookButtonBar::HandleSize = 12;
 
 awsNotebookButtonBar::awsNotebookButtonBar ():
-  next(NULL),
-  prev(NULL),
-  next_slot(NULL),
-  prev_slot(NULL),
-  nextimg(NULL),
-  previmg(NULL),
+  next(0),
+  prev(0),
+  next_slot(0),
+  prev_slot(0),
+  nextimg(0),
+  previmg(0),
   first(-1),
   active(-1),
   is_top(true),
-  sink(NULL)
+  sink(0)
 {
   SCF_CONSTRUCT_EMBEDDED_IBASE(scfiAwsClientRect);
 }
@@ -684,14 +681,14 @@ awsNotebookButtonBar::~awsNotebookButtonBar ()
     next_slot->Disconnect (next, awsCmdButton::signalClicked,
                            sink, sink->GetTriggerID ("Next"));
 
-  SCF_DEC_REF (prev);
-  SCF_DEC_REF (next);
-  SCF_DEC_REF (prev_slot);
-  SCF_DEC_REF (next_slot);
-  SCF_DEC_REF (sink);
+  if (sink) sink->DecRef ();
+  if (prev_slot) prev_slot->DecRef ();
+  if (next_slot) next_slot->DecRef ();
+
+  SCF_DESTRUCT_EMBEDDED_IBASE(scfiAwsClientRect);
 }
 
-bool awsNotebookButtonBar::Setup (iAws *_wmgr, awsComponentNode *settings)
+bool awsNotebookButtonBar::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
   if (!awsComponent::Setup (_wmgr, settings)) return false;
 
@@ -701,11 +698,11 @@ bool awsNotebookButtonBar::Setup (iAws *_wmgr, awsComponentNode *settings)
 
   awsKeyFactory previnfo, nextinfo;
 
-  previnfo.Initialize (new scfString ("prev"), new scfString ("Slider Button"));
-  nextinfo.Initialize (new scfString ("next"), new scfString ("Slider Button"));
+  previnfo.Initialize ("prev", "Slider Button");
+  nextinfo.Initialize ("next", "Slider Button");
 
-  previnfo.AddIntKey (new scfString ("Style"), awsCmdButton::fsToolbar);
-  nextinfo.AddIntKey (new scfString ("Style"), awsCmdButton::fsToolbar);
+  previnfo.AddIntKey ("Style", awsCmdButton::fsToolbar);
+  nextinfo.AddIntKey ("Style", awsCmdButton::fsToolbar);
 
   nextimg = WindowManager ()->GetPrefMgr ()->GetTexture ("ScrollBarRt");
   previmg = WindowManager ()->GetPrefMgr ()->GetTexture ("ScrollBarLt");
@@ -715,13 +712,10 @@ bool awsNotebookButtonBar::Setup (iAws *_wmgr, awsComponentNode *settings)
   csRect r(0, 0, HandleSize, HandleSize);
 
   r.Move (Frame ().Width () - 2*HandleSize-1, Frame ().Height ()-HandleSize);
-  previnfo.AddRectKey (new scfString ("Frame"), r);
+  previnfo.AddRectKey ("Frame", r);
 
   r.Move (HandleSize+1, 0);
-  nextinfo.AddRectKey (new scfString ("Frame"), r);
-
-  prev->SetWindow (Window ());
-  next->SetWindow (Window ());
+  nextinfo.AddRectKey ("Frame", r);
 
   prev->SetParent (this);
   next->SetParent (this);
@@ -732,7 +726,9 @@ bool awsNotebookButtonBar::Setup (iAws *_wmgr, awsComponentNode *settings)
   prev->SetProperty ("Image", previmg);
   next->SetProperty ("Image", nextimg);
 
-  sink = new awsSink (this);
+  awsSink* _sink = new awsSink ();
+  _sink->SetParm (this);
+  sink = _sink;
 
   sink->RegisterTrigger ("Prev", &PrevClicked);
   sink->RegisterTrigger ("Next", &NextClicked);
@@ -784,23 +780,25 @@ void awsNotebookButtonBar::DoLayout ()
   {
     r.ymin = cr.ymax - r.Height ();
     r.ymax = cr.ymax;
+	// should this be -= or just = ???
+	// Noah
     cr.ymax -= (r.ymin+1);
   }
 
-  Frame () = r;
+  ResizeTo(r);
 
   for (i=first-1; i >=0 ; i--)
   {
     awsNotebookButton *btn = vTabs.Get (i)->button;
-    csRect &br =  btn->Frame ();
+    csRect br =  btn->Frame ();
     btn->Hide ();
     r.xmax = r.xmin - 1;
     r.xmin = r.xmax - br.Width ();
-    br = r;
+    btn->ResizeTo(r);
     btn->SetTop (is_top);
-    csRect o = vTabs.Get (i)->comp->Frame ();
-    vTabs.Get (i)->comp->Frame () = cr;
-    vTabs.Get (i)->comp->MoveChildren (cr.xmin - o.xmin, cr.ymin - o.ymin);
+    //csRect o = vTabs.Get (i)->comp->Frame ();
+    vTabs.Get (i)->comp->ResizeTo(cr);
+    //vTabs.Get (i)->comp->MoveChildren (cr.xmin - o.xmin, cr.ymin - o.ymin);
   }
 
   r = Frame ();
@@ -808,15 +806,17 @@ void awsNotebookButtonBar::DoLayout ()
   for (i=MAX(first,0); i < vTabs.Length (); i++)
   {
     awsNotebookButton *btn = vTabs.Get (i)->button;
-    csRect &br =  btn->Frame ();
+    csRect br =  btn->Frame ();
     r.xmax = r.xmin + br.Width ();
-    br = r;
+    btn->ResizeTo(r);
+	br = r;
     r.xmin = r.xmax+1;
     x += br.Width ();
     btn->SetTop (is_top);
-    csRect o = vTabs.Get (i)->comp->Frame ();
-    vTabs.Get (i)->comp->Frame () = cr;
-    vTabs.Get (i)->comp->MoveChildren (cr.xmin - o.xmin, cr.ymin - o.ymin);
+    //csRect o = vTabs.Get (i)->comp->Frame ();
+    vTabs.Get(i)->comp->ResizeTo(cr);
+    //vTabs.Get (i)->comp->Frame () = cr;
+    //vTabs.Get (i)->comp->MoveChildren (cr.xmin - o.xmin, cr.ymin - o.ymin);
   }
 
   if (x > Frame ().Width ())
@@ -825,10 +825,10 @@ void awsNotebookButtonBar::DoLayout ()
     r.xmin = r.xmax - 2*HandleSize-1;
     r.ymin = r.ymax - HandleSize;
     r.xmax = r.xmin + HandleSize;
-    prev->Frame () = r;
+    prev->ResizeTo(r);
     prev->Show ();
     r.Move (HandleSize+1,0);
-    next->Frame () = r;
+    next->ResizeTo(r);
     next->Show ();
   }
   else
@@ -851,12 +851,12 @@ csRect awsNotebookButtonBar::GetClientRect ()
 bool awsNotebookButtonBar::Add (iAwsComponent *comp)
 {
   // determine caption of tab
-  iString *str = NULL;
+  iString *str = 0;
 
-  comp->GetProperty ("Caption", (csSome*)&str);
+  comp->GetProperty ("Caption", (void**)&str);
   if (!str || !str->GetData ())
   {
-    SCF_DEC_REF(str);
+    if (str) str->DecRef ();
     csString theCap ("Tab ");
     theCap += vTabs.Length ()+1;
     str = new scfString ((const char*)theCap);
@@ -868,20 +868,18 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
   // initialize and setup the button
   awsKeyFactory btninfo;
 
-  str->IncRef ();
-  btninfo.Initialize (str, new scfString ("Notebook Button"));
-  btninfo.AddRectKey (new scfString ("Frame"), csRect (0, 0, Frame ().Width (), Frame ().Height ()));
+  btninfo.Initialize (str->GetData(), "Notebook Button");
+  btninfo.AddRectKey ("Frame", csRect (0, 0, Frame ().Width (), Frame ().Height ()));
 
-  iString *icon = NULL;
-  if (comp->GetProperty ("Icon", (csSome*)&icon) && icon && icon->Length ())
+  iString *icon = 0;
+  if (comp->GetProperty ("Icon", (void**)&icon) && icon && icon->Length ())
   {
-    btninfo.AddStringKey (new scfString ("Icon"), icon);
+    btninfo.AddStringKey ("Icon", icon ? icon->GetData() : "");
     int *iconalign;
-    if (comp->GetProperty ("IconAlign", (csSome*)&iconalign))
-      btninfo.AddIntKey (new scfString ("IconAlign"), *iconalign);
+    if (comp->GetProperty ("IconAlign", (void**)&iconalign))
+      btninfo.AddIntKey ("IconAlign", *iconalign);
   }
 
-  btn->SetWindow (Window ());
   btn->SetParent (this);
   btn->Setup (WindowManager (), btninfo.GetThisNode ());
   btn->SetProperty ("Caption", str);
@@ -892,9 +890,10 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
   if (r.Height () > Frame ().Height())
   {
     int delta = r.Height () - Frame ().Height ();
-    Frame ().ymax +=delta;
+    Resize(Frame().Width(), Frame().Height() + delta);
   }
-  btn->Frame () = r;
+  //btn->Frame () = r;
+  btn->ResizeTo(r);
 
   if (last>0)
   {
@@ -919,7 +918,9 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
   awsSlot *slot = new awsSlot;
   slot->Connect (btn, awsNotebookButton::signalActivateTab, sink, sink->GetTriggerID ("ActivateTab"));
   vTabs.Push (btn, slot, comp, sink);
-  comp->Frame ().ymin = Frame ().ymax+1;
+  //csRect newFrame(Frame());
+  //newFrame.ymin = newFrame.ymax+1;
+  //ResizeTo(newFrame);
   DoLayout ();
 
   btn->Invalidate ();
@@ -929,7 +930,7 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
 
 bool awsNotebookButtonBar::Remove (iAwsComponent *comp)
 {
-  int idx = vTabs.Find ((csSome)comp);
+  int idx = vTabs.Find ((awsNotebookButtonBar::tabEntry*)comp);
 
   if (idx!=-1)
   {
@@ -949,7 +950,8 @@ bool awsNotebookButtonBar::Remove (iAwsComponent *comp)
     if (idx < active)
       active--;
 
-    vTabs.Delete (idx);
+    vTabs.FreeItem (vTabs[idx]);
+    vTabs.DeleteIndex (idx);
     return true;
   }
   return false;
@@ -962,7 +964,7 @@ void awsNotebookButtonBar::OnDraw (csRect)
 
   int dark = pm->GetColor (AC_SHADOW);
   
-  const csRect &r = Frame();
+  csRect r = Frame();
 
   int y = (is_top ? r.ymax : r.ymin);
 
@@ -999,12 +1001,12 @@ void awsNotebookButtonBar::ScrollLeft ()
     for (int i=0; i < vTabs.Length (); i++)
     {
       awsNotebookButton *btn = vTabs.Get (i)->button;
-      csRect &f = btn->Frame ();
-      f.Move (-xdelta,0);
-      if (f.xmin >= Frame ().xmax || f.xmax <= Frame ().xmin)
-        btn->Hide ();
-      else
-        btn->Show ();
+      csRect f = btn->Frame ();
+      btn->Move (-xdelta,0);
+      //if (f.xmin >= Frame ().xmax || f.xmax <= Frame ().xmin)
+      //  btn->Hide ();
+      //else
+      //  btn->Show ();
     }
     first++;
     vTabs.Get (first)->button->SetFirst (true);
@@ -1020,12 +1022,12 @@ void awsNotebookButtonBar::ScrollRight ()
     for (int i=0; i < vTabs.Length (); i++)
     {
       awsNotebookButton *btn = vTabs.Get (i)->button;
-      csRect &f = btn->Frame ();
-      f.Move (xdelta,0);
-      if (f.xmin >= Frame ().xmax || f.xmax <= Frame ().xmin)
-        btn->Hide ();
-      else
-        btn->Show ();
+      csRect f = btn->Frame ();
+      btn->Move (xdelta,0);
+      //if (f.xmin >= Frame ().xmax || f.xmax <= Frame ().xmin)
+      //  btn->Hide ();
+      //else
+      //  btn->Show ();
     }
     first--;
     vTabs.Get (first)->button->SetFirst (true);
@@ -1071,7 +1073,8 @@ void awsNotebookButtonBar::Activate (int idx)
 void awsNotebookButtonBar::ActivateTab (void *sk, iAwsSource *source)
 {
   awsNotebookButtonBar *bb = (awsNotebookButtonBar *)sk;
-  int idx = bb->vTabs.FindKey ((csConstSome)source->GetComponent (), 1);
+  int idx = bb->vTabs.FindKey (
+    bb->vTabs.KeyButtonFunctor((awsNotebookButton*)source->GetComponent ()));
   if (idx != -1 && bb->active != idx)
   {
     // hide the active and make the new one active
