@@ -17,24 +17,29 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include <ctype.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#define CS_SYSDEF_PROVIDE_PATH
 #define CS_SYSDEF_PROVIDE_DIR
 #define CS_SYSDEF_PROVIDE_ACCESS
 #define CS_SYSDEF_PROVIDE_MKDIR
 #define CS_SYSDEF_PROVIDE_UNLINK
 #define CS_SYSDEF_VFS_PROVIDE_CHECK_VAR
+#define CS_SYSDEF_PROVIDE_HARDWARE_MMIO 1
 #include "cssysdef.h"
-#include "cssys/sysfunc.h"
+#include "csutil/sysfunc.h"
+#include "csutil/syspath.h"
 #include "vfs.h"
 #include "csutil/archive.h"
 #include "csutil/util.h"
-#include "csutil/scfstrv.h"
+#include "csutil/scfstringarray.h"
 #include "csutil/databuf.h"
 #include "csutil/csstring.h"
+#include "csutil/parray.h"
+#include "csutil/cmdline.h"
+#include "iutil/objreg.h"
 
 CS_IMPLEMENT_PLUGIN
 
@@ -56,11 +61,24 @@ class DiskFile : public csFile
 
   // The file
   FILE *file;
+  // Contains the complete file, if GetAllData() was called
+  csRef<iDataBuffer> alldata;
   // constructor
-  DiskFile(int Mode, VfsNode* ParentNode, int RIndex, const char* NameSuffix);
+  DiskFile(int Mode, VfsNode* ParentNode, int RIndex,
+  	const char* NameSuffix);
   // set Error according to errno
   void CheckError ();
+  // whether this file was opened for writing or reading
+  bool writemode;
+  // 'real-world' path of this file
+  char *fName;
+  // position in the data buffer
+  size_t fpos;
+  // whether alldata is null-terminated
+  bool buffernt;
 
+  // attempt to create a file mapping buffer from this file
+  iDataBuffer* TryCreateMapping ();
 public:
   SCF_DECLARE_IBASE;
 
@@ -70,12 +88,18 @@ public:
   virtual size_t Read (char *Data, size_t DataSize);
   // write a block of data
   virtual size_t Write (const char *Data, size_t DataSize);
+  /// flush stream
+  virtual void Flush ();
   // check for EOF
   virtual bool AtEOF ();
   /// Query current file pointer
   virtual size_t GetPos ();
   /// Clear file error after queriyng status
   virtual int GetStatus ();
+  /// Set file position
+  virtual bool SetPos (size_t newpos);
+  /// Get all data
+  virtual csPtr<iDataBuffer> GetAllData (bool nullterm = false);
 private:
   // Create a directory or a series of directories starting from PathBase
   void MakeDir (const char *PathBase, const char *PathSuffix);
@@ -87,6 +111,7 @@ class VfsArchive;
 // This is a version of csFile which "lives" in archives
 class ArchiveFile : public csFile
 {
+private:
   friend class VfsNode;
 
   // parent archive
@@ -95,6 +120,8 @@ class ArchiveFile : public csFile
   void *fh;
   // file data (for read mode)
   char *data;
+  // buffer, where read mode data is contained
+  csRef<iDataBuffer> databuf;
   // current data pointer
   size_t fpos;
   // constructor
@@ -112,15 +139,22 @@ public:
   virtual size_t Write (const char *Data, size_t DataSize);
   // check for EOF
   virtual bool AtEOF ();
+  /// flush stream
+  virtual void Flush ();
   /// Query current file pointer
   virtual size_t GetPos ();
   /// Get all the data at once
-  virtual iDataBuffer *GetAllData ();
+  virtual csPtr<iDataBuffer> GetAllData (bool nullterm = false);
+  /// Set current file pointer
+  virtual bool SetPos (size_t newpos);
 };
 
 class VfsArchive : public csArchive
 {
 public:
+  /// Mutex to make VFS thread-safe.
+  csRef<csMutex> archive_mutex;
+
   // Last time this archive was used
   long LastUseTime;
   // Number of references (open files) to this archive
@@ -160,6 +194,8 @@ public:
 #ifdef VFS_DEBUG
     printf ("VFS: opening archive \"%s\"\n", filename);
 #endif
+    // We need a recursive mutex.
+    archive_mutex = csMutex::Create (true);
   }
   virtual ~VfsArchive ()
   {
@@ -173,45 +209,66 @@ public:
   }
 };
 
-class VfsArchiveCache : public csVector
+/// This class is thread-safe because it is global.
+class VfsArchiveCache
 {
+private:
+  csPDelArray<VfsArchive> array;
+
 public:
-  VfsArchiveCache () : csVector (8, 8)
+  VfsArchiveCache () : array (8, 8)
   {
   }
   virtual ~VfsArchiveCache ()
   {
-    DeleteAll ();
+    array.DeleteAll ();
   }
-  int CompareKey (csSome Item, csConstSome Key, int Mode) const
+
+  /// Find a given archive file.
+  int FindKey (const char* Key) const
   {
-    (void)Mode;
-    return strcmp (((VfsArchive *)Item)->GetName (), (char *)Key);
-  }
-  virtual bool FreeItem (csSome Item)
-  {
-    delete (VfsArchive *)Item;
-    return true;
+    int i;
+    for (i = 0; i < array.Length (); i++)
+      if (strcmp (array[i]->GetName (), Key) == 0)
+        return i;
+    return -1;
   }
+
   VfsArchive *Get (int iIndex)
   {
-    return (VfsArchive *)csVector::Get (iIndex);
+    return array.Get (iIndex);
   }
+
+  int Length () const
+  {
+    return array.Length ();
+  }
+
+  void Push (VfsArchive* ar)
+  {
+    array.Push (ar);
+  }
+
+  void DeleteAll ()
+  {
+    array.DeleteAll ();
+  }
+
   void CheckUp ()
   {
-	int i;
-    for (i = Length () - 1; i >= 0; i--)
+    int i;
+    for (i = array.Length () - 1; i >= 0; i--)
     {
-      VfsArchive *a = Get (i);
+      VfsArchive *a = array.Get (i);
       if (a->CheckUp ())
-        Delete (i);
+        array.DeleteIndex (i);
     }
   }
 };
 
 // Private structure used to keep a "node" in virtual filesystem tree.
 // The program can be made even fancier if we use a object for each
-// "real" path (i.e. each VfsNode will contain an csVector of real-world
+// "real" path (i.e. each VfsNode will contain an array of real-world
 // nodes - both "directory" and "archive" types) but since we have to
 // balance between pretty understandable code and effective code, this
 // time we choose effectivity - the cost can become very big in this case.
@@ -223,9 +280,9 @@ public:
   // Configuration section key
   char *ConfigKey;
   // The array of real paths/archives bound to this virtual path
-  csStrVector RPathV;
+  csStringArray RPathV;
   // The array of unexpanded real paths
-  csStrVector UPathV;
+  csStringArray UPathV;
   // The system interface
   iObjectRegistry *object_reg;
 
@@ -239,7 +296,7 @@ public:
   // Remove a real-world path
   bool RemoveRPath (const char *RealPath);
   // Find all files in a subpath
-  void FindFiles (const char *Suffix, const char *Mask, iStrVector *FileList);
+  void FindFiles (const char *Suffix, const char *Mask, iStringArray *FileList);
   // Find a file and return the appropiate csFile object
   iFile *Open (int Mode, const char *Suffix);
   // Delete a file
@@ -262,7 +319,7 @@ private:
 };
 
 // The global archive cache
-static VfsArchiveCache *ArchiveCache = NULL;
+static VfsArchiveCache *ArchiveCache = 0;
 
 // -------------------------------------------------------------- csFile --- //
 
@@ -295,12 +352,54 @@ int csFile::GetStatus ()
   return rc;
 }
 
-iDataBuffer *csFile::GetAllData ()
+// ------------------------------------------------------------ DiskFile --- //
+
+#ifdef CS_HAS_MEMORY_MAPPED_IO
+bool MemoryMapFile(mmioInfo*, char const* filename);
+void UnMemoryMapFile(mmioInfo*);
+
+class csMMapDataBuffer : public iDataBuffer
 {
-  return NULL;
+  mmioInfo mapping;
+  bool status;
+public:
+  SCF_DECLARE_IBASE;
+
+  csMMapDataBuffer (const char* filename);
+  virtual ~csMMapDataBuffer ();
+
+  bool GetStatus() { return status; }
+
+  virtual size_t GetSize () const { return mapping.file_size; };
+  virtual char* GetData () const { return (char*)mapping.data; };
+};
+
+SCF_IMPLEMENT_IBASE (csMMapDataBuffer)
+  SCF_IMPLEMENTS_INTERFACE (iDataBuffer);
+SCF_IMPLEMENT_IBASE_END
+
+csMMapDataBuffer::csMMapDataBuffer (const char* filename)
+{
+  SCF_CONSTRUCT_IBASE (0);
+
+  status = MemoryMapFile (&mapping, filename);
+  if (!status)
+  {
+    mapping.data = 0; 
+    mapping.file_size = 0;
+  }
 }
 
-// ------------------------------------------------------------ DiskFile --- //
+csMMapDataBuffer::~csMMapDataBuffer ()
+{
+  if (status)
+  {
+    UnMemoryMapFile (&mapping);
+  }
+  SCF_DESTRUCT_IBASE();
+}
+
+#endif
 
 SCF_IMPLEMENT_IBASE (DiskFile)
   SCF_IMPLEMENTS_INTERFACE (iFile)
@@ -313,14 +412,22 @@ SCF_IMPLEMENT_IBASE_END
 #define VFS_READ_MODE	(O_RDONLY | O_BINARY)
 #define VFS_WRITE_MODE	(O_CREAT | O_TRUNC | O_WRONLY | O_BINARY)
 
+// files above this size are attempted to be mapped into memory, 
+// instead of accessed via 'normal' file operations
+#define VFS_DISKFILE_MAPPING_THRESHOLD_MIN	    0
+// same as above, but upper size limit
+#define VFS_DISKFILE_MAPPING_THRESHOLD_MAX	    256*1024*1024
+// disabled for now.
+// #define VFS_DISKFILE_MAPPING
+
 DiskFile::DiskFile (int Mode, VfsNode *ParentNode, int RIndex,
   const char *NameSuffix) : csFile (Mode, ParentNode, RIndex, NameSuffix)
 {
-  SCF_CONSTRUCT_IBASE (NULL);
+  SCF_CONSTRUCT_IBASE (0);
   char *rp = (char *)Node->RPathV [Index];
   size_t rpl = strlen (rp);
   size_t nsl = strlen (NameSuffix);
-  char *fName = new char [rpl + nsl + 1];
+  fName = new char [rpl + nsl + 1];
   memcpy (fName, rp, rpl);
   memcpy (fName + rpl, NameSuffix, nsl + 1);
 
@@ -330,30 +437,31 @@ DiskFile::DiskFile (int Mode, VfsNode *ParentNode, int RIndex,
     if (fName [rpl + n] == VFS_PATH_SEPARATOR)
       fName [rpl + n] = PATH_SEPARATOR;
 
+  writemode = (Mode & VFS_FILE_MODE) == VFS_FILE_WRITE;
+
   int t;
   for (t = 1; t <= 2; t++)
   {
 #ifdef VFS_DEBUG
     printf ("VFS: Trying to open disk file \"%s\"\n", fName);
 #endif
-    file = fopen (fName, (Mode & VFS_FILE_MODE) == VFS_FILE_WRITE ? "wb":"rb");
+    file = fopen (fName, writemode ? "wb" : "rb");
     if (file || (t != 1))
       break;
-    char *lastps = strrchr (NameSuffix, VFS_PATH_SEPARATOR);
-    if (!lastps)
-      break;
 
     // we don't need to create a directory if we only want to read
     if ((Mode & VFS_FILE_MODE) == VFS_FILE_READ)
       break;
+    
+    char *lastps = (char*)strrchr (NameSuffix, VFS_PATH_SEPARATOR);
+    if (!lastps)
+      break;
 
     *lastps = 0;
     MakeDir (rp, NameSuffix);
     *lastps = VFS_PATH_SEPARATOR;
   }
 
-  delete [] fName;
-
   if (!file)
     CheckError ();
   if (Error == VFS_STATUS_OK)
@@ -373,6 +481,22 @@ DiskFile::DiskFile (int Mode, VfsNode *ParentNode, int RIndex,
   if (file)
     printf ("VFS: Successfully opened, handle = %d\n", fileno (file));
 #endif
+
+#if defined(VFS_DISKFILE_MAPPING) && defined(CS_HAS_MEMORY_MAPPED_IO)
+  if ((Error == VFS_STATUS_OK) && (!writemode) && 
+    (Size >= VFS_DISKFILE_MAPPING_THRESHOLD_MIN) &&
+    (Size <= VFS_DISKFILE_MAPPING_THRESHOLD_MAX))
+  {
+    alldata = csPtr<iDataBuffer> (TryCreateMapping ());
+    if (alldata)
+    {
+      fclose (file);
+      file = 0;
+      SetPos (0);
+      buffernt = false;
+    }
+  }
+#endif
 }
 
 DiskFile::~DiskFile ()
@@ -386,6 +510,9 @@ DiskFile::~DiskFile ()
 
   if (file)
     fclose (file);
+  delete [] fName;
+
+  SCF_DESTRUCT_IBASE();
 }
 
 void DiskFile::MakeDir (const char *PathBase, const char *PathSuffix)
@@ -394,7 +521,7 @@ void DiskFile::MakeDir (const char *PathBase, const char *PathSuffix)
   size_t pl = pbl + strlen (PathSuffix) + 1;
   char *path = new char [pl];
   char *cur = path + pbl;
-  char *prev = NULL;
+  char *prev = 0;
 
   strcpy (path, PathBase);
   strcpy (cur, PathSuffix);
@@ -504,28 +631,162 @@ void DiskFile::CheckError ()
 
 size_t DiskFile::Read (char *Data, size_t DataSize)
 {
-  size_t rc = fread (Data, 1, DataSize, file);
-  if (rc < DataSize)
-    CheckError ();
-  return rc;
+  if (writemode)
+  {
+    Error = VFS_STATUS_ACCESSDENIED;
+    return 0;
+  }
+  else
+  {
+    if (file)
+    {
+      size_t rc = fread (Data, 1, DataSize, file);
+      if (rc < DataSize)
+	CheckError ();
+      return rc;
+    }
+    else
+    {
+      size_t rc = MIN (DataSize, Size - fpos);
+      memcpy (Data, (void*)(alldata->GetData() + fpos), rc);
+      fpos += rc;
+      return rc;
+    }
+  }
 }
 
 size_t DiskFile::Write (const char *Data, size_t DataSize)
 {
-  size_t rc = fwrite (Data, 1, DataSize, file);
-  if (rc < DataSize)
-    CheckError ();
-  return rc;
+  if (!writemode)
+  {
+    Error = VFS_STATUS_ACCESSDENIED;
+    return 0;
+  }
+  else
+  {
+    size_t rc = fwrite (Data, 1, DataSize, file);
+    if (rc < DataSize)
+      CheckError ();
+    return rc;
+  }
+}
+
+void DiskFile::Flush ()
+{
+  if (file)
+    fflush (file);
 }
 
 bool DiskFile::AtEOF ()
 {
-  return feof (file);
+  if (file)
+  {
+    return feof (file);
+  }
+  else
+  {
+    return (fpos >= Size);
+  }
 }
 
 size_t DiskFile::GetPos ()
 {
-  return ftell (file);
+  if (file)
+  {
+    return ftell (file);
+  }
+  else
+  {
+    return fpos;
+  }
+}
+
+bool DiskFile::SetPos (size_t newpos)
+{
+  if (file)
+  {
+    return (fseek (file, newpos, SEEK_SET) == 0);
+  }
+  else
+  {
+    fpos = (newpos > Size) ? Size : newpos;
+    return true;
+  }
+}
+
+csPtr<iDataBuffer> DiskFile::GetAllData (bool nullterm)
+{
+// retrieve file contents
+
+  // refuse to work when writing
+  if (!writemode)
+  {
+    // do we already have everything?
+    if (!alldata)
+    {
+      iDataBuffer* newbuf = 0;
+      // attempt to create file mapping
+      size_t oldpos = GetPos();
+      if (!nullterm)
+      {
+	newbuf = TryCreateMapping ();
+      }
+      // didn't succeed or not supported -
+      // old style readin'
+      if (!newbuf)
+      {
+	SetPos (0);
+
+	char* data = new char [Size + 1];
+	csDataBuffer* dbuf = new csDataBuffer (data, Size);
+	Read (data, Size);
+	*((char*)data + Size) = 0;
+
+	newbuf = dbuf;
+      }
+      // close file, set correct pos
+      fclose (file);
+      file = 0;
+      SetPos (oldpos);
+      // setup buffer.
+      alldata = csPtr<iDataBuffer> (newbuf);
+      buffernt = nullterm;
+    }
+    else
+    {
+      // The data was already read.
+      if (nullterm && !buffernt)
+      {
+	// However, a null-terminated buffer is requested,
+	// but this one isn't yet - copy data, append null
+	alldata = csPtr<iDataBuffer> 
+	  (new csDataBuffer (alldata));
+
+        buffernt = nullterm;
+      }
+    }
+    return csPtr<iDataBuffer> (alldata);
+  }
+  else
+  {
+    return 0;
+  }
+}
+
+iDataBuffer* DiskFile::TryCreateMapping ()
+{
+#ifdef CS_HAS_MEMORY_MAPPED_IO  
+  csMMapDataBuffer* buf = new csMMapDataBuffer (fName);
+  if (buf->GetStatus())
+  {
+    return buf;
+  }
+  else
+  {
+    delete buf;
+  }
+#endif
+  return 0;
 }
 
 // --------------------------------------------------------- ArchiveFile --- //
@@ -538,14 +799,15 @@ ArchiveFile::ArchiveFile (int Mode, VfsNode *ParentNode, int RIndex,
   const char *NameSuffix, VfsArchive *ParentArchive) :
   csFile (Mode, ParentNode, RIndex, NameSuffix)
 {
-  SCF_CONSTRUCT_IBASE (NULL);
+  SCF_CONSTRUCT_IBASE (0);
   Archive = ParentArchive;
   Error = VFS_STATUS_OTHER;
   Size = 0;
-  fh = NULL;
-  data = NULL;
+  fh = 0;
+  data = 0;
   fpos = 0;
 
+  csScopedMutexLock lock (Archive->archive_mutex);
   Archive->UpdateTime ();
   ArchiveCache->CheckUp ();
 
@@ -560,7 +822,10 @@ ArchiveFile::ArchiveFile (int Mode, VfsNode *ParentNode, int RIndex,
     if (Archive->Writing == 0)
       Archive->Flush ();
     if ((data = Archive->Read (NameSuffix, &Size)))
+    {
       Error = VFS_STATUS_OK;
+      databuf = csPtr<iDataBuffer> (new csDataBuffer (data, Size));
+    }
   }
   else if ((Mode & VFS_FILE_MODE) == VFS_FILE_WRITE)
   {
@@ -579,24 +844,40 @@ ArchiveFile::~ArchiveFile ()
   printf ("VFS: Closing some file from archive \"%s\"\n", Archive->GetName ());
 #endif
 
-  delete [] data;
+  csScopedMutexLock lock (Archive->archive_mutex);
   if (fh)
     Archive->Writing--;
   Archive->DecRef ();
+
+  SCF_DESTRUCT_IBASE();
 }
 
 size_t ArchiveFile::Read (char *Data, size_t DataSize)
 {
-  size_t sz = DataSize;
-  if (fpos + sz > Size)
-    sz = Size - fpos;
-  memcpy (Data, data + fpos, sz);
-  fpos += sz;
-  return sz;
+  if (data)
+  {
+    size_t sz = DataSize;
+    if (fpos + sz > Size)
+      sz = Size - fpos;
+    memcpy (Data, data + fpos, sz);
+    fpos += sz;
+    return sz;
+  }
+  else
+  {
+    Error = VFS_STATUS_ACCESSDENIED;
+    return 0;
+  }
 }
 
 size_t ArchiveFile::Write (const char *Data, size_t DataSize)
 {
+  if (data)
+  {
+    Error = VFS_STATUS_ACCESSDENIED;
+    return 0;
+  }
+  csScopedMutexLock lock (Archive->archive_mutex);
   if (!Archive->Write (fh, Data, DataSize))
   {
     Error = VFS_STATUS_NOSPACE;
@@ -605,6 +886,15 @@ size_t ArchiveFile::Write (const char *Data, size_t DataSize)
   return DataSize;
 }
 
+void ArchiveFile::Flush ()
+{
+  if (Archive)
+  {
+    csScopedMutexLock lock (Archive->archive_mutex);
+    Archive->Flush ();
+  }
+}
+
 bool ArchiveFile::AtEOF ()
 {
   if (data)
@@ -618,13 +908,29 @@ size_t ArchiveFile::GetPos ()
   return fpos;
 }
 
-iDataBuffer *ArchiveFile::GetAllData ()
+bool ArchiveFile::SetPos (size_t newpos)
 {
-  iDataBuffer *db = new csDataBuffer (data, Size);
-  data = NULL;
-  fpos = 0;
-  Size = 0;
-  return db;
+  if (data)
+  {
+    fpos = (newpos > Size) ? Size : newpos;
+    return true;
+  }
+  else
+  {
+    return false;
+  }
+}
+
+csPtr<iDataBuffer> ArchiveFile::GetAllData (bool nullterm)
+{
+  if (data)
+  {
+    return csPtr<iDataBuffer> (databuf);
+  }
+  else
+  {
+    return 0;
+  }
 }
 
 // ------------------------------------------------------------- VfsNode --- //
@@ -666,12 +972,12 @@ bool VfsNode::AddRPath (const char *RealPath, csVFS *Parent)
       src [cl] = 0;
 
       rc = true;
-      UPathV.Push (csStrNew (src));
+      UPathV.Push (src);
 
       // Now parse and expand this path
       char rpath [CS_MAXPATHLEN + 1];
       Expand (Parent, rpath, src, CS_MAXPATHLEN);
-      RPathV.Push (csStrNew (rpath));
+      RPathV.Push (rpath);
       src = cur + 1;
     } /* endif */
 
@@ -693,8 +999,8 @@ bool VfsNode::RemoveRPath (const char *RealPath)
   for (i = 0; i < RPathV.Length (); i++)
     if (strcmp ((char *)RPathV.Get (i), RealPath) == 0)
     {
-      RPathV.Delete (i);
-      UPathV.Delete (i);
+      RPathV.DeleteIndex (i);
+      UPathV.DeleteIndex (i);
       return true;
     } /* endif */
 
@@ -777,20 +1083,20 @@ const char *VfsNode::GetValue (csVFS *Parent, const char *VarName)
 
   iConfigFile *Config = &(Parent->config);
 
-  // Now look in "VFS.Solaris" section, for example
+  // Now look in "VFS.Unix" section, for example
   csString Keyname;
   Keyname << "VFS." CS_PLATFORM_NAME "." << VarName;
-  value = Config->GetStr (Keyname, NULL);
+  value = Config->GetStr (Keyname, 0);
   if (value)
     return value;
 
   // Now look in "VFS.Alias" section for alias section name
-  const char *alias = Config->GetStr ("VFS.Alias." CS_PLATFORM_NAME, NULL);
+  const char *alias = Config->GetStr ("VFS.Alias." CS_PLATFORM_NAME, 0);
   // If there is one, look into that section too
   if (alias) {
     Keyname.Clear();
     Keyname << alias << '.' << VarName;
-    value = Config->GetStr (Keyname, NULL);
+    value = Config->GetStr (Keyname, 0);
   }
   if (value)
     return value;
@@ -799,11 +1105,11 @@ const char *VfsNode::GetValue (csVFS *Parent, const char *VarName)
   // can override them in config file or environment
 
   // check for OS-specific predefined variables
-#ifdef CS_SYSDEF_VFS_CHECK_VAR
-  CS_SYSDEF_VFS_CHECK_VAR (VarName)
-#endif
+#ifdef CS_PROVIDES_VFS_VARS
+  value = csCheckPlatformVFSVar(VarName);
   if (value)
     return value;
+#endif
 
   static char path_separator [] = {VFS_PATH_SEPARATOR, 0};
   if (strcmp (VarName, path_separator) == 0)	// Path separator variable?
@@ -812,17 +1118,24 @@ const char *VfsNode::GetValue (csVFS *Parent, const char *VarName)
     return path_sep;
   }
 
+  if (strcmp (VarName, "*") == 0) // Resource directory?
+    return Parent->appdir;
+    
+  if (strcmp (VarName, "^") == 0) // Application or Cocoa wrapper directory?
+    return Parent->appdir;
+    
   if (strcmp (VarName, "@") == 0) // Installation directory?
     return Parent->basedir;
 
-  return NULL;
+  return 0;
 }
 
 void VfsNode::FindFiles (const char *Suffix, const char *Mask,
-  iStrVector *FileList)
+  iStringArray *FileList)
 {
   // Look through all RPathV's for file or directory
   int i;
+  csString vpath;
   for (i = 0; i < RPathV.Length (); i++)
   {
     char *rpath = (char *)RPathV [i];
@@ -838,15 +1151,17 @@ void VfsNode::FindFiles (const char *Suffix, const char *Mask,
       strcpy (tpath + rpl, Suffix);
       rpl = strlen (tpath);
       if ((rpl > 1)
-#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
-       && ((rpl > 3) || (tpath [1] != ':'))
+#if defined (OS_DOS) || defined (OS_WIN32)
+       && ((rpl > 2) || (tpath [1] != ':'))
+       && (!((rpl == 3) && (tpath [1] == ':') && (tpath [2] == '\\')))
+       // keep trailing backslash for drive letters
 #endif
        && ((tpath [rpl - 1] == '/') || (tpath [rpl - 1] == PATH_SEPARATOR)))
         tpath [rpl - 1] = 0;		// remove trailing PATH_SEPARATOR
 
-      if ((dh = opendir (tpath)) == NULL)
+      if ((dh = opendir (tpath)) == 0)
         continue;
-      while ((de = readdir (dh)) != NULL)
+      while ((de = readdir (dh)) != 0)
       {
         if ((strcmp (de->d_name, ".") == 0)
          || (strcmp (de->d_name, "..") == 0))
@@ -856,16 +1171,11 @@ void VfsNode::FindFiles (const char *Suffix, const char *Mask,
           continue;
 
         bool append_slash = isdir (tpath, de);
-        char *vpath = new char [strlen (VPath) + strlen (Suffix) +
-	  strlen (de->d_name) + (append_slash ? 2 : 1)];
-	strcpy (vpath, VPath);
-	strcat (vpath, Suffix);
-	strcat (vpath, de->d_name);
+	vpath.Clear();
+	vpath << VPath << Suffix << de->d_name;
 	if (append_slash)
 	{
-	  size_t sl = strlen (vpath);
-	  vpath [sl++] = VFS_PATH_SEPARATOR;
-	  vpath [sl] = 0;
+	  vpath << VFS_PATH_SEPARATOR;
 	}
 
 	FileList->Push (vpath);
@@ -903,6 +1213,11 @@ void VfsNode::FindFiles (const char *Suffix, const char *Mask,
          && csGlobMatches (fname, Mask))
 	{
           size_t cur = sl;
+
+          // Do not return an entry for the directory itself.
+          if (fname[cur] == 0)
+            continue;
+
 	  while (cur < fnl)
 	  {
 	    if (fname [cur] == VFS_PATH_SEPARATOR)
@@ -912,23 +1227,21 @@ void VfsNode::FindFiles (const char *Suffix, const char *Mask,
 	  if (cur < fnl)
 	    cur++;
           size_t vpl = strlen (VPath);
-          char *vpath = new char [vpl + cur + 1];
-          memcpy (vpath, VPath, vpl);
-          memcpy (vpath + vpl, fname, cur);
-	  vpath [vpl + cur] = 0;
+	  vpath.Clear();
+	  vpath << VPath;
+	  vpath << fname;
+	  vpath.Truncate (vpl + cur);
 	  if (FileList->Find (vpath) == -1)
             FileList->Push (vpath);
-	  else
-	    delete [] vpath;
         }
       }
     }
   }
 }
 
-iFile *VfsNode::Open (int Mode, const char *FileName)
+iFile* VfsNode::Open (int Mode, const char *FileName)
 {
-  csFile *f = NULL;
+  csFile *f = 0;
 
   // Look through all RPathV's for file or directory
   int i;
@@ -944,7 +1257,7 @@ iFile *VfsNode::Open (int Mode, const char *FileName)
       else
       {
         delete f;
-	f = NULL;
+	f = 0;
       }
     }
     else
@@ -965,13 +1278,14 @@ iFile *VfsNode::Open (int Mode, const char *FileName)
         ArchiveCache->Push (new VfsArchive (rpath, object_reg));
       }
 
-      f = new ArchiveFile (Mode, this, i, FileName, ArchiveCache->Get (idx));
+      f = new ArchiveFile (Mode, this, i, FileName,
+      	ArchiveCache->Get (idx));
       if (f->GetStatus () == VFS_STATUS_OK)
         break;
       else
       {
         delete f;
-	f = NULL;
+	f = 0;
       }
     }
   }
@@ -992,7 +1306,7 @@ bool VfsNode::FindFile (const char *Suffix, char *RealPath,
       size_t rl = strlen (rpath);
       memcpy (RealPath, rpath, rl);
       strcpy (RealPath + rl, Suffix);
-      Archive = NULL;
+      Archive = 0;
       if (access (RealPath, F_OK) == 0)
         return true;
     }
@@ -1013,7 +1327,7 @@ bool VfsNode::FindFile (const char *Suffix, char *RealPath,
 
       VfsArchive *a = ArchiveCache->Get (idx);
       a->UpdateTime ();
-      if (a->FileExists (Suffix, NULL))
+      if (a->FileExists (Suffix, 0))
       {
         Archive = a;
         strcpy (RealPath, Suffix);
@@ -1118,31 +1432,9 @@ bool VfsNode::GetFileSize (const char *Suffix, size_t &oSize)
 
 // ----------------------------------------------------------- VfsVector --- //
 
-csVFS::VfsVector::VfsVector () : csVector (16, 16)
+int csVFS::VfsVector::Compare (VfsNode* const& Item1, VfsNode* const& Item2)
 {
-}
-
-csVFS::VfsVector::~VfsVector ()
-{
-  DeleteAll ();
-}
-
-bool csVFS::VfsVector::FreeItem (csSome Item)
-{
-  delete (VfsNode *)Item;
-  return true;
-}
-
-int csVFS::VfsVector::Compare (csSome Item1, csSome Item2, int Mode) const
-{
-  (void)Mode;
-  return strcmp (((VfsNode *)Item1)->VPath, ((VfsNode *)Item2)->VPath);
-}
-
-int csVFS::VfsVector::CompareKey (csSome Item, csConstSome Key, int Mode) const
-{
-  (void)Mode;
-  return strcmp (((VfsNode *)Item)->VPath, (char *)Key);
+  return strcmp (Item1->VPath, Item2->VPath);
 }
 
 // --------------------------------------------------------------- csVFS --- //
@@ -1158,53 +1450,98 @@ SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csVFS)
 
-SCF_EXPORT_CLASS_TABLE (vfs)
-  SCF_EXPORT_CLASS (csVFS, "crystalspace.kernel.vfs",
-    "Crystal Space Virtual File System")
-SCF_EXPORT_CLASS_TABLE_END
 
 csVFS::csVFS (iBase *iParent) : dirstack (8, 8)
 {
-  object_reg = NULL;
+  object_reg = 0;
   SCF_CONSTRUCT_IBASE (iParent);
   SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
   cwd = new char [2];
   cwd [0] = VFS_PATH_SEPARATOR;
   cwd [1] = 0;
-  cnode = NULL;
+  cnode = 0;
   cnsufx [0] = 0;
-  basedir = NULL;
+  basedir = 0;
+  resdir = 0;
+  appdir = 0;
   CS_ASSERT (!ArchiveCache);
   ArchiveCache = new VfsArchiveCache ();
+
+  // We need a recursive mutex.
+  mutex = csMutex::Create (true);
 }
 
 csVFS::~csVFS ()
 {
   delete [] cwd;
   delete [] basedir;
+  delete [] resdir;
+  delete [] appdir;
   CS_ASSERT (ArchiveCache);
   delete ArchiveCache;
-  ArchiveCache = NULL;
+  ArchiveCache = 0;
+  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
+  SCF_DESTRUCT_IBASE();
+}
+
+static void add_final_delimiter(csString& s)
+{
+  if (!s.IsEmpty() && s[s.Length() - 1] != PATH_SEPARATOR)
+    s << PATH_SEPARATOR;
+}
+
+static char* alloc_normalized_path(char const* s)
+{
+  char* t = 0;
+  if (s != 0)
+  {
+    csString c(s);
+    add_final_delimiter(c);
+    t = csStrNew(c);
+  }
+  return t;
+}
+
+static bool load_vfs_config(csConfigFile& config, char const* dir)
+{
+  bool ok = false;
+  if (dir != 0)
+  {
+    csString s(dir);
+    add_final_delimiter(s);
+    s << "vfs.cfg";
+    ok = config.Load(s);
+  }
+  return ok;
 }
 
 bool csVFS::Initialize (iObjectRegistry *object_reg)
 {
   csVFS::object_reg = object_reg;
-  char vfsconfigpath [CS_MAXPATHLEN + 1];
-  csGetInstallPath (vfsconfigpath, sizeof (vfsconfigpath));
-  basedir = csStrNew (vfsconfigpath);
-  strcat (vfsconfigpath, "vfs.cfg");
-  config.Load (vfsconfigpath);
+
+  basedir = alloc_normalized_path(csGetConfigPath());
+
+  csRef<iCommandLineParser> cmdline =
+    CS_QUERY_REGISTRY (object_reg, iCommandLineParser);
+  if (cmdline)
+  {
+    resdir = alloc_normalized_path(cmdline->GetResourceDir());
+    appdir = alloc_normalized_path(cmdline->GetAppDir());
+  }
+
+  if (!load_vfs_config(config, resdir) &&
+      !load_vfs_config(config, appdir))
+       load_vfs_config(config, basedir);
+
   return ReadConfig ();
 }
 
 bool csVFS::ReadConfig ()
 {
-  iConfigIterator *iterator = config.Enumerate ("VFS.Mount.");
+  csRef<iConfigIterator> iterator (config.Enumerate ("VFS.Mount."));
   while (iterator->Next ())
     AddLink (iterator->GetKey (true), iterator->GetStr ());
-  iterator->DecRef ();
-  NodeList.QuickSort (0);
+  NodeList.Sort (NodeList.Compare);
   return true;
 }
 
@@ -1287,10 +1624,10 @@ char *csVFS::_ExpandPath (const char *Path, bool IsDir) const
   return ret;
 }
 
-iDataBuffer *csVFS::ExpandPath (const char *Path, bool IsDir) const
+csPtr<iDataBuffer> csVFS::ExpandPath (const char *Path, bool IsDir) const
 {
   char *xp = _ExpandPath (Path, IsDir);
-  return new csDataBuffer (xp, strlen (xp) + 1);
+  return csPtr<iDataBuffer> (new csDataBuffer (xp, strlen (xp) + 1));
 }
 
 VfsNode *csVFS::GetNode (const char *Path, char *NodePrefix,
@@ -1322,24 +1659,25 @@ VfsNode *csVFS::GetNode (const char *Path, char *NodePrefix,
     }
     return (VfsNode *)NodeList [best_i];
   }
-  return NULL;
+  return 0;
 }
 
 bool csVFS::PreparePath (const char *Path, bool IsDir, VfsNode *&Node,
   char *Suffix, size_t SuffixSize) const
 {
-  Node = NULL; *Suffix = 0;
+  Node = 0; *Suffix = 0;
   char *fname = _ExpandPath (Path, IsDir);
   if (!fname)
     return false;
 
   Node = GetNode (fname, Suffix, SuffixSize);
   delete [] fname;
-  return (Node != NULL);
+  return (Node != 0);
 }
 
 bool csVFS::ChDir (const char *Path)
 {
+  csScopedMutexLock lock (mutex);
   // First, transform Path to absolute
   char *newwd = _ExpandPath (Path, true);
   if (!newwd)
@@ -1355,13 +1693,19 @@ bool csVFS::ChDir (const char *Path)
   return true;
 }
 
-void csVFS::PushDir ()
+void csVFS::PushDir (char const* Path)
 {
-  dirstack.Push (csStrNew (cwd));
+  { // Scope.
+    csScopedMutexLock lock (mutex);
+    dirstack.Push (cwd);
+  }
+  if (Path != 0)
+    ChDir(Path);
 }
 
 bool csVFS::PopDir ()
 {
+  csScopedMutexLock lock (mutex);
   if (!dirstack.Length ())
     return false;
   char *olddir = (char *) dirstack.Pop ();
@@ -1378,6 +1722,7 @@ bool csVFS::Exists (const char *Path) const
   VfsNode *node;
   char suffix [VFS_MAX_PATH_LEN + 1];
 
+  csScopedMutexLock lock (mutex);
   PreparePath (Path, false, node, suffix, sizeof (suffix));
   bool exists = (node && (!suffix [0] || node->Exists (suffix)));
 
@@ -1385,145 +1730,177 @@ bool csVFS::Exists (const char *Path) const
   return exists;
 }
 
-iStrVector *csVFS::FindFiles (const char *Path) const
+csRef<iStringArray> csVFS::MountRoot (const char *Path)
 {
-  if (!Path)
-    return NULL;
+  scfStringArray* outv = new scfStringArray;
 
-  VfsNode *node;				// the node we are searching
-  char suffix [VFS_MAX_PATH_LEN + 1];		// the suffix relative to node
-  char mask [VFS_MAX_PATH_LEN + 1];		// the filename mask
-  char XPath [VFS_MAX_PATH_LEN + 1];		// the expanded path
-  scfStrVector *fl = new scfStrVector (16, 16);// the output list
-
-  PreparePath (Path, false, node, suffix, sizeof (suffix));
-
-  // Now separate the mask from directory suffix
-  int dirlen = strlen (suffix);
-  while (dirlen && suffix [dirlen - 1] != VFS_PATH_SEPARATOR)
-    dirlen--;
-  strcpy (mask, suffix + dirlen);
-  suffix [dirlen] = 0;
-  if (!mask [0])
-    strcpy (mask, "*");
-
-  if (node)
+  csScopedMutexLock lock (mutex);
+  if (Path != 0)
   {
-    strcpy (XPath, node->VPath);
-    strcat (XPath, suffix);
-  }
-  else
-  {
-    char *s = _ExpandPath (Path, true);
-    strcpy (XPath, s);
-    delete [] s;
-  }
-
-  // first add all nodes that are located one level deeper
-  // these are "directories" and will have a slash appended
-  size_t sl = strlen (XPath);
-  int i;
-  for (i = 0; i < NodeList.Length (); i++)
-  {
-    VfsNode *node = (VfsNode *)NodeList [i];
-    if ((memcmp (node->VPath, XPath, sl) == 0)
-     && (node->VPath [sl]))
+    csRef<iStringArray> roots = csFindSystemRoots();
+    for (int i = 0, n = roots->Length(); i < n; i++)
     {
-      const char *pp = node->VPath + sl;
-      while (*pp && *pp == VFS_PATH_SEPARATOR)
-	pp++;
-      while (*pp && *pp != VFS_PATH_SEPARATOR)
-        pp++;
-      while (*pp && *pp == VFS_PATH_SEPARATOR)
-        pp++;
-      char *news = new char [pp - node->VPath + 1];
-      memcpy (news, node->VPath, pp - node->VPath);
-      news [pp - node->VPath] = 0;
-      if (fl->Find (news) == -1)
-        fl->Push (news);
-      else
-        delete [] news;
+      char const* t = roots->Get(i);
+      csString s(t);
+      int const slen = s.Length();
+      char c = '\0';
+
+      csString vfs_dir;
+      vfs_dir << Path << '/';
+      for (int j = 0; j < slen; j++)
+      {
+        c = s.GetAt(j);
+        if (c == '_' || c == '-' || isalnum(c))
+	  vfs_dir << (char)tolower(c);
+      }
+
+      csString real_dir(s);
+      if (slen > 0 && (c = real_dir.GetAt(slen - 1)) == '/' || c == '\\')
+        real_dir.Truncate(slen - 1);
+      real_dir << "$/";
+
+      outv->Push (vfs_dir);
+      Mount(vfs_dir, real_dir);
     }
   }
 
-  // Now find all files in given directory node
-  if (node)
-    node->FindFiles (suffix, mask, fl);
-
-  if (fl->Length () == 0)
-  {
-    delete fl;
-    fl = NULL;
-  }
-
-  ArchiveCache->CheckUp ();
-  return fl;
+  csRef<iStringArray> v(outv);
+  outv->DecRef ();
+  return v;
 }
 
-iFile *csVFS::Open (const char *FileName, int Mode)
+csPtr<iStringArray> csVFS::FindFiles (const char *Path) const
+{
+  csScopedMutexLock lock (mutex);
+  scfStringArray *fl = new scfStringArray;		// the output list
+
+  csString news;
+  if (Path != 0)
+  {
+    VfsNode *node;				// the node we are searching
+    char suffix [VFS_MAX_PATH_LEN + 1];		// the suffix relative to node
+    char mask [VFS_MAX_PATH_LEN + 1];		// the filename mask
+    char XPath [VFS_MAX_PATH_LEN + 1];		// the expanded path
+
+    PreparePath (Path, false, node, suffix, sizeof (suffix));
+
+    // Now separate the mask from directory suffix
+    int dirlen = strlen (suffix);
+    while (dirlen && suffix [dirlen - 1] != VFS_PATH_SEPARATOR)
+      dirlen--;
+    strcpy (mask, suffix + dirlen);
+    suffix [dirlen] = 0;
+    if (!mask [0])
+      strcpy (mask, "*");
+
+    if (node)
+    {
+      strcpy (XPath, node->VPath);
+      strcat (XPath, suffix);
+    }
+    else
+    {
+      char *s = _ExpandPath (Path, true);
+      strcpy (XPath, s);
+      delete [] s;
+    }
+
+    // first add all nodes that are located one level deeper
+    // these are "directories" and will have a slash appended
+    size_t sl = strlen (XPath);
+    int i;
+    for (i = 0; i < NodeList.Length (); i++)
+    {
+      VfsNode *node = (VfsNode *)NodeList [i];
+      if ((memcmp (node->VPath, XPath, sl) == 0) && (node->VPath [sl]))
+      {
+        const char *pp = node->VPath + sl;
+        while (*pp && *pp == VFS_PATH_SEPARATOR)
+	  pp++;
+        while (*pp && *pp != VFS_PATH_SEPARATOR)
+          pp++;
+        while (*pp && *pp == VFS_PATH_SEPARATOR)
+          pp++;
+	news.Clear();
+	news.Append (node->VPath);
+	news.Truncate (pp - node->VPath);
+        if (fl->Find (news) == -1)
+          fl->Push (news);
+      }
+    }
+
+    // Now find all files in given directory node
+    if (node)
+      node->FindFiles (suffix, mask, fl);
+
+    ArchiveCache->CheckUp ();
+  }
+
+  csPtr<iStringArray> v(fl);
+  return v;
+}
+
+csPtr<iFile> csVFS::Open (const char *FileName, int Mode)
 {
   if (!FileName)
-    return NULL;
+    return 0;
 
   VfsNode *node;
   char suffix [VFS_MAX_PATH_LEN + 1];
+  csScopedMutexLock lock (mutex);
   if (!PreparePath (FileName, false, node, suffix, sizeof (suffix)))
-    return NULL;
+    return 0;
 
   iFile *f = node->Open (Mode, suffix);
 
   ArchiveCache->CheckUp ();
-  return f;
+  return csPtr<iFile> (f);
 }
 
 bool csVFS::Sync ()
 {
+  csScopedMutexLock lock (mutex);
   ArchiveCache->DeleteAll ();
   return (ArchiveCache->Length () == 0);
 }
 
-iDataBuffer *csVFS::ReadFile (const char *FileName)
+csPtr<iDataBuffer> csVFS::ReadFile (const char *FileName, bool nullterm)
 {
-  iFile *F = Open (FileName, VFS_FILE_READ);
+  csScopedMutexLock lock (mutex);
+  csRef<iFile> F (Open (FileName, VFS_FILE_READ));
   if (!F)
-    return NULL;
+    return 0;
 
   size_t Size = F->GetSize ();
-  iDataBuffer *data = F->GetAllData ();
+  csRef<iDataBuffer> data (F->GetAllData (nullterm));
   if (data)
   {
-    F->DecRef ();
-    return data;
+    return csPtr<iDataBuffer> (data);
   }
 
   char *buff = new char [Size + 1];
   if (!buff)
-  {
-    F->DecRef ();
-    return NULL;
-  }
+    return 0;
 
   // Make the file zero-terminated in the case we'll use it as an ASCIIZ string
   buff [Size] = 0;
   if (F->Read (buff, Size) != Size)
   {
     delete [] buff;
-    F->DecRef ();
-    return NULL;
+    return 0;
   }
 
-  F->DecRef ();
-  return new csDataBuffer (buff, Size);
+  return csPtr<iDataBuffer> (new csDataBuffer (buff, Size));
 }
 
 bool csVFS::WriteFile (const char *FileName, const char *Data, size_t Size)
 {
-  iFile *F = Open (FileName, VFS_FILE_WRITE);
+  csScopedMutexLock lock (mutex);
+  csRef<iFile> F (Open (FileName, VFS_FILE_WRITE));
   if (!F)
     return false;
 
   bool success = (F->Write (Data, Size) == Size);
-  F->DecRef ();
   return success;
 }
 
@@ -1534,6 +1911,7 @@ bool csVFS::DeleteFile (const char *FileName)
 
   VfsNode *node;
   char suffix [VFS_MAX_PATH_LEN + 1];
+  csScopedMutexLock lock (mutex);
   if (!PreparePath (FileName, false, node, suffix, sizeof (suffix)))
     return false;
 
@@ -1545,6 +1923,7 @@ bool csVFS::DeleteFile (const char *FileName)
 
 bool csVFS::Mount (const char *VirtualPath, const char *RealPath)
 {
+  csScopedMutexLock lock (mutex);
   ArchiveCache->CheckUp ();
 
   if (!VirtualPath || !RealPath)
@@ -1565,7 +1944,7 @@ bool csVFS::Mount (const char *VirtualPath, const char *RealPath)
   {
     int idx = NodeList.Find (node);
     if (idx >= 0)
-      NodeList.Delete (idx);
+      NodeList.DeleteIndex (idx);
     return false;
   }
 
@@ -1574,6 +1953,7 @@ bool csVFS::Mount (const char *VirtualPath, const char *RealPath)
 
 bool csVFS::Unmount (const char *VirtualPath, const char *RealPath)
 {
+  csScopedMutexLock lock (mutex);
   ArchiveCache->CheckUp ();
 
   if (!VirtualPath)
@@ -1595,7 +1975,7 @@ bool csVFS::Unmount (const char *VirtualPath, const char *RealPath)
     config.DeleteKey (s);
     int idx = NodeList.Find (node);
     if (idx >= 0)
-      NodeList.Delete (idx);
+      NodeList.DeleteIndex (idx);
   }
 
   return true;
@@ -1603,6 +1983,7 @@ bool csVFS::Unmount (const char *VirtualPath, const char *RealPath)
 
 bool csVFS::SaveMounts (const char *FileName)
 {
+  csScopedMutexLock lock (mutex);
   int i;
   for (i = 0; i < NodeList.Length (); i++)
   {
@@ -1644,6 +2025,7 @@ bool csVFS::GetFileTime (const char *FileName, csFileTime &oTime) const
 
   VfsNode *node;
   char suffix [VFS_MAX_PATH_LEN + 1];
+  csScopedMutexLock lock (mutex);
   PreparePath (FileName, false, node, suffix, sizeof (suffix));
 
   bool success = node ? node->GetFileTime (suffix, oTime) : false;
@@ -1659,6 +2041,7 @@ bool csVFS::SetFileTime (const char *FileName, const csFileTime &iTime)
 
   VfsNode *node;
   char suffix [VFS_MAX_PATH_LEN + 1];
+  csScopedMutexLock lock (mutex);
   PreparePath (FileName, false, node, suffix, sizeof (suffix));
 
   bool success = node ? node->SetFileTime (suffix, iTime) : false;
@@ -1674,6 +2057,7 @@ bool csVFS::GetFileSize (const char *FileName, size_t &oSize)
 
   VfsNode *node;
   char suffix [VFS_MAX_PATH_LEN + 1];
+  csScopedMutexLock lock (mutex);
   PreparePath (FileName, false, node, suffix, sizeof (suffix));
 
   bool success = node ? node->GetFileSize (suffix, oSize) : false;
@@ -1682,15 +2066,16 @@ bool csVFS::GetFileSize (const char *FileName, size_t &oSize)
   return success;
 }
 
-iDataBuffer *csVFS::GetRealPath (const char *FileName)
+csPtr<iDataBuffer> csVFS::GetRealPath (const char *FileName)
 {
   if (!FileName)
-    return NULL;
+    return 0;
 
   VfsNode *node;
   char suffix [VFS_MAX_PATH_LEN + 1];
+  csScopedMutexLock lock (mutex);
   PreparePath (FileName, false, node, suffix, sizeof (suffix));
-  if (!node) return NULL;
+  if (!node) return 0;
 
   char path [CS_MAXPATHLEN + 1];
   int i;
@@ -1704,5 +2089,6 @@ iDataBuffer *csVFS::GetRealPath (const char *FileName)
 
   strcat (strcpy (path, node->RPathV.Get (0)), suffix);
 done:
-  return new csDataBuffer (csStrNew (path), strlen (path) + 1);
+  return csPtr<iDataBuffer> (
+  	new csDataBuffer (csStrNew (path), strlen (path) + 1));
 }
