@@ -27,11 +27,14 @@
 #include "ivideo/vbufmgr.h"
 #include "iengine/material.h"
 #include "iengine/camera.h"
+#include "iengine/sector.h"
 #include "igeom/clip2d.h"
 #include "iengine/engine.h"
 #include "iengine/light.h"
 #include "iutil/objreg.h"
+
 #include "qsqrt.h"
+#include "iutil/strset.h"
 
 //#define BALL_DEBUG
 
@@ -39,26 +42,64 @@ CS_IMPLEMENT_PLUGIN
 
 SCF_IMPLEMENT_IBASE (csBallMeshObject)
   SCF_IMPLEMENTS_INTERFACE (iMeshObject)
+  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iObjectModel)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iBallState)
+#ifndef CS_USE_NEW_RENDERER
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iVertexBufferManagerClient)
+#endif
+  {
+    static scfInterfaceID iPolygonMesh_scfID = (scfInterfaceID)-1;		
+    if (iPolygonMesh_scfID == (scfInterfaceID)-1)				
+      iPolygonMesh_scfID = iSCF::SCF->GetInterfaceID ("iPolygonMesh");		
+    if (iInterfaceID == iPolygonMesh_scfID &&				
+      scfCompatibleVersion(iVersion, scfInterface<iPolygonMesh>::GetVersion()))
+    {
+#ifdef CS_DEBUG
+      printf ("Deprecated feature use: iPolygonMesh queried from Ball "
+	"object; use iMeshObject->GetObjectModel()->"
+	"GetPolygonMeshColldet() instead.\n");
+#endif
+      (&scfiPolygonMesh)->IncRef ();						
+      return CS_STATIC_CAST(iPolygonMesh*, &scfiPolygonMesh);				
+    }
+  }
 SCF_IMPLEMENT_IBASE_END
 
+#ifndef CS_USE_NEW_RENDERER
+SCF_IMPLEMENT_EMBEDDED_IBASE (csBallMeshObject::eiVertexBufferManagerClient)
+  SCF_IMPLEMENTS_INTERFACE (iVertexBufferManagerClient)
+SCF_IMPLEMENT_EMBEDDED_IBASE_END
+#endif
+
 SCF_IMPLEMENT_EMBEDDED_IBASE (csBallMeshObject::BallState)
   SCF_IMPLEMENTS_INTERFACE (iBallState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBallMeshObject::eiVertexBufferManagerClient)
-  SCF_IMPLEMENTS_INTERFACE (iVertexBufferManagerClient)
+SCF_IMPLEMENT_EMBEDDED_IBASE (csBallMeshObject::ObjectModel)
+  SCF_IMPLEMENTS_INTERFACE (iObjectModel)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
+SCF_IMPLEMENT_IBASE(csBallMeshObject::PolyMesh)
+  SCF_IMPLEMENTS_INTERFACE(iPolygonMesh)
+SCF_IMPLEMENT_IBASE_END
 
 csBallMeshObject::csBallMeshObject (iMeshObjectFactory* factory)
 {
-  SCF_CONSTRUCT_IBASE (NULL);
+  SCF_CONSTRUCT_IBASE (0);
+  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiBallState);
+#ifndef CS_USE_NEW_RENDERER
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiVertexBufferManagerClient);
+#endif
   csBallMeshObject::factory = factory;
-  logparent = NULL;
+
+  scfiPolygonMesh.SetBall (this);
+  scfiObjectModel.SetPolygonMeshBase (&scfiPolygonMesh);
+  scfiObjectModel.SetPolygonMeshColldet (&scfiPolygonMesh);
+  scfiObjectModel.SetPolygonMeshViscull (0);
+  scfiObjectModel.SetPolygonMeshShadows (0);
+
+  logparent = 0;
   initialized = false;
   cur_cameranr = -1;
   cur_movablenr = -1;
@@ -66,40 +107,77 @@ csBallMeshObject::csBallMeshObject (iMeshObjectFactory* factory)
   max_radius.Set (0.0f, 0.0f, 0.0f);
   shift.Set (0.0f, 0.0f, 0.0f);
   verts_circle = 6;
-  material = NULL;
+  material = 0;
   MixMode = 0;
-  vis_cb = NULL;
-  top_normals = NULL;
-  ball_vertices = NULL;
-  ball_colors = NULL;
-  ball_texels = NULL;
-  top_mesh.triangles = NULL;
-  top_mesh.vertex_fog = NULL;
-  shapenr = 0;
+  vis_cb = 0;
+  top_normals = 0;
+  ball_vertices = 0;
+  ball_colors = 0;
+  ball_texels = 0;
+#ifdef CS_USE_NEW_RENDERER
+  ball_indices = 0;
+#else
+  top_mesh.triangles = 0;
+  top_mesh.vertex_fog = 0;
+  vbufmgr = 0;
+#endif
   reversed = false;
   toponly = false;
   cyl_mapping = false;
   do_lighting = true;
+  generated_colors = false;
   color.red = 0.0f;
   color.green = 0.0f;
   color.blue = 0.0f;
   current_lod = 1.0f;
   current_features = 0;
-  vbuf = NULL;
-  vbufmgr = NULL;
+  polygons = 0;
+  
+#ifdef CS_USE_NEW_RENDERER
+  g3d = CS_QUERY_REGISTRY (
+  	((csBallMeshObjectFactory*)factory)->object_reg, iGraphics3D);
+  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (
+    	((csBallMeshObjectFactory*)factory)->object_reg, 
+	"crystalspace.shared.stringset", iStringSet);
+
+  vertex_name = strings->Request ("vertices");
+  texel_name = strings->Request ("texture coordinates");
+  normal_name = strings->Request ("normals");
+  color_name = strings->Request ("colors");
+  index_name = strings->Request ("indices");
+
+  ball_vertices_dirty_flag = false;
+  ball_texels_dirty_flag = false;
+  ball_normals_dirty_flag = false;
+  ball_colors_dirty_flag = false;
+  ball_triangle_dirty_flag = false;
+
+  svcontext.AttachNew (new csShaderVariableContext);
+#endif
 }
 
 csBallMeshObject::~csBallMeshObject ()
 {
   if (vis_cb) vis_cb->DecRef ();
-  if (vbufmgr) vbufmgr->RemoveClient (&scfiVertexBufferManagerClient);
-  if (vbuf) vbuf->DecRef ();
   delete[] top_normals;
   delete[] ball_vertices;
   delete[] ball_colors;
   delete[] ball_texels;
+  delete[] polygons;
+#ifdef CS_USE_NEW_RENDERER
+  delete[] ball_indices;
+#else
   delete[] top_mesh.triangles;
   delete[] top_mesh.vertex_fog;
+  if (vbufmgr) vbufmgr->RemoveClient (&scfiVertexBufferManagerClient);
+#endif
+
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel);
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiBallState);
+#ifndef CS_USE_NEW_RENDERER
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiVertexBufferManagerClient);
+#endif
+  SCF_DESTRUCT_IBASE();
 }
 
 void csBallMeshObject::GetTransformedBoundingBox (long cameranr,
@@ -148,7 +226,7 @@ void csBallMeshObject::SetRadius (float radiusx, float radiusy, float radiusz)
   csBallMeshObject::radiusy = radiusy;
   csBallMeshObject::radiusz = radiusz;
   max_radius.Set (radiusx, radiusy, radiusz);
-  shapenr++;
+  scfiObjectModel.ShapeChanged ();
 }
 
 float csBallMeshObject::GetScreenBoundingBox (long cameranr,
@@ -189,8 +267,12 @@ float csBallMeshObject::GetScreenBoundingBox (long cameranr,
   return cbox.MaxZ ();
 }
 
+#ifdef CS_USE_NEW_RENDERER
+void csBallMeshObject::GenerateSphere (int num)
+#else
 void csBallMeshObject::GenerateSphere (int num, G3DTriangleMesh& mesh,
-    csVector3*& normals)
+                     csVector3*& normals)
+#endif
 {
   int num_vertices = 0;
   int num_triangles = 0;
@@ -378,13 +460,21 @@ void csBallMeshObject::GenerateSphere (int num, G3DTriangleMesh& mesh,
     }
 
   // Scale and shift all the vertices.
+#ifdef CS_USE_NEW_RENDERER
+  top_normals = new csVector3[num_vertices];
+#else
   normals = new csVector3[num_vertices];
+#endif
   for (i = 0 ; i < num_vertices ; i++)
   {
     vertices[i].x *= radiusx;
     vertices[i].y *= radiusy;
     vertices[i].z *= radiusz;
+#ifdef CS_USE_NEW_RENDERER
+    top_normals[i] = vertices[i].Unit ();
+#else
     normals[i] = vertices[i].Unit ();
+#endif
     vertices[i] += shift;
   }
 
@@ -408,68 +498,192 @@ void csBallMeshObject::GenerateSphere (int num, G3DTriangleMesh& mesh,
   ball_colors = new csColor[num_vertices];
   for (i = 0 ; i < num_vertices ; i++)
     ball_colors[i].Set (1.0f, 1.0f, 1.0f);
+
+#ifdef CS_USE_NEW_RENDERER
+  ball_triangles = num_triangles;
+  ball_indices = new unsigned int[num_triangles*3];
+  memcpy (ball_indices, triangles, sizeof(csTriangle)*num_triangles);
+
+  ball_vertices_dirty_flag = true;
+  ball_texels_dirty_flag = true;
+  ball_normals_dirty_flag = true;
+  ball_colors_dirty_flag = true;
+  ball_triangle_dirty_flag = true;
+#else
   mesh.vertex_fog = new G3DFogInfo[num_vertices];
   mesh.num_triangles = num_triangles;
   mesh.triangles = new csTriangle[num_triangles];
   memcpy (mesh.triangles, triangles, sizeof(csTriangle)*num_triangles);
+#endif
 
   delete[] vertices;
   delete[] uvverts;
   delete[] triangles;
 }
 
+
 void csBallMeshObject::SetupObject ()
 {
   if (!initialized)
   {
     initialized = true;
+
+#ifdef CS_USE_NEW_RENDERER
+    delete[] ball_indices;
+    ball_indices = 0;
+#else
     SetupVertexBuffer ();
+    delete[] top_mesh.triangles;
+    delete[] top_mesh.vertex_fog;
+    top_mesh.triangles = 0;
+    top_mesh.vertex_fog = 0;
+#endif
+   
     delete[] top_normals;
     delete[] ball_vertices;
     delete[] ball_colors;
     delete[] ball_texels;
-    delete[] top_mesh.triangles;
-    delete[] top_mesh.vertex_fog;
-    top_normals = NULL;
-    ball_vertices = NULL;
-    ball_colors = NULL;
-    ball_texels = NULL;
-    top_mesh.triangles = NULL;
-    top_mesh.vertex_fog = NULL;
+    
+    top_normals = 0;
+    ball_vertices = 0;
+    ball_colors = 0;
+    ball_texels = 0;
 
+#ifdef CS_USE_NEW_RENDERER
+    GenerateSphere (verts_circle);
+#else
     GenerateSphere (verts_circle, top_mesh, top_normals);
+#endif
     object_bbox.StartBoundingBox (
       csVector3 (-radiusx, -radiusy, -radiusz) + shift);
     object_bbox.AddBoundingVertexSmart (
       csVector3 ( radiusx,  radiusy,  radiusz) + shift);
+#ifndef CS_USE_NEW_RENDERER
     top_mesh.morph_factor = 0.0f;
     top_mesh.num_vertices_pool = 1;
     top_mesh.do_morph_texels = false;
     top_mesh.do_morph_colors = false;
     top_mesh.vertex_mode = G3DTriangleMesh::VM_WORLDSPACE;
+#endif
   }
 }
 
-void csBallMeshObject::SetupVertexBuffer ()
+#ifdef CS_USE_NEW_RENDERER
+iRenderBuffer* csBallMeshObject::GetRenderBuffer (csStringID name)
 {
- if (!vbuf)
- {
-   iObjectRegistry* object_reg = ((csBallMeshObjectFactory*)factory)
-     ->object_reg;
-   iGraphics3D* g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
-   // @@@ priority should be a parameter.
-   vbufmgr = g3d->GetVertexBufferManager ();
-   g3d->DecRef ();
-   vbuf = vbufmgr->CreateBuffer (0);
-   vbufmgr->AddClient (&scfiVertexBufferManagerClient);
-   top_mesh.buffers[0] = vbuf;
- }
+  if (name == vertex_name)
+  {
+    if (ball_vertices_dirty_flag)
+    {
+      vertex_buffer = g3d->CreateRenderBuffer (
+        sizeof (csVector3)*num_ball_vertices, CS_BUF_STATIC, 
+        CS_BUFCOMP_FLOAT, 3, false);
+      ball_vertices_dirty_flag = false;
+      vertex_buffer->CopyToBuffer(ball_vertices, sizeof(csVector3)*num_ball_vertices);
+    }
+    if (vertex_buffer)
+    {
+      return vertex_buffer;
+    }
+    return 0;
+  }
+  if (name == texel_name)
+  {
+    if (ball_texels_dirty_flag)
+    {
+      texel_buffer = g3d->CreateRenderBuffer (
+        sizeof (csVector2)*num_ball_vertices, CS_BUF_STATIC, 
+        CS_BUFCOMP_FLOAT, 2, false);
+      ball_texels_dirty_flag = false;
+      texel_buffer->CopyToBuffer (ball_texels,
+      	sizeof (csVector2) * num_ball_vertices);
+    }
+    return texel_buffer;
+  }
+  if (name == normal_name)
+  {
+    if (ball_normals_dirty_flag )
+    {
+      normal_buffer = g3d->CreateRenderBuffer (
+        sizeof (csVector3)*num_ball_vertices, CS_BUF_STATIC,
+        CS_BUFCOMP_FLOAT, 3, false);
+      ball_normals_dirty_flag = false;
+      normal_buffer->CopyToBuffer (top_normals,
+      	sizeof (csVector3)*num_ball_vertices);
+    }
+    return normal_buffer;
+  }
+  if (name == color_name)
+  {
+    if (ball_colors_dirty_flag)
+    {
+      color_buffer = g3d->CreateRenderBuffer (
+        sizeof (csColor)*num_ball_vertices, CS_BUF_STATIC,
+        CS_BUFCOMP_FLOAT, 3, false);
+      ball_colors_dirty_flag = false;
+      color_buffer->CopyToBuffer (ball_colors,
+      	sizeof (csColor) * num_ball_vertices);
+    }
+    return color_buffer;
+  }
+  if (name == index_name)
+  {
+    if (ball_triangle_dirty_flag)
+    {
+      index_buffer = g3d->CreateRenderBuffer (
+        sizeof (unsigned int)*ball_triangles*3, CS_BUF_STATIC,
+        CS_BUFCOMP_UNSIGNED_INT, 1, true);
+      ball_triangle_dirty_flag = false;
+      index_buffer->CopyToBuffer (ball_indices,
+      	sizeof (unsigned int) * ball_triangles *3);
+    }
+    return index_buffer;
+  }
+  return 0;
 }
+#endif
 
-bool csBallMeshObject::DrawTest (iRenderView* rview, iMovable* movable)
+#ifdef CS_USE_NEW_RENDERER
+void csBallMeshObject::UpdateBufferSV()
 {
+  csShaderVariable *sv;
+  if (ball_vertices_dirty_flag)
+  {
+    sv = svcontext->GetVariableAdd (vertex_name);
+    sv->SetValue (GetRenderBuffer(vertex_name));
+  }
+  if (ball_texels_dirty_flag)
+  {
+    sv = svcontext->GetVariableAdd (texel_name);
+    sv->SetValue (GetRenderBuffer(texel_name));
+  }
+  if (ball_normals_dirty_flag)
+  {
+    sv = svcontext->GetVariableAdd (normal_name);
+    sv->SetValue (GetRenderBuffer(normal_name));
+  }
+  if (ball_colors_dirty_flag)
+  {
+    sv = svcontext->GetVariableAdd (color_name);
+    sv->SetValue (GetRenderBuffer(color_name));
+  }
+  if (ball_triangle_dirty_flag)
+  {
+    sv = svcontext->GetVariableAdd (index_name);
+    sv->SetValue (GetRenderBuffer(index_name));
+  }
+}
+#endif // CS_USE_NEW_RENDERER
+
+csRenderMesh **csBallMeshObject::GetRenderMeshes (int &num, iRenderView* rview, 
+                                                  iMovable* movable,
+						  uint32 frustum_mask)
+{
+#ifdef CS_USE_NEW_RENDERER
   SetupObject ();
-  iGraphics3D* g3d = rview->GetGraphics3D ();
+
+  num = 0;
+
   iCamera* camera = rview->GetCamera ();
 
   // First create the transformation from object to camera space directly:
@@ -478,94 +692,186 @@ bool csBallMeshObject::DrawTest (iRenderView* rview, iMovable* movable)
   // ->
   //   C = Mwc * (Mow * O - Vow - Vwc)
   //   C = Mwc * Mow * O - Mwc * (Vow + Vwc)
-  csReversibleTransform tr_o2c = camera->GetTransform ()
-    	* movable->GetFullTransform ().GetInverse ();
+  csReversibleTransform tr_o2c;
+  tr_o2c = camera->GetTransform ();
+  if (!movable->IsFullTransformIdentity ())
+    tr_o2c /= movable->GetFullTransform ();
 
-#if 1
-  csVector3 radius;
-  csSphere sphere;
-  GetRadius (radius, sphere.GetCenter ());
-  float max_radius = radius.x;
-  if (max_radius < radius.y) max_radius = radius.y;
-  if (max_radius < radius.z) max_radius = radius.z;
-  sphere.SetRadius (max_radius);
   int clip_portal, clip_plane, clip_z_plane;
-  if (rview->ClipBSphere (tr_o2c, sphere, clip_portal, clip_plane,
-  	clip_z_plane) == false)
-    return false;
+  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
+  	clip_z_plane);
+  csVector3 camera_origin = tr_o2c.GetT2OTranslation ();
+
+  if (((csBallMeshObjectFactory*)factory)->light_mgr)
+  {
+    const csArray<iLight*>& relevant_lights
+      = ((csBallMeshObjectFactory*)factory)->light_mgr
+      ->GetRelevantLights (logparent, -1, false);
+    UpdateLighting (relevant_lights, movable);
+  }
+
+  iMaterialWrapper* mater = material;
+
+  if (!mater)
+  {
+    printf ("INTERNAL ERROR: mesh used without material!\n");
+    return 0;
+  }
+  UpdateBufferSV ();
+  mater->Visit ();
+
+  bool rmCreated;
+  csRenderMesh*& meshPtr = rmHolder.GetUnusedMesh (rmCreated,
+    rview->GetCurrentFrameNumber ());
+  
+  meshPtr->mixmode = MixMode;
+  meshPtr->clip_portal = clip_portal;
+  meshPtr->clip_plane = clip_plane;
+  meshPtr->clip_z_plane = clip_z_plane;
+  meshPtr->do_mirror = camera->IsMirrored ();  
+  meshPtr->meshtype = CS_MESHTYPE_TRIANGLES;
+  meshPtr->indexstart = 0;
+  meshPtr->indexend = ball_triangles * 3;
+  meshPtr->material = mater;
+  meshPtr->object2camera = tr_o2c;
+  meshPtr->camera_origin = camera_origin;
+  if (rmCreated)
+  {
+    meshPtr->variablecontext = svcontext;
+  }
+  meshPtr->geometryInstance = (void*)factory;
+  
+  num = 1;
+  return &meshPtr;
 #else
-  float fov = camera->GetFOV ();
-  float shiftx = camera->GetShiftX ();
-  float shifty = camera->GetShiftY ();
+  num = 0;
+  return 0;
+#endif
+}
+
+#ifndef CS_USE_NEW_RENDERER
+
+void csBallMeshObject::SetupVertexBuffer ()
+{
+  if (!vbuf)
+  {
+    iObjectRegistry* object_reg = ((csBallMeshObjectFactory*)factory)
+      ->object_reg;
+    csRef<iGraphics3D> g3d (CS_QUERY_REGISTRY (object_reg, iGraphics3D));
+    // @@@ priority should be a parameter.
+    vbufmgr = g3d->GetVertexBufferManager ();
+    vbuf = vbufmgr->CreateBuffer (0);
+    vbufmgr->AddClient (&scfiVertexBufferManagerClient);
+    top_mesh.buffers[0] = vbuf;
+  }
+}
 
-  // Test visibility of entire ball by clipping bounding box against clipper.
-  // There are three possibilities:
-  //	1. box is not visible -> ball is not visible.
-  //	2. box is entirely visible -> ball is visible and need not be clipped.
-  //	3. box is partially visible -> ball is visible and needs to be clipped
-  //	   if rview has do_clip_plane set to true.
-  csBox2 sbox;
-  csBox3 cbox;
-  if (GetScreenBoundingBox (camera->GetCameraNumber (),
-  	movable->GetUpdateNumber (), fov, shiftx, shifty,
-  	tr_o2c, sbox, cbox) < 0)
-    return false;
-  int clip_portal, clip_plane, clip_z_plane;
-  if (rview->ClipBBox (sbox, cbox, clip_portal, clip_plane,
-  	clip_z_plane) == false)
-    return false;
 #endif
 
+bool csBallMeshObject::DrawTest (iRenderView* rview, iMovable* movable,
+	uint32 frustum_mask)
+{
+  SetupObject ();
+
+  iCamera* camera = rview->GetCamera ();
+
+  // First create the transformation from object to camera space directly:
+  //   W = Mow * O - Vow;
+  //   C = Mwc * (W - Vwc)
+  // ->
+  //   C = Mwc * (Mow * O - Vow - Vwc)
+  //   C = Mwc * Mow * O - Mwc * (Vow + Vwc)
+  csReversibleTransform tr_o2c;
+  tr_o2c = camera->GetTransform ();
+  if (!movable->IsFullTransformIdentity ())
+    tr_o2c /= movable->GetFullTransform ();
+
+  int clip_portal, clip_plane, clip_z_plane;
+  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
+  	clip_z_plane);
+  csVector3 camera_origin = tr_o2c.GetT2OTranslation ();
+
+#ifndef CS_USE_NEW_RENDERER
+  iGraphics3D* g3d = rview->GetGraphics3D ();
   g3d->SetObjectToCamera (&tr_o2c);
   top_mesh.clip_portal = clip_portal;
   top_mesh.clip_plane = clip_plane;
   top_mesh.clip_z_plane = clip_z_plane;
   top_mesh.do_mirror = camera->IsMirrored ();
+#endif
+
+  if (((csBallMeshObjectFactory*)factory)->light_mgr)
+  {
+    const csArray<iLight*>& relevant_lights
+    	= ((csBallMeshObjectFactory*)factory)->light_mgr
+    	->GetRelevantLights (logparent, -1, false);
+    UpdateLighting (relevant_lights, movable);
+  }
+
   return true;
 }
 
-void csBallMeshObject::UpdateLighting (iLight** lights, int num_lights,
+void csBallMeshObject::UpdateLighting (const csArray<iLight*>& lights,
     iMovable* movable)
 {
-  SetupObject ();
+  if (generated_colors) return;
+
+#ifdef CS_USE_NEW_RENDERER
+  ball_colors_dirty_flag = true;
+#endif
 
   int i, l;
   csColor* colors = ball_colors;
 
-  // Set all colors to ambient light (@@@ NEED TO GET AMBIENT!)
+  // Set all colors to ambient light.
+  csColor col;
+  if (((csBallMeshObjectFactory*)factory)->engine)
+  {
+    ((csBallMeshObjectFactory*)factory)->engine->GetAmbientLight (col);
+    col += color;
+    iSector* sect = movable->GetSectors ()->Get (0);
+    if (sect)
+      col += sect->GetDynamicAmbientLight ();
+  }
+  else
+  {
+    col = color;
+  }
   for (i = 0 ; i < num_ball_vertices ; i++)
-    colors[i] = color;
+    colors[i] = col;
+
   if (!do_lighting) return;
-    // @@@ it is not effiecient to do this all the time.
+    // @@@ it is not efficient to do this all the time.
 
   // Do the lighting.
-  csVector3 obj_center (0);
   csReversibleTransform trans = movable->GetFullTransform ();
-  csVector3 wor_center = trans.This2Other (obj_center);
+  // the object center in world coordinates. "0" because the object
+  // center in object space is obviously at (0,0,0).
   csColor color;
+  int num_lights = lights.Length ();
   for (l = 0 ; l < num_lights ; l++)
   {
     iLight* li = lights[l];
     // Compute light position in object coordinates
     csVector3 wor_light_pos = li->GetCenter ();
-    float wor_sq_dist = csSquaredDist::PointPoint (wor_light_pos, wor_center);
-    if (wor_sq_dist >= li->GetSquaredRadius ()) continue;
-
     csVector3 obj_light_pos = trans.Other2This (wor_light_pos);
-    float obj_sq_dist = csSquaredDist::PointPoint (obj_light_pos, obj_center);
-    float in_obj_dist = qisqrt (obj_sq_dist);
-
-    csVector3 obj_light_dir = (obj_light_pos - obj_center);
+    float obj_sq_dist = csSquaredDist::PointPoint (obj_light_pos, 0);
+    if (obj_sq_dist >= li->GetInfluenceRadiusSq ()) continue;
+    float in_obj_dist = (obj_sq_dist >= SMALL_EPSILON)
+    	? qisqrt (obj_sq_dist)
+	: 1.0f;
 
     csColor light_color = li->GetColor () * (256.0f / CS_NORMAL_LIGHT_LEVEL)
-      * li->GetBrightnessAtDistance (qsqrt (wor_sq_dist));
+      * li->GetBrightnessAtDistance (qsqrt (obj_sq_dist));
 
     for (i = 0; i < num_ball_vertices; i++)
     {
       csVector3 normal = top_normals[i];
       float cosinus;
       if (obj_sq_dist < SMALL_EPSILON) cosinus = 1;
-      else cosinus = obj_light_dir * normal;
+      else cosinus = obj_light_pos * normal; 
+      // because the vector from the object center to the light center
+      // in object space is equal to the position of the light
 
       if (cosinus > 0)
       {
@@ -599,6 +905,7 @@ bool csBallMeshObject::Draw (iRenderView* rview, iMovable* /*movable*/,
 
   if (vis_cb) if (!vis_cb->BeforeDrawing (this, rview)) return false;
 
+#ifndef CS_USE_NEW_RENDERER
   iGraphics3D* g3d = rview->GetGraphics3D ();
 
   // Prepare for rendering.
@@ -608,13 +915,15 @@ bool csBallMeshObject::Draw (iRenderView* rview, iMovable* /*movable*/,
   material->Visit ();
   top_mesh.mat_handle = mat;
   top_mesh.use_vertex_color = true;
-  top_mesh.mixmode = MixMode | CS_FX_GOURAUD;
+  top_mesh.mixmode = MixMode;
   CS_ASSERT (!vbuf->IsLocked ());
   vbufmgr->LockBuffer (vbuf,
-  	ball_vertices, ball_texels, ball_colors, num_ball_vertices, 0);
+    ball_vertices, ball_texels, ball_colors, num_ball_vertices,
+    0, object_bbox);
   rview->CalculateFogMesh (g3d->GetObjectToCamera (), top_mesh);
   g3d->DrawTriangleMesh (top_mesh);
   vbufmgr->UnlockBuffer (vbuf);
+#endif
 
   return true;
 }
@@ -629,7 +938,7 @@ void csBallMeshObject::HardTransform (const csReversibleTransform& t)
 {
   shift = t.This2Other (shift);
   initialized = false;
-  shapenr++;
+  scfiObjectModel.ShapeChanged ();
 }
 
 bool csBallMeshObject::HitBeamOutline (const csVector3& start,
@@ -640,8 +949,14 @@ bool csBallMeshObject::HitBeamOutline (const csVector3& start,
   // will be a bit faster than its more accurate cousin (below).
 
   csSegment3 seg (start, end);
-  int i, max = top_mesh.num_triangles;
+  int i, max;
+#ifdef CS_USE_NEW_RENDERER
+  csTriangle *tr = (csTriangle*)ball_indices;
+  max = ball_triangles;
+#else
   csTriangle *tr = top_mesh.triangles;
+  max = top_mesh.num_triangles;
+#endif
   csVector3 *vrt = ball_vertices;
   for (i = 0 ; i < max ; i++)
   {
@@ -658,61 +973,70 @@ bool csBallMeshObject::HitBeamOutline (const csVector3& start,
 }
 
 bool csBallMeshObject::HitBeamObject(const csVector3& start,
-  const csVector3& end, csVector3& isect, float *pr)
+  const csVector3& end, csVector3& isect, float *pr, int* polygon_idx)
 {
+  if (polygon_idx) *polygon_idx = -1;
   // This is the slow version. Use for an accurate hit on the object.
   // It will cycle through every triangle in the mesh serching for the
   // closest intersection. Slower, but returns the closest hit.
   // Usegae is optional.
 
   csSegment3 seg (start, end);
-  int i, max = top_mesh.num_triangles;
+  int i, max;
   float tot_dist = csSquaredDist::PointPoint (start, end);
   float dist, temp;
   float itot_dist = 1.0f / tot_dist;
   dist = temp = tot_dist;
   csVector3 *vrt = ball_vertices, tmp;
+#ifdef CS_USE_NEW_RENDERER
+  csTriangle *tr = (csTriangle*)ball_indices;
+  max = ball_triangles;
+#else
   csTriangle *tr = top_mesh.triangles;
+  max = top_mesh.num_triangles;
+#endif
   for (i = 0; i < max; i++)
   {
     if (csIntersect3::IntersectTriangle (vrt[tr[i].a], vrt[tr[i].b],
     	vrt[tr[i].c], seg, tmp))
     {
-      if ( dist > (temp = csSquaredDist::PointPoint (start, tmp)))
+      temp = csSquaredDist::PointPoint (start, tmp);
+      if (temp < dist)
       {
-          isect = tmp;
-	  dist = temp;
-          if (pr) *pr = qsqrt(dist * itot_dist);
+        isect = tmp;
+	dist = temp;
       }
     }
   }
-  if (dist == tot_dist)
+  if (pr) *pr = qsqrt(dist * itot_dist);
+  if (dist >= tot_dist)
       return false;
   return true;
 }
 
+#ifndef CS_USE_NEW_RENDERER
 void csBallMeshObject::eiVertexBufferManagerClient::ManagerClosing ()
 {
   if (scfParent->vbuf)
   {
-    scfParent->vbuf->DecRef ();
-    scfParent->vbuf = NULL;
-    scfParent->vbufmgr = NULL;
+    scfParent->vbuf = 0;
+    scfParent->vbufmgr = 0;
   }
 }
+#endif
 
 
 /// interpolate a gradient
 static void GetGradientColor(float **gradient, float val, csColor& col)
 {
-  if(gradient == NULL || gradient[0] == NULL)
+  if(gradient == 0 || gradient[0] == 0)
   {
     col.Set(0.0f, 0.0f, 0.0f);
     return;
   }
   int entry = 0;
   while( gradient[entry] && (gradient[entry][0] < val) ) entry++;
-  if(gradient[entry] == NULL)
+  if(gradient[entry] == 0)
   {
     if(entry>0) col.Set( gradient[entry-1][1], gradient[entry-1][2],
       gradient[entry-1][3]);
@@ -754,6 +1078,7 @@ void csBallMeshObject::ApplyVertGradient(float horizon_height,
     GetGradientColor(gradient, val, color);
     ball_colors[i] = color;
   }
+  generated_colors = true;
 }
 
 void csBallMeshObject::ApplyLightSpot(const csVector3& position, float size,
@@ -767,7 +1092,7 @@ void csBallMeshObject::ApplyLightSpot(const csVector3& position, float size,
   /// see if gradient is given
   float sun1[] = {0.0f, 1.0f, 1.0f, 0.6f};
   float sun2[] = {1.0f, 1.0f, 0.8f, 0.6f};
-  float *sungrad[] = {sun1, sun2, NULL};
+  float *sungrad[] = {sun1, sun2, 0};
   float **grad = sungrad;
   if(gradient) grad = gradient;
   /// compute the max distance for the lightspot given radius.
@@ -787,30 +1112,31 @@ void csBallMeshObject::ApplyLightSpot(const csVector3& position, float size,
     ball_colors[i].Clamp(2.0f, 2.0f, 2.0f);
     ball_colors[i].ClampDown();
   }
+  generated_colors = true;
 }
 
 
-void csBallMeshObject::PaintSky(float time, float **dayvert, float **nightvert,
+void csBallMeshObject::PaintSky (float time, float **dayvert, float **nightvert,
   float **topsun, float **sunset)
 {
   // apply defaults if needed
   float sky1[] = {0.0f, 0.5f, 0.6f, 1.0f};
   float sky2[] = {1.0f, 0.1f, 0.3f, 0.8f};
-  float* def_dayvert[] = {sky1, sky2, NULL};
+  float* def_dayvert[] = {sky1, sky2, 0};
 
   float night1[] = {0.0f, 0.1f, 0.1f, 0.1f};
   float night2[] = {1.0f, 0.0f, 0.0f, 0.0f};
-  float* def_nightvert[] = {night1, night2, NULL};
+  float* def_nightvert[] = {night1, night2, 0};
 
   float sun0[] = {0.0f, 1.0f, 1.0f, 0.6f};
   float sun1[] = {0.5f, 1.0f, 0.9f, 0.6f};
   float sun2[] = {1.0f, 1.0f, 0.8f, 0.6f};
-  float *def_topsun[] = {sun0, sun1, sun2, NULL};
+  float *def_topsun[] = {sun0, sun1, sun2, 0};
 
   float sunset0[] = {0.0f, 0.9f, 0.9f, -0.9f};
   float sunset1[] = {0.5f, 0.1f, -0.8f, -0.6f};
   float sunset2[] = {1.0f, 1.0f, -0.9f, 1.0f};
-  float* def_sunset[] = {sunset0, sunset1, sunset2, NULL};
+  float* def_sunset[] = {sunset0, sunset1, sunset2, 0};
 
   if(!dayvert) dayvert = def_dayvert;
   if(!nightvert) nightvert = def_nightvert;
@@ -914,6 +1240,7 @@ void csBallMeshObject::PaintSky(float time, float **dayvert, float **nightvert,
   for(i=0; i<num_ball_vertices; i++)
     if(ball_vertices[i].y < shift.y)
       ball_colors[i] = apparent;
+  generated_colors = true;
 
   /// delete gradients
   for(i=0; i<vertlen; i++)
@@ -924,6 +1251,55 @@ void csBallMeshObject::PaintSky(float time, float **dayvert, float **nightvert,
   delete[] applysun;
 }
 
+csMeshedPolygon* csBallMeshObject::GetPolygons ()
+{
+  if (!polygons)
+  {
+#ifdef CS_USE_NEW_RENDERER
+    csTriangle* triangles = (csTriangle*)ball_indices;
+    polygons = new csMeshedPolygon [ball_triangles];
+    int i;
+    for (i = 0 ; i < ball_triangles ; i++)
+#else
+    csTriangle* triangles = top_mesh.triangles;
+    polygons = new csMeshedPolygon [top_mesh.num_triangles];int i;
+    for (i = 0 ; i < top_mesh.num_triangles ; i++)
+#endif
+    
+    {
+      polygons[i].num_vertices = 3;
+      polygons[i].vertices = &triangles[i].a;
+    }
+  }
+
+  return polygons;
+}
+
+void csBallMeshObject::PolyMesh::Cleanup ()
+{
+  delete[] ball->polygons;
+  ball->polygons = 0;
+}
+
+int csBallMeshObject::PolyMesh::GetVertexCount ()
+{
+  return ball->GetVertexCount ();
+}
+
+csVector3* csBallMeshObject::PolyMesh::GetVertices ()
+{
+  return ball->GetVertices ();
+}
+
+int csBallMeshObject::PolyMesh::GetPolygonCount ()
+{
+  return ball->GetTriangleCount ();
+}
+
+csMeshedPolygon* csBallMeshObject::PolyMesh::GetPolygons ()
+{
+  return ball->GetPolygons ();
+}
 
 //----------------------------------------------------------------------
 
@@ -936,19 +1312,23 @@ csBallMeshObjectFactory::csBallMeshObjectFactory (iBase *pParent,
 {
   SCF_CONSTRUCT_IBASE (pParent);
   csBallMeshObjectFactory::object_reg = object_reg;
-  logparent = NULL;
+  logparent = 0;
+  csRef<iEngine> eng = CS_QUERY_REGISTRY (object_reg, iEngine);
+  engine = eng;	// We don't want a circular reference.
+  light_mgr = CS_QUERY_REGISTRY (object_reg, iLightManager);
 }
 
 csBallMeshObjectFactory::~csBallMeshObjectFactory ()
 {
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObject* csBallMeshObjectFactory::NewInstance ()
+csPtr<iMeshObject> csBallMeshObjectFactory::NewInstance ()
 {
   csBallMeshObject* cm = new csBallMeshObject ((iMeshObjectFactory*)this);
-  iMeshObject* im = SCF_QUERY_INTERFACE (cm, iMeshObject);
-  im->DecRef ();
-  return im;
+  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (cm, iMeshObject));
+  cm->DecRef ();
+  return csPtr<iMeshObject> (im);
 }
 
 //----------------------------------------------------------------------
@@ -964,10 +1344,6 @@ SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csBallMeshObjectType)
 
-SCF_EXPORT_CLASS_TABLE (ball)
-  SCF_EXPORT_CLASS (csBallMeshObjectType, "crystalspace.mesh.object.ball",
-    "Crystal Space Ball Mesh Type")
-SCF_EXPORT_CLASS_TABLE_END
 
 csBallMeshObjectType::csBallMeshObjectType (iBase* pParent)
 {
@@ -977,13 +1353,17 @@ csBallMeshObjectType::csBallMeshObjectType (iBase* pParent)
 
 csBallMeshObjectType::~csBallMeshObjectType ()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObjectFactory* csBallMeshObjectType::NewFactory ()
+csPtr<iMeshObjectFactory> csBallMeshObjectType::NewFactory ()
 {
   csBallMeshObjectFactory* cm = new csBallMeshObjectFactory (this,
   	object_reg);
-  iMeshObjectFactory* ifact = SCF_QUERY_INTERFACE (cm, iMeshObjectFactory);
-  ifact->DecRef ();
-  return ifact;
+  csRef<iMeshObjectFactory> ifact (
+  	SCF_QUERY_INTERFACE (cm, iMeshObjectFactory));
+  cm->DecRef ();
+  return csPtr<iMeshObjectFactory> (ifact);
 }
+
