@@ -16,11 +16,12 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define CS_SYSDEF_PROVIDE_ALLOCA
 #include "cssysdef.h"
 #include "csgeom/math3d.h"
 #include "csgeom/math2d.h"
 #include "csgeom/box.h"
+#include "csgeom/transfrm.h"
+#include "cstool/rbuflock.h"
 #include "spr2d.h"
 #include "iengine/movable.h"
 #include "iengine/rview.h"
@@ -32,16 +33,22 @@
 #include "igeom/clip2d.h"
 #include "iengine/engine.h"
 #include "iengine/light.h"
+#include "iutil/objreg.h"
 #include "qsqrt.h"
 
 CS_IMPLEMENT_PLUGIN
 
 SCF_IMPLEMENT_IBASE (csSprite2DMeshObject)
   SCF_IMPLEMENTS_INTERFACE (iMeshObject)
+  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iObjectModel)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iSprite2DState)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iParticle)
 SCF_IMPLEMENT_IBASE_END
 
+SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObject::ObjectModel)
+  SCF_IMPLEMENTS_INTERFACE (iObjectModel)
+SCF_IMPLEMENT_EMBEDDED_IBASE_END
+
 SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObject::Sprite2DState)
   SCF_IMPLEMENTS_INTERFACE (iSprite2DState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
@@ -50,29 +57,50 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObject::Particle)
   SCF_IMPLEMENTS_INTERFACE (iParticle)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
+#ifdef CS_USE_NEW_RENDERER
+SCF_IMPLEMENT_IBASE (csSprite2DMeshObject::eiShaderVariableAccessor)
+  SCF_IMPLEMENTS_INTERFACE (iShaderVariableAccessor)
+SCF_IMPLEMENT_IBASE_END
+
+csStringID csSprite2DMeshObject::vertex_name = csInvalidStringID;
+csStringID csSprite2DMeshObject::texel_name = csInvalidStringID;
+csStringID csSprite2DMeshObject::color_name = csInvalidStringID;
+csStringID csSprite2DMeshObject::index_name = csInvalidStringID;
+#endif
+
 csSprite2DMeshObject::csSprite2DMeshObject (csSprite2DMeshObjectFactory* factory)
 {
-  SCF_CONSTRUCT_IBASE (NULL);
+  SCF_CONSTRUCT_IBASE (0);
+  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiSprite2DState);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiParticle);
   csSprite2DMeshObject::factory = factory;
-  logparent = NULL;
+  logparent = 0;
   ifactory = SCF_QUERY_INTERFACE (factory, iMeshObjectFactory);
   material = factory->GetMaterialWrapper ();
   lighting = factory->HasLighting ();
   MixMode = factory->GetMixMode ();
   initialized = false;
-  vis_cb = NULL;
-  shapenr = 0;
+  vis_cb = 0;
   current_lod = 1;
   current_features = 0;
-  o2t.Identity();
+  uvani = 0;
+#ifdef CS_USE_NEW_RENDERER
+  vertices_dirty = true;
+  texels_dirty = true;
+  colors_dirty = true;
+  indicesSize = (size_t)-1;
+#endif
 }
 
 csSprite2DMeshObject::~csSprite2DMeshObject ()
 {
   if (vis_cb) vis_cb->DecRef ();
-  if (ifactory) ifactory->DecRef ();
+  delete uvani;
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel);
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiSprite2DState);
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiParticle);
+  SCF_DESTRUCT_IBASE ();
 }
 
 void csSprite2DMeshObject::SetupObject ()
@@ -99,23 +127,63 @@ void csSprite2DMeshObject::SetupObject ()
     }
     float max_dist = qsqrt (max_sq_dist);
     radius.Set (max_dist, max_dist, max_dist);
+
+#ifdef CS_USE_NEW_RENDERER
+    if ((vertex_name == csInvalidStringID) ||
+      (color_name == csInvalidStringID) ||
+      (texel_name == csInvalidStringID) ||
+      (index_name  == csInvalidStringID))
+    {
+      csRef<iStringSet> strings = 
+	CS_QUERY_REGISTRY_TAG_INTERFACE (factory->object_reg,
+	"crystalspace.shared.stringset", iStringSet);
+      vertex_name = strings->Request ("vertices");
+      texel_name = strings->Request ("texture coordinates");
+      color_name = strings->Request ("colors");
+      index_name = strings->Request ("indices");
+    }
+    
+    svcontext.AttachNew (new csShaderVariableContext ());
+    csRef<iShaderVariableAccessor> accessor;
+    accessor.AttachNew (new eiShaderVariableAccessor (this));
+    csShaderVariable* sv;
+    sv = svcontext->GetVariableAdd (index_name);
+    sv->SetAccessor (accessor);
+    sv = svcontext->GetVariableAdd (vertex_name);
+    sv->SetAccessor (accessor);
+    sv = svcontext->GetVariableAdd (texel_name);
+    sv->SetAccessor (accessor);
+    sv = svcontext->GetVariableAdd (color_name);
+    sv->SetAccessor (accessor);
+#endif
   }
 }
 
-bool csSprite2DMeshObject::DrawTest (iRenderView* rview, iMovable* movable)
+static csVector3 cam;
+
+bool csSprite2DMeshObject::DrawTest (iRenderView* rview, iMovable* movable,
+	uint32)
 {
   SetupObject ();
 
   // Camera transformation for the single 'position' vector.
-  cam = rview->GetCamera ()->GetTransform ().Other2This (movable->GetFullPosition ());
+  cam = rview->GetCamera ()->GetTransform ().Other2This (
+  	movable->GetFullPosition ());
   if (cam.z < SMALL_Z) return false;
+
+  if (factory->light_mgr)
+  {
+    const csArray<iLight*>& relevant_lights = factory->light_mgr
+    	->GetRelevantLights (logparent, -1, false);
+    UpdateLighting (relevant_lights, movable, csVector3 (0.0f));
+  }
+
   return true;
 }
 
-void csSprite2DMeshObject::UpdateLighting (iLight** lights, int num_lights,
+void csSprite2DMeshObject::UpdateLighting (const csArray<iLight*>& lights,
     const csVector3& pos)
 {
-  SetupObject ();
   if (!lighting) return;
   csColor color (0, 0, 0);
 
@@ -129,10 +197,11 @@ void csSprite2DMeshObject::UpdateLighting (iLight** lights, int num_lights,
   //}
 
   int i;
+  int num_lights = lights.Length ();
   for (i = 0; i < num_lights; i++)
   {
     csColor light_color = lights[i]->GetColor () * (256. / CS_NORMAL_LIGHT_LEVEL);
-    float sq_light_radius = lights [i]->GetSquaredRadius ();
+    float sq_light_radius = lights [i]->GetInfluenceRadiusSq ();
     // Compute light position.
     csVector3 wor_light_pos = lights [i]->GetCenter ();
     float wor_sq_dist =
@@ -149,27 +218,43 @@ void csSprite2DMeshObject::UpdateLighting (iLight** lights, int num_lights,
     vertices[i].color = vertices[i].color_init + color;
     vertices[i].color.Clamp (2, 2, 2);
   }
+#ifdef CS_USE_NEW_RENDERER
+  colors_dirty = true;
+#endif
 }
 
-void csSprite2DMeshObject::UpdateLighting (iLight** lights, int num_lights,
-    iMovable* movable)
+void csSprite2DMeshObject::UpdateLighting (const csArray<iLight*>& lights,
+    iMovable* movable, csVector3 offset)
 {
   if (!lighting) return;
   csVector3 pos = movable->GetFullPosition ();
-  UpdateLighting (lights, num_lights, pos);
+  UpdateLighting (lights, pos + offset);
 }
 
-#define INTERPOLATE1(component) \
-  g3dpoly->vertices [i].component = inpoly [vt].component + \
-    t * (inpoly [vt2].component - inpoly [vt].component);
+#define INTERPOLATE1_S(var) \
+  g3dpoly->var [i]= inpoly_##var [vt]+ \
+    t * (inpoly_##var [vt2]- inpoly_##var [vt]);
 
-#define INTERPOLATE(component) \
+#define INTERPOLATE1(var,component) \
+  g3dpoly->var [i].component = inpoly_##var [vt].component + \
+    t * (inpoly_##var [vt2].component - inpoly_##var [vt].component);
+
+#define INTERPOLATE_S(var) \
 { \
-  float v1 = inpoly [edge_from [0]].component + \
-    t1 * (inpoly [edge_to [0]].component - inpoly [edge_from [0]].component); \
-  float v2 = inpoly [edge_from [1]].component + \
-    t2 * (inpoly [edge_to [1]].component - inpoly [edge_from [1]].component); \
-  g3dpoly->vertices [i].component = v1 + t * (v2 - v1); \
+  float v1 = inpoly_##var [edge_from [0]] + \
+    t1 * (inpoly_##var [edge_to [0]] - inpoly_##var [edge_from [0]]); \
+  float v2 = inpoly_##var [edge_from [1]] + \
+    t2 * (inpoly_##var [edge_to [1]] - inpoly_##var [edge_from [1]]); \
+  g3dpoly->var [i] = v1 + t * (v2 - v1); \
+}
+
+#define INTERPOLATE(var,component) \
+{ \
+  float v1 = inpoly_##var [edge_from [0]].component + \
+    t1 * (inpoly_##var [edge_to [0]].component - inpoly_##var [edge_from [0]].component); \
+  float v2 = inpoly_##var [edge_from [1]].component + \
+    t2 * (inpoly_##var [edge_to [1]].component - inpoly_##var [edge_from [1]].component); \
+  g3dpoly->var [i].component = v1 + t * (v2 - v1); \
 }
 
 static void PreparePolygonFX2 (G3DPolygonDPFX* g3dpoly,
@@ -178,43 +263,44 @@ static void PreparePolygonFX2 (G3DPolygonDPFX* g3dpoly,
 {
   // first we copy the first texture coordinates to a local buffer
   // to avoid that they are overwritten when interpolating.
-  ALLOC_STACK_ARRAY (inpoly, G3DTexturedVertex, orig_num_vertices);
+  CS_ALLOC_STACK_ARRAY (csVector2, inpoly_vertices, orig_num_vertices);
+  CS_ALLOC_STACK_ARRAY (csVector2, inpoly_texels, orig_num_vertices);
+  CS_ALLOC_STACK_ARRAY (csColor, inpoly_colors, orig_num_vertices);
+  CS_ALLOC_STACK_ARRAY (float, inpoly_z, orig_num_vertices);
   int i;
-  for (i = 0; i < orig_num_vertices; i++)
-    inpoly[i] = g3dpoly->vertices[i];
+  memcpy (inpoly_vertices, g3dpoly->vertices,
+  	orig_num_vertices * sizeof (csVector2));
+  memcpy (inpoly_texels, g3dpoly->texels,
+  	orig_num_vertices * sizeof (csVector2));
+  memcpy (inpoly_colors, g3dpoly->colors, orig_num_vertices * sizeof (csColor));
+  memcpy (inpoly_z, g3dpoly->z, orig_num_vertices * sizeof (float));
 
   int vt, vt2;
   float t;
   for (i = 0; i < num_vertices; i++)
   {
-    g3dpoly->vertices [i].x = clipped_verts [i].x;
-    g3dpoly->vertices [i].y = clipped_verts [i].y;
+    g3dpoly->vertices [i] = clipped_verts [i];
     switch (clipped_vtstats[i].Type)
     {
       case CS_VERTEX_ORIGINAL:
         vt = clipped_vtstats[i].Vertex;
-        g3dpoly->vertices [i].z = inpoly [vt].z;
-        g3dpoly->vertices [i].u = inpoly [vt].u;
-        g3dpoly->vertices [i].v = inpoly [vt].v;
+        g3dpoly->z [i] = inpoly_z [vt];
+        g3dpoly->texels [i] = inpoly_texels [vt];
 	if (gouraud)
-	{
-          g3dpoly->vertices [i].r = inpoly [vt].r;
-          g3dpoly->vertices [i].g = inpoly [vt].g;
-          g3dpoly->vertices [i].b = inpoly [vt].b;
-	}
+          g3dpoly->colors [i] = inpoly_colors [vt];
 	break;
       case CS_VERTEX_ONEDGE:
         vt = clipped_vtstats[i].Vertex;
 	vt2 = vt + 1; if (vt2 >= orig_num_vertices) vt2 = 0;
 	t = clipped_vtstats [i].Pos;
-	INTERPOLATE1 (z);
-	INTERPOLATE1 (u);
-	INTERPOLATE1 (v);
+	INTERPOLATE1_S (z);
+	INTERPOLATE1 (texels,x);
+	INTERPOLATE1 (texels,y);
 	if (gouraud)
 	{
-	  INTERPOLATE1 (r);
-	  INTERPOLATE1 (g);
-	  INTERPOLATE1 (b);
+	  INTERPOLATE1 (colors,red);
+	  INTERPOLATE1 (colors,green);
+	  INTERPOLATE1 (colors,blue);
 	}
 	break;
       case CS_VERTEX_INSIDE:
@@ -226,8 +312,8 @@ static void PreparePolygonFX2 (G3DPolygonDPFX* g3dpoly,
 	j1 = orig_num_vertices - 1;
 	for (j = 0; j < orig_num_vertices; j++)
 	{
-          if ((y >= inpoly [j].y && y <= inpoly [j1].y) ||
-	      (y <= inpoly [j].y && y >= inpoly [j1].y))
+          if ((y >= inpoly_vertices [j].y && y <= inpoly_vertices [j1].y) ||
+	      (y <= inpoly_vertices [j].y && y >= inpoly_vertices [j1].y))
 	  {
 	    edge_from [edge] = j;
 	    edge_to [edge] = j1;
@@ -242,23 +328,23 @@ static void PreparePolygonFX2 (G3DPolygonDPFX* g3dpoly,
 	  edge_from [1] = edge_from [0];
 	  edge_to [1] = edge_to [0];
 	}
-	G3DTexturedVertex& A = inpoly [edge_from [0]];
-	G3DTexturedVertex& B = inpoly [edge_to [0]];
-	G3DTexturedVertex& C = inpoly [edge_from [1]];
-	G3DTexturedVertex& D = inpoly [edge_to [1]];
+	csVector2& A = inpoly_vertices [edge_from [0]];
+	csVector2& B = inpoly_vertices [edge_to [0]];
+	csVector2& C = inpoly_vertices [edge_from [1]];
+	csVector2& D = inpoly_vertices [edge_to [1]];
 	float t1 = (y - A.y) / (B.y - A.y);
 	float t2 = (y - C.y) / (D.y - C.y);
 	float x1 = A.x + t1 * (B.x - A.x);
 	float x2 = C.x + t2 * (D.x - C.x);
 	t = (x - x1) / (x2 - x1);
-	INTERPOLATE (z);
-	INTERPOLATE (u);
-	INTERPOLATE (v);
+	INTERPOLATE_S (z);
+	INTERPOLATE (texels,x);
+	INTERPOLATE (texels,y);
 	if (gouraud)
 	{
-	  INTERPOLATE (r);
-	  INTERPOLATE (g);
-	  INTERPOLATE (b);
+	  INTERPOLATE (colors,red);
+	  INTERPOLATE (colors,green);
+	  INTERPOLATE (colors,blue);
 	}
 	break;
     }
@@ -295,12 +381,13 @@ bool csSprite2DMeshObject::Draw (iRenderView* rview, iMovable* /*movable*/,
 
   material->Visit ();
 
+  G3DPolygonDPFX& g3dpolyfx = factory->g3dpolyfx;
   g3dpolyfx.num = vertices.Length ();
   g3dpolyfx.mat_handle = mat;
   g3dpolyfx.mat_handle->GetTexture ()->GetMeanColor (g3dpolyfx.flat_color_r,
     g3dpolyfx.flat_color_g, g3dpolyfx.flat_color_b);
 
-  ALLOC_STACK_ARRAY (poly2d, csVector2, vertices.Length ());
+  CS_ALLOC_STACK_ARRAY (csVector2, poly2d, vertices.Length ());
   csVector2 clipped_poly2d[MAX_OUTPUT_VERTICES];
   csVertexStatus clipped_vtstats[MAX_OUTPUT_VERTICES];
 
@@ -311,32 +398,30 @@ bool csSprite2DMeshObject::Draw (iRenderView* rview, iMovable* /*movable*/,
 
   for (i = 0; i < vertices.Length (); i++)
   {
-    g3dpolyfx.vertices [i].z = iz;
+    g3dpolyfx.z [i] = iz;
     g3dpolyfx.vertices [i].x =
     poly2d [i].x = (cam.x + vertices [i].pos.x) * iza + camera->GetShiftX ();
     g3dpolyfx.vertices [i].y =
     poly2d [i].y = (cam.y + vertices [i].pos.y) * iza + camera->GetShiftY ();
-    g3dpolyfx.vertices [i].r = vertices [i].color.red;
-    g3dpolyfx.vertices [i].g = vertices [i].color.green;
-    g3dpolyfx.vertices [i].b = vertices [i].color.blue;
+    g3dpolyfx.colors [i] = vertices [i].color;
   }
 
-  if (!uvani.animate)
+  if (!uvani)
   {
     for (i = 0; i < vertices.Length (); i++)
     {
-      g3dpolyfx.vertices [i].u = vertices [i].u;
-      g3dpolyfx.vertices [i].v = vertices [i].v;
+      g3dpolyfx.texels [i].x = vertices [i].u;
+      g3dpolyfx.texels [i].y = vertices [i].v;
     }
   }
   else
   {
     int n;
-    const csVector2 *uv = uvani.GetVertices (n);
+    const csVector2 *uv = uvani->GetVertices (n);
     for (i = 0; i < n; i++)
     {
-      g3dpolyfx.vertices [i].u = uv [i].x;
-      g3dpolyfx.vertices [i].v = uv [i].y;
+      g3dpolyfx.texels [i].x = uv [i].x;
+      g3dpolyfx.texels [i].y = uv [i].y;
     }
   }
 
@@ -351,19 +436,200 @@ bool csSprite2DMeshObject::Draw (iRenderView* rview, iMovable* /*movable*/,
     	clipped_vtstats, vertices.Length (), true);
 
   rview->CalculateFogPolygon (g3dpolyfx);
-  g3dpolyfx.mixmode = MixMode | CS_FX_GOURAUD;
+  g3dpolyfx.mixmode = MixMode;
   g3d->DrawPolygonFX (g3dpolyfx);
-
   return true;
 }
 
+csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n, 
+						      iRenderView* rview, 
+						      iMovable* movable, 
+						      uint32 frustum_mask,
+						      csVector3 offset)
+{
+#ifdef CS_USE_NEW_RENDERER
+  SetupObject ();
+
+  iCamera* camera = rview->GetCamera ();
+
+  // Camera transformation for the single 'position' vector.
+  cam = rview->GetCamera ()->GetTransform ().Other2This (
+  	movable->GetFullPosition () + offset);
+  if (cam.z < SMALL_Z) 
+  {
+    n = 0;
+    return 0;
+  }
+
+  if (factory->light_mgr)
+  {
+    const csArray<iLight*>& relevant_lights = factory->light_mgr
+    	->GetRelevantLights (logparent, -1, false);
+    UpdateLighting (relevant_lights, movable, offset);
+  }
+
+  csReversibleTransform temp = camera->GetTransform ();
+  if (!movable->IsFullTransformIdentity ())
+    temp /= movable->GetFullTransform ();
+
+  int clip_portal, clip_plane, clip_z_plane;
+  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane, 
+    clip_z_plane);
+  csVector3 camera_origin = temp.GetT2OTranslation ();
+
+  csReversibleTransform tr_o2c;
+  tr_o2c.SetO2TTranslation (-temp.Other2This (offset));
+
+  bool meshCreated;
+  csRenderMesh*& rm = rmHolder.GetUnusedMesh (meshCreated,
+    rview->GetCurrentFrameNumber ());
+  if (meshCreated)
+  {
+    rm->meshtype = CS_MESHTYPE_TRIANGLEFAN;
+    rm->material = material;
+    rm->variablecontext = svcontext;
+    rm->geometryInstance = this;
+  }
+  
+  rm->mixmode = MixMode;
+  rm->clip_portal = clip_portal;
+  rm->clip_plane = clip_plane;
+  rm->clip_z_plane = clip_z_plane;
+  rm->do_mirror = false/* camera->IsMirrored () */; 
+    /*
+      Force to false as the front-face culling will let the sprite 
+      disappear. 
+     */
+  rm->indexstart = 0;
+  rm->indexend = vertices.Length();
+  rm->object2camera = tr_o2c;
+  rm->camera_origin = camera_origin;
+
+  n = 1; 
+  return &rm; 
+#endif
+  n = 0; 
+  return 0;
+}
+
+#ifdef CS_USE_NEW_RENDERER
+void csSprite2DMeshObject::PreGetShaderVariableValue (csShaderVariable* variable)
+{
+  const csStringID name = variable->GetName ();
+  if (name == index_name)
+  {
+    size_t indexSize = sizeof (uint) * vertices.Length();
+    if (!index_buffer.IsValid() || 
+      (indicesSize != indexSize))
+    {
+      index_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
+	indexSize, CS_BUF_DYNAMIC, 
+	CS_BUFCOMP_UNSIGNED_INT, 1, true));
+      variable->SetValue (index_buffer);
+
+      csRenderBufferLock<uint> indexLock (index_buffer);
+      uint* ptr = indexLock;
+
+      for (int i = 0; i < vertices.Length(); i++)
+      {
+	*ptr++ = i;
+      }
+      indicesSize = indexSize;
+    }
+  }
+  else if (name == texel_name)
+  {
+    if (texels_dirty)
+    {
+		
+	  int texels_count;
+	  const csVector2 *uvani_uv;
+	  if (!uvani)
+	    texels_count = vertices.Length ();
+	  else
+	 	uvani_uv = uvani->GetVertices (texels_count);
+	  
+	  size_t texelSize = sizeof (float) * texels_count * 2;
+      if (!texel_buffer.IsValid() || (size_t)texel_buffer->GetSize() != texelSize)
+      {
+		texel_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
+		texelSize, CS_BUF_STATIC, 
+		CS_BUFCOMP_FLOAT, 2, false));
+		variable->SetValue (texel_buffer);
+      }
+
+      csRenderBufferLock<csVector2> texelLock (texel_buffer);
+
+	  for (int i = 0; i < texels_count; i++)
+	  {
+	  	csVector2& v = texelLock[i];
+	  	if (!uvani)
+		{
+			v.x = vertices[i].u;
+			v.y = vertices[i].v;
+		}
+		else
+		{
+			v.x = uvani_uv[i].x;
+			v.y = uvani_uv[i].y;
+		}
+	  }
+      texels_dirty = false;
+    }
+  }
+  else if (name == color_name)
+  {
+    if (colors_dirty)
+    {
+      size_t color_size = sizeof (float) * vertices.Length() * 3;
+      if (!color_buffer.IsValid() ||
+	(size_t)color_buffer->GetSize() != color_size)
+      {
+	color_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
+	  color_size, CS_BUF_STATIC, 
+	  CS_BUFCOMP_FLOAT, 3, false));
+	variable->SetValue (color_buffer);
+      }
+
+      csRenderBufferLock<csColor> colorLock (color_buffer);
+
+      for (int i = 0; i < vertices.Length(); i++)
+      {
+	colorLock[i] = vertices[i].color;
+      }
+      colors_dirty = false;
+    }
+  }
+  else if (name == vertex_name)
+  {
+    if (vertices_dirty)
+    {
+      size_t vertices_size = sizeof (float) * vertices.Length() * 3;
+      if (!vertex_buffer.IsValid() ||
+	(size_t)vertex_buffer->GetSize() != vertices_size)
+      {
+	vertex_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
+	  vertices_size, CS_BUF_STATIC, 
+	  CS_BUFCOMP_FLOAT, 3, false));
+	variable->SetValue (vertex_buffer);
+      }
+
+      csRenderBufferLock<csVector3> vertexLock (vertex_buffer);
+
+      for (int i = 0; i < vertices.Length(); i++)
+      {
+	vertexLock[i].Set (vertices[i].pos.x, vertices[i].pos.y, 0.0f);
+      }
+      vertices_dirty = false;
+    }
+  }
+}
+#endif
+
 void csSprite2DMeshObject::GetObjectBoundingBox (csBox3& bbox, int /*type*/)
 {
   SetupObject ();
-  //@@@ TODO. This is an initial test....and is invalid
-  csVector2 st = bbox_2d.Min(), end = bbox_2d.Max();
-  bbox.StartBoundingBox(csVector3(st.x,st.y,0));
-  bbox.AddBoundingVertexSmart(csVector3(end.x,end.y,0));
+  bbox.Set (-radius, radius);
 }
 
 void csSprite2DMeshObject::HardTransform (const csReversibleTransform& t)
@@ -376,7 +642,6 @@ void csSprite2DMeshObject::CreateRegularVertices (int n, bool setuv)
 {
   double angle_inc = TWO_PI / n;
   double angle = 0.0;
-  vertices.SetLimit (n);
   vertices.SetLength (n);
   int i;
   for (i = 0; i < vertices.Length (); i++, angle += angle_inc)
@@ -392,20 +657,33 @@ void csSprite2DMeshObject::CreateRegularVertices (int n, bool setuv)
     vertices [i].color.Set (1, 1, 1);
     vertices [i].color_init.Set (1, 1, 1);
   }
-  shapenr++;
+#ifdef CS_USE_NEW_RENDERER
+  vertices_dirty = true;
+  texels_dirty = true;
+  colors_dirty = true;
+#endif
+  scfiObjectModel.ShapeChanged ();
 }
 
-void csSprite2DMeshObject::NextFrame (csTicks current_time)
+void csSprite2DMeshObject::NextFrame (csTicks current_time, const csVector3& /*pos*/)
 {
-  if (uvani.animate && !uvani.halted)
-    uvani.Advance (current_time);
+  if (uvani && !uvani->halted)
+#ifdef CS_USE_NEW_RENDERER
+  {
+	int old_frame_index = uvani->frameindex;
+#endif
+    uvani->Advance (current_time);
+#ifdef CS_USE_NEW_RENDERER
+	texels_dirty = old_frame_index != uvani->frameindex;
+  }
+#endif
 }
 
-void csSprite2DMeshObject::Particle::UpdateLighting (iLight** lights,
-    int num_lights, const csReversibleTransform& transform)
+void csSprite2DMeshObject::Particle::UpdateLighting (const csArray<iLight*>& lights,
+    const csReversibleTransform& transform)
 {
   csVector3 new_pos = transform.This2Other (part_pos);
-  scfParent->UpdateLighting (lights, num_lights, new_pos);
+  scfParent->UpdateLighting (lights, new_pos);
 }
 
 void csSprite2DMeshObject::Particle::Draw (iRenderView* rview,
@@ -415,9 +693,15 @@ void csSprite2DMeshObject::Particle::Draw (iRenderView* rview,
 
   // Camera transformation for the single 'position' vector.
   csVector3 new_pos = transform.This2Other (part_pos);
-  scfParent->cam = rview->GetCamera ()->GetTransform ().Other2This (new_pos);
-  if (scfParent->cam.z < SMALL_Z) return;
-  scfParent->Draw (rview, NULL, mode);
+  cam = rview->GetCamera ()->GetTransform ().Other2This (new_pos);
+  if (cam.z < SMALL_Z) return;
+  scfParent->Draw (rview, 0, mode);
+}
+    
+csRenderMesh** csSprite2DMeshObject::Particle::GetRenderMeshes (int& n, 
+	iRenderView* rview, iMovable* movable, uint32 frustum_mask)
+{
+  return scfParent->GetRenderMeshes (n, rview, movable, frustum_mask, part_pos);
 }
 
 void csSprite2DMeshObject::Particle::SetColor (const csColor& col)
@@ -429,6 +713,9 @@ void csSprite2DMeshObject::Particle::SetColor (const csColor& col)
   if (!scfParent->lighting)
     for (i = 0 ; i < vertices.Length () ; i++)
       vertices[i].color = col;
+#ifdef CS_USE_NEW_RENDERER  
+  scfParent->colors_dirty = true;
+#endif
 }
 
 void csSprite2DMeshObject::Particle::AddColor (const csColor& col)
@@ -440,6 +727,9 @@ void csSprite2DMeshObject::Particle::AddColor (const csColor& col)
   if (!scfParent->lighting)
     for (i = 0 ; i < vertices.Length () ; i++)
       vertices[i].color = vertices[i].color_init;
+#ifdef CS_USE_NEW_RENDERER  
+  scfParent->colors_dirty = true;
+#endif
 }
 
 void csSprite2DMeshObject::Particle::ScaleBy (float factor)
@@ -448,7 +738,10 @@ void csSprite2DMeshObject::Particle::ScaleBy (float factor)
   int i;
   for (i = 0; i < vertices.Length (); i++)
     vertices[i].pos *= factor;
-  scfParent->shapenr++;
+#ifdef CS_USE_NEW_RENDERER
+  scfParent->vertices_dirty = true;
+#endif
+  scfParent->scfiObjectModel.ShapeChanged ();
 }
 
 void csSprite2DMeshObject::Particle::Rotate (float angle)
@@ -457,75 +750,99 @@ void csSprite2DMeshObject::Particle::Rotate (float angle)
   int i;
   for (i = 0; i < vertices.Length (); i++)
     vertices[i].pos.Rotate (angle);
-  scfParent->shapenr++;
+#ifdef CS_USE_NEW_RENDERER
+  scfParent->vertices_dirty = true;
+#endif
+  scfParent->scfiObjectModel.ShapeChanged ();
 }
 
-void csSprite2DMeshObject::Sprite2DState::SetUVAnimation (const char *name, int style, bool loop)
+void csSprite2DMeshObject::Sprite2DState::SetUVAnimation (const char *name,
+	int style, bool loop)
 {
   if (name)
   {
     iSprite2DUVAnimation *ani = scfParent->factory->GetUVAnimation (name);
     if (ani && ani->GetFrameCount ())
     {
-      scfParent->uvani.animate = true;
-      scfParent->uvani.ani = ani;
-      scfParent->uvani.last_time = 0;
-      scfParent->uvani.frameindex = 0;
-      scfParent->uvani.framecount = ani->GetFrameCount ();
-      scfParent->uvani.frame = ani->GetFrame (0);
-      scfParent->uvani.style = style;
-      scfParent->uvani.counter = 0;
-      scfParent->uvani.loop = loop;
-      scfParent->uvani.halted = false;
+      scfParent->uvani = new uvAnimationControl ();
+      scfParent->uvani->ani = ani;
+      scfParent->uvani->last_time = 0;
+      scfParent->uvani->frameindex = 0;
+      scfParent->uvani->framecount = ani->GetFrameCount ();
+      scfParent->uvani->frame = ani->GetFrame (0);
+      scfParent->uvani->style = style;
+      scfParent->uvani->counter = 0;
+      scfParent->uvani->loop = loop;
+      scfParent->uvani->halted = false;
     }
   }
   else
   {
     // stop animation and show the normal texture
-    scfParent->uvani.animate = false;
+    delete scfParent->uvani;
+    scfParent->uvani = 0;
   }
 }
 
 void csSprite2DMeshObject::Sprite2DState::StopUVAnimation (int idx)
 {
-  if (scfParent->uvani.animate)
+  if (scfParent->uvani)
   {
     if (idx != -1)
     {
-      scfParent->uvani.frameindex = MIN(MAX(idx, 0), scfParent->uvani.framecount-1);
-      scfParent->uvani.frame = scfParent->uvani.ani->GetFrame (scfParent->uvani.frameindex);
+      scfParent->uvani->frameindex = MIN(MAX(idx, 0),
+      	scfParent->uvani->framecount-1);
+      scfParent->uvani->frame = scfParent->uvani->ani->GetFrame (
+      	scfParent->uvani->frameindex);
     }
-    scfParent->uvani.halted = true;
+    scfParent->uvani->halted = true;
   }
 }
 
 void csSprite2DMeshObject::Sprite2DState::PlayUVAnimation (int idx, int style, bool loop)
 {
-  if (scfParent->uvani.animate)
+  if (scfParent->uvani)
   {
     if (idx != -1)
     {
-      scfParent->uvani.frameindex = MIN(MAX(idx, 0), scfParent->uvani.framecount-1);
-      scfParent->uvani.frame = scfParent->uvani.ani->GetFrame (scfParent->uvani.frameindex);
+      scfParent->uvani->frameindex = MIN(MAX(idx, 0), scfParent->uvani->framecount-1);
+      scfParent->uvani->frame = scfParent->uvani->ani->GetFrame (scfParent->uvani->frameindex);
     }
-    scfParent->uvani.halted = false;
-    scfParent->uvani.counter = 0;
-    scfParent->uvani.last_time = 0;
-    scfParent->uvani.loop = loop;
-    scfParent->uvani.style = style;
+    scfParent->uvani->halted = false;
+    scfParent->uvani->counter = 0;
+    scfParent->uvani->last_time = 0;
+    scfParent->uvani->loop = loop;
+    scfParent->uvani->style = style;
   }
 }
 
 int csSprite2DMeshObject::Sprite2DState::GetUVAnimationCount () const
-{ return scfParent->factory->GetUVAnimationCount ();}
+{
+  return scfParent->factory->GetUVAnimationCount ();
+}
+
 iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::CreateUVAnimation ()
-{ return scfParent->factory->CreateUVAnimation ();}
-void csSprite2DMeshObject::Sprite2DState::RemoveUVAnimation (iSprite2DUVAnimation *anim)
-{ scfParent->factory->RemoveUVAnimation (anim); }
-iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (const char *name)
-{ return scfParent->factory->GetUVAnimation (name); }
-iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (int idx)
-{ return scfParent->factory->GetUVAnimation (idx); }
+{
+  return scfParent->factory->CreateUVAnimation ();
+}
+
+void csSprite2DMeshObject::Sprite2DState::RemoveUVAnimation (
+	iSprite2DUVAnimation *anim)
+{
+  scfParent->factory->RemoveUVAnimation (anim);
+}
+
+iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
+	const char *name)
+{
+  return scfParent->factory->GetUVAnimation (name);
+}
+
+iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
+	int idx)
+{
+  return scfParent->factory->GetUVAnimation (idx);
+}
 
 
 void csSprite2DMeshObject::uvAnimationControl::Advance (csTicks current_time)
@@ -610,7 +927,7 @@ const csVector2 *csSprite2DMeshObject::uvAnimationControl::GetVertices (int &num
 // of its bounding box can be hit (if at all).
 
 void csSprite2DMeshObject::CheckBeam (const csVector3& start,
-      const csVector3& pl, float sqr)
+      const csVector3& pl, float sqr, csMatrix3& o2t)
 {
   // This method is an optimized version of LookAt() based on
   // the presumption that the up vector is always (0,1,0).
@@ -620,8 +937,6 @@ void csSprite2DMeshObject::CheckBeam (const csVector3& start,
   // The transformation matrix is stored and used again if the
   // start vector for the beam is in the same position. MHV.
 
-  if (start == cached_start) return;
-  cached_start.Set (start);
   csVector3 pl2 = pl * qisqrt (sqr);
   csVector3 v1( pl2.z, 0, -pl2.x);
   sqr = v1*v1;
@@ -643,7 +958,8 @@ bool csSprite2DMeshObject::HitBeamOutline(const csVector3& start,
   float dist;
   csIntersect3::Plane(start, end, pl, 0, isect, dist);
   if (pr) *pr = dist;
-  CheckBeam (start, pl, sqr);
+  csMatrix3 o2t;
+  CheckBeam (start, pl, sqr, o2t);
   csVector3 r = o2t * isect;
   int trail, len = vertices.Length();
   trail = len - 1;
@@ -666,26 +982,32 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObjectFactory::Sprite2DFactoryState)
   SCF_IMPLEMENTS_INTERFACE (iSprite2DFactoryState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iBase *pParent)
+csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iBase *pParent,
+	iObjectRegistry* object_reg)
 {
   SCF_CONSTRUCT_IBASE (pParent);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiSprite2DFactoryState);
-  material = NULL;
+  material = 0;
   MixMode = 0;
   lighting = true;
-  logparent = NULL;
+  logparent = 0;
+  csSprite2DMeshObjectFactory::object_reg = object_reg;
+  light_mgr = CS_QUERY_REGISTRY (object_reg, iLightManager);
+  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
 }
 
 csSprite2DMeshObjectFactory::~csSprite2DMeshObjectFactory ()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiSprite2DFactoryState);
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObject* csSprite2DMeshObjectFactory::NewInstance ()
+csPtr<iMeshObject> csSprite2DMeshObjectFactory::NewInstance ()
 {
   csSprite2DMeshObject* cm = new csSprite2DMeshObject (this);
-  iMeshObject* im = SCF_QUERY_INTERFACE (cm, iMeshObject);
-  im->DecRef ();
-  return im;
+  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (cm, iMeshObject));
+  cm->DecRef ();
+  return csPtr<iMeshObject> (im);
 }
 
 //----------------------------------------------------------------------
@@ -701,10 +1023,6 @@ SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csSprite2DMeshObjectType)
 
-SCF_EXPORT_CLASS_TABLE (spr2d)
-  SCF_EXPORT_CLASS (csSprite2DMeshObjectType, "crystalspace.mesh.object.sprite.2d",
-    "Crystal Space Sprite2D Mesh Type")
-SCF_EXPORT_CLASS_TABLE_END
 
 csSprite2DMeshObjectType::csSprite2DMeshObjectType (iBase* pParent)
 {
@@ -714,13 +1032,22 @@ csSprite2DMeshObjectType::csSprite2DMeshObjectType (iBase* pParent)
 
 csSprite2DMeshObjectType::~csSprite2DMeshObjectType ()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObjectFactory* csSprite2DMeshObjectType::NewFactory ()
+csPtr<iMeshObjectFactory> csSprite2DMeshObjectType::NewFactory ()
 {
-  csSprite2DMeshObjectFactory* cm = new csSprite2DMeshObjectFactory (this);
-  iMeshObjectFactory* ifact = SCF_QUERY_INTERFACE (cm, iMeshObjectFactory);
-  ifact->DecRef ();
-  return ifact;
+  csSprite2DMeshObjectFactory* cm = new csSprite2DMeshObjectFactory (this,
+  	object_reg);
+  csRef<iMeshObjectFactory> ifact =
+  	SCF_QUERY_INTERFACE (cm, iMeshObjectFactory);
+  cm->DecRef ();
+  return csPtr<iMeshObjectFactory> (ifact);
 }
 
+bool csSprite2DMeshObjectType::Initialize (iObjectRegistry* object_reg)
+{
+  csSprite2DMeshObjectType::object_reg = object_reg;
+  return true;
+}
