@@ -17,28 +17,49 @@
 */
 
 #include "cssysdef.h"
-#include "cssys/sysfunc.h"
+#include "csutil/sysfunc.h"
 #include "csutil/scf.h"
 #include "csutil/csstring.h"
-#include "g2d.h"
-#include "video/canvas/directxcommon/directdetection.h"
 #include "csgeom/csrect.h"
 #include "csutil/util.h"
 #include "iutil/event.h"
 #include "iutil/objreg.h"
 #include "ivaria/reporter.h"
-#include "cssys/win32/win32.h"
+#include "csutil/win32/win32.h"
 #include "iutil/cmdline.h"
 
+#define INITGUID
+#include "plugins/video/canvas/directxcommon/directdetection.h"
+
+#include "csutil/win32/wintools.h"
+#include "plugins/video/canvas/common/softfontcache.h"
+
+#include "g2d.h"
+
 #ifndef DD_FALSE
   // This is normally being done in the ddraw.h file
   #define DD_FALSE S_FALSE
 #endif
 
 #define WINDOW_STYLE (WS_CAPTION | WS_MINIMIZEBOX | WS_POPUP | WS_SYSMENU)
+#define FULLSCREEN_STYLE (WS_POPUP | WS_SYSMENU)
 
-static DirectDetection DDetection;
-static DirectDetectionDevice *DirectDevice;
+// These don't exist on some older SDKs
+#ifndef _WIN64
+  #ifndef SetWindowLongPtrA
+    #define SetWindowLongPtrA SetWindowLongA
+  #endif
+  #ifndef SetWindowLongPtrW
+    #define SetWindowLongPtrW SetWindowLongW
+  #endif
+  
+  #ifndef GetWindowLongPtrA
+    #define GetWindowLongPtrA GetWindowLongA
+  #endif
+  #ifndef GetWindowLongPtrW
+    #define GetWindowLongPtrW GetWindowLongW
+  #endif
+#endif
 
 //--//--//--//--//--//--//--//--//--//--//--//--//-- csGraphics2DDDraw3 --//--//
 
@@ -46,19 +67,16 @@ CS_IMPLEMENT_PLUGIN
 
 SCF_IMPLEMENT_FACTORY (csGraphics2DDDraw3)
 
-SCF_EXPORT_CLASS_TABLE (csddraw)
-  SCF_EXPORT_CLASS_DEP (csGraphics2DDDraw3, CS_SOFTWARE_2D_DRIVER,
-    "Crystal Space 2D DirectDraw driver", "crystalspace.font.server.")
-SCF_EXPORT_CLASS_TABLE_END
 
 csGraphics2DDDraw3::csGraphics2DDDraw3(iBase *iParent) :
   csGraphics2D (iParent),
-  m_lpDD (NULL),
-  m_lpddsPrimary (NULL),
-  m_lpddsBack (NULL),
-  m_lpddPal (NULL),
-  m_hWnd (NULL),
-  m_hWndPalette (NULL),
+  m_lpDD (0),
+  m_lpddsPrimary (0),
+  m_lpddsBack (0),
+  m_lpddPal (0),
+  m_hWnd (0),
+  m_hWndPalette (0),
+  m_piWin32Assistant (0),
   m_bPalettized (false),
   m_bPaletteChanged (false),
   m_nActivePage (0),
@@ -66,7 +84,7 @@ csGraphics2DDDraw3::csGraphics2DDDraw3(iBase *iParent) :
   m_bDoubleBuffer (false),
   m_bAllowWindowed (false)
 {
-  m_hInstance = GetModuleHandle (NULL);
+  m_hInstance = GetModuleHandle (0);
 }
 
 csGraphics2DDDraw3::~csGraphics2DDDraw3 ()
@@ -78,12 +96,9 @@ void csGraphics2DDDraw3::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  iReporter* rep = CS_QUERY_REGISTRY (object_reg, iReporter);
+  csRef<iReporter> rep = CS_QUERY_REGISTRY (object_reg, iReporter);
   if (rep)
-  {
     rep->ReportV (severity, "crystalspace.canvas.ddraw", msg, arg);
-    rep->DecRef ();
-  }
   else
   {
     csPrintfV (msg, arg);
@@ -97,12 +112,23 @@ bool csGraphics2DDDraw3::Initialize (iObjectRegistry *object_reg)
   if (!csGraphics2D::Initialize (object_reg))
     return false;
 
-  iCommandLineParser* cmdline = CS_QUERY_REGISTRY (object_reg,
+  m_piWin32Assistant = CS_QUERY_REGISTRY (object_reg, iWin32Assistant);
+  if (!m_piWin32Assistant)
+  {
+    MessageBox (0, 
+      "csGraphics2DDDraw3::Open(QI) -- system passed does not support iWin32Assistant.",
+      0,
+      MB_OK | MB_ICONERROR);
+    exit(1);
+  }
+
+  DDetection.object_reg = object_reg;
+
+  csRef<iCommandLineParser> cmdline = CS_QUERY_REGISTRY (object_reg,
 						   iCommandLineParser);
   m_bHardwareCursor = config->GetBool ("Video.SystemMouseCursor", true);
   if (cmdline->GetOption ("sysmouse")) m_bHardwareCursor = true;
   if (cmdline->GetOption ("nosysmouse")) m_bHardwareCursor = false;
-  cmdline->DecRef ();
 
   return true;
 }
@@ -126,35 +152,47 @@ bool csGraphics2DDDraw3::Open ()
   m_rcWindow.bottom = m_rcWindow.top + wheight;
 
   // create the window.
-  m_hWnd = CreateWindow (CS_WIN32_WINDOW_CLASS_NAME, win_title, 0,
-    m_rcWindow.left, m_rcWindow.top, m_rcWindow.right - m_rcWindow.left,
-    m_rcWindow.bottom - m_rcWindow.top, NULL, NULL, m_hInstance, NULL);
+  if (cswinIsWinNT ())
+  {
+    m_hWnd = CreateWindowW (CS_WIN32_WINDOW_CLASS_NAMEW, 0, 0,
+      m_rcWindow.left, m_rcWindow.top, m_rcWindow.right - m_rcWindow.left,
+      m_rcWindow.bottom - m_rcWindow.top, 0, 0, m_hInstance, 0);
+  }
+  else
+  {
+    m_hWnd = CreateWindowA (CS_WIN32_WINDOW_CLASS_NAME, 0, 0,
+      m_rcWindow.left, m_rcWindow.top, m_rcWindow.right - m_rcWindow.left,
+      m_rcWindow.bottom - m_rcWindow.top, 0, 0, m_hInstance, 0);
+  }
   ASSERT (m_hWnd);
 
-  // Subclass the window
-  m_OldWndProc = (WNDPROC)(m_hWnd, GWL_WNDPROC, (LONG) WindowProc);
-  SetWindowLong (m_hWnd, GWL_USERDATA, (LONG)this);
+  SetTitle (win_title);
 
-  // Decide whenever we allow windowed mode at all
-  hdc = GetDC (m_hWnd);
-  m_bAllowWindowed = (GetDeviceCaps (hdc, RASTERCAPS) == Depth);
-  ReleaseDC (m_hWnd, hdc);
-  if (FullScreen && !m_bAllowWindowed)
-    FullScreen = true;
+  // Subclass the window
+  if (cswinIsWinNT ())
+  {
+    m_OldWndProc = (WNDPROC)GetWindowLongPtrW (m_hWnd, GWL_WNDPROC);
+    SetWindowLongPtrW (m_hWnd, GWL_WNDPROC, (LONG_PTR)WindowProc);
+    SetWindowLongPtrW (m_hWnd, GWL_USERDATA, (LONG_PTR)this);
+  }
   else
-    FullScreen = false;
+  {
+    m_OldWndProc = (WNDPROC)GetWindowLongA (m_hWnd, GWL_WNDPROC);
+    SetWindowLongPtrA (m_hWnd, GWL_WNDPROC, (LONG_PTR)WindowProc);
+    SetWindowLongPtrA (m_hWnd, GWL_USERDATA, (LONG_PTR)this);
+  }
 
   // Get ahold of the main DirectDraw object...
   DDetection.checkDevices2D ();
-  DirectDevice = DDetection.findBestDevice2D ();
+  DirectDevice = DDetection.findBestDevice2D (DisplayNumber);
 
-  if (DirectDevice == NULL)
+  if (DirectDevice == 0)
   {
     InitFail (DD_FALSE, "Error creating DirectDevice\n");
     return false;
   }
 
-  LPGUID pGuid = NULL;
+  LPGUID pGuid = 0;
   if (!DirectDevice->IsPrimary2D)
     pGuid = &DirectDevice->Guid2D;
 
@@ -163,20 +201,33 @@ bool csGraphics2DDDraw3::Open ()
 
   // Create a DD object for either the primary device or the secondary.
   HRESULT ddrval;
-  if ((ddrval = DirectDrawCreate (pGuid, &m_lpDD, NULL)) != DD_OK)
+  if ((ddrval = DirectDrawCreate (pGuid, &m_lpDD, 0)) != DD_OK)
   {
     InitFail (ddrval, "DirectDrawCreate FAILED (Code: %08lx)\n");
     return false;
   }
 
-  Memory = NULL;
-  m_hWndPalette = NULL;
+  Memory = 0;
+  m_hWndPalette = 0;
   // Default to no double buffering since usually this is SLOW
   m_bDoubleBuffer = false;
 
+  HDC DC = GetDC (0);
+  int desktopDepth = GetDeviceCaps (DC, BITSPIXEL);
+  ReleaseDC (0, DC);
   if (InitSurfaces () != DD_OK)
     return false;
 
+  // Determine if switching from FS to windowed is allowed
+  m_bAllowWindowed = !FullScreen || (desktopDepth == Depth);
+  if (FullScreen)
+  {
+    if (m_bAllowWindowed)
+      Report (CS_REPORTER_SEVERITY_NOTIFY, "Windowed mode allowed");
+    else
+      Report (CS_REPORTER_SEVERITY_NOTIFY, "Windowed mode not allowed");
+  }
+
   return true;
 }
 
@@ -191,19 +242,22 @@ void csGraphics2DDDraw3::Close ()
     if (m_lpddPal)
     {
       m_lpddPal->Release ();
-      m_lpddPal = NULL;
+      m_lpddPal = 0;
     }
     m_lpDD->Release ();
-    m_lpDD = NULL;
+    m_lpDD = 0;
   }
 
+  if (m_hWnd != 0)
+    DestroyWindow (m_hWnd);
+
   if (!FullScreen)
   {
     // restore the original system palette.
-    HDC dc = GetDC (NULL);
+    HDC dc = GetDC (0);
     SetSystemPaletteUse (dc, SYSPAL_STATIC);
     PostMessage (HWND_BROADCAST, WM_SYSCOLORCHANGE, 0, 0);
-    ReleaseDC (NULL, dc);
+    ReleaseDC (0, dc);
   }
 
   csGraphics2D::Close ();
@@ -231,7 +285,7 @@ bool csGraphics2DDDraw3::GetDoubleBufferState ()
   return (FullScreen && m_bDoubleBuffer);
 }
 
-void csGraphics2DDDraw3::Print (csRect *area)
+void csGraphics2DDDraw3::Print (csRect const* area)
 {
   bool loop = true;
 
@@ -246,7 +300,7 @@ void csGraphics2DDDraw3::Print (csRect *area)
     if (FullScreen && m_bDoubleBuffer)
     {
       // We are in fullscreen mode, so perform a flip.
-      hRet = m_lpddsPrimary->Flip (NULL, DDFLIP_WAIT);
+      hRet = m_lpddsPrimary->Flip (0, DDFLIP_WAIT);
       m_nActivePage ^= 1;
     }
     else
@@ -279,7 +333,7 @@ void csGraphics2DDDraw3::Print (csRect *area)
       }
 
       hRet = m_lpddsPrimary->Blt (&rcScreen, m_lpddsBack,
-        area ? &rcSource : NULL, DDBLT_WAIT, NULL);
+        area ? &rcSource : 0, DDBLT_WAIT, 0);
 
       if (m_bPalettized)
         SelectPalette (hdc, oldPal, FALSE);
@@ -311,6 +365,8 @@ void csGraphics2DDDraw3::Refresh (RECT &rect)
    || (rect.right <= rect.left) || (rect.bottom <= rect.top))
     return;
 
+  if (!m_lpddsPrimary) return;
+
   bool loop = true;
   while (loop)
   {
@@ -338,7 +394,7 @@ void csGraphics2DDDraw3::Refresh (RECT &rect)
     }
 
     hRet = m_lpddsPrimary->Blt (&rcScreen, m_lpddsBack,
-      &rcSource, DDBLT_WAIT, NULL);
+      &rcSource, DDBLT_WAIT, 0);
 
     if (m_bPalettized)
       SelectPalette (hdc, oldPal, FALSE);
@@ -373,11 +429,11 @@ bool csGraphics2DDDraw3::BeginDraw ()
 
   DDSURFACEDESC ddsd;
   ddsd.dwSize = sizeof (ddsd);
-  ddsd.lpSurface = NULL;
+  ddsd.lpSurface = 0;
 
   HRESULT ret = DDERR_WASSTILLDRAWING;
   while (ret == DDERR_WASSTILLDRAWING)
-    ret = m_lpddsBack->Lock (NULL, &ddsd, DDLOCK_SURFACEMEMORYPTR, NULL);
+    ret = m_lpddsBack->Lock (0, &ddsd, DDLOCK_SURFACEMEMORYPTR, 0);
 
   if (ret != DD_OK)
   {
@@ -399,9 +455,9 @@ void csGraphics2DDDraw3::FinishDraw ()
 
   if (m_bLocked)
   {
-    m_lpddsBack->Unlock (NULL);
+    m_lpddsBack->Unlock (0);
     m_bLocked = false;
-    Memory = NULL;
+    Memory = 0;
   }
 }
 
@@ -416,16 +472,16 @@ void csGraphics2DDDraw3::SetColorPalette ()
     if (m_lpddPal)
     {
       m_lpddPal->Release ();
-      m_lpddPal = NULL;
+      m_lpddPal = 0;
     }
 
-    ret = m_lpDD->CreatePalette (DDPCAPS_8BIT, (PALETTEENTRY *)Palette, &m_lpddPal, NULL);
+    ret = m_lpDD->CreatePalette (DDPCAPS_8BIT, (PALETTEENTRY *)Palette, &m_lpddPal, 0);
     if (ret == DD_OK) m_lpddsPrimary->SetPalette (m_lpddPal);
 
     if (!FullScreen)
     {
       HPALETTE oldPal;
-      HDC dc = GetDC (NULL);
+      HDC dc = GetDC (0);
 
       SetSystemPaletteUse (dc, SYSPAL_NOSTATIC);
       PostMessage (HWND_BROADCAST, WM_SYSCOLORCHANGE, 0, 0);
@@ -442,7 +498,7 @@ void csGraphics2DDDraw3::SetColorPalette ()
 
       RealizePalette (dc);
       SelectPalette (dc, oldPal, FALSE);
-      ReleaseDC (NULL, dc);
+      ReleaseDC (0, dc);
     }
   }
 }
@@ -455,16 +511,13 @@ void csGraphics2DDDraw3::SetRGB (int i, int r, int g, int b)
 
 bool csGraphics2DDDraw3::SetMouseCursor (csMouseCursorID iShape)
 {
-  iWin32Assistant* winhelper = CS_QUERY_REGISTRY (object_reg, iWin32Assistant);
-  CS_ASSERT (winhelper != NULL);
-  bool rc;
+bool rc;
   if (!m_bHardwareCursor) {
-    winhelper->SetCursor (csmcNone);
+    m_piWin32Assistant->SetCursor (csmcNone);
     rc = false;
   } else {
-    rc = winhelper->SetCursor (iShape);
+    rc = m_piWin32Assistant->SetCursor (iShape);
   }
-  winhelper->DecRef ();
   return rc;
 }
 
@@ -486,7 +539,7 @@ bool csGraphics2DDDraw3::PerformExtensionV (char const* command, va_list args)
   bool rc = true;
   if (!strcmp (command, "fullscreen"))
   {
-    bool fs = va_arg (args, bool);
+    bool fs = bool(va_arg (args, int));
     if (fs != FullScreen
      && (m_bAllowWindowed || fs))
     {
@@ -494,7 +547,17 @@ bool csGraphics2DDDraw3::PerformExtensionV (char const* command, va_list args)
       if (!FullScreen)
         GetWindowRect (m_hWnd, &m_rcWindow);
       FullScreen = fs;
-      ChangeCoopLevel ();
+      if (FAILED(ChangeCoopLevel ()))
+	Report (CS_REPORTER_SEVERITY_WARNING,
+	  "ChangeCoopLevel() failed!");
+    } 
+    else
+    {
+      if (!(m_bAllowWindowed || fs))
+      {
+	Report (CS_REPORTER_SEVERITY_NOTIFY,
+	  "Windowed mode not available!");
+      }
     }
   }
   else
@@ -513,12 +576,12 @@ HRESULT csGraphics2DDDraw3::ReleaseAllObjects ()
     if (m_lpddsBack)
     {
       m_lpddsBack->Release ();
-      m_lpddsBack = NULL;
+      m_lpddsBack = 0;
     }
     if (m_lpddsPrimary)
     {
       m_lpddsPrimary->Release ();
-      m_lpddsPrimary = NULL;
+      m_lpddsPrimary = 0;
     }
     m_lpDD->SetCooperativeLevel (m_hWnd, DDSCL_NORMAL);
   }
@@ -531,30 +594,51 @@ HRESULT csGraphics2DDDraw3::ReleaseAllObjects ()
 //-----------------------------------------------------------------------------
 HRESULT csGraphics2DDDraw3::InitSurfaces ()
 {
-  HRESULT hRet;
+  HRESULT hRet = 0;
   DDSURFACEDESC ddsd;
   DDSCAPS ddscaps;
   DDPIXELFORMAT ddpf;
 
+  if (FullScreen)
+  {
+    // Set FS video mode
+#if DIRECTDRAW_VERSION >= 0x0700
+    // changing the display frequency is a dd7+ feature
+    LPDIRECTDRAW7 lpDD7;
+    if (m_lpDD->QueryInterface (IID_IDirectDraw7, (LPVOID*)&lpDD7) == S_OK)
+    {
+      hRet = lpDD7->SetDisplayMode (Width, Height, Depth, refreshRate, 0);
+      lpDD7->Release ();
+    }
+#else
+    int lpDD7 = 0;
+#endif
+    // either we have no dd7 or the modeswitch failed
+    if ((!lpDD7) || (hRet != DD_OK))
+    {
+      // maybe just the monitor frequency is not supported.
+      // so try without setting it
+      hRet = m_lpDD->SetDisplayMode (Width, Height, Depth);
+    }
+    if (hRet != DD_OK)
+    {
+      Report (CS_REPORTER_SEVERITY_WARNING,
+        "SetDisplayMode FAILED (Code: %08lx); will try windowed mode", hRet);
+      FullScreen = false;
+    }
+  }
+
   hRet = m_lpDD->SetCooperativeLevel (m_hWnd, FullScreen ?
     (DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN) : DDSCL_NORMAL);
   if (hRet != DD_OK)
     return InitFail (hRet, "SetCooperativeLevel FAILED (Code: %08lx)\n");
 
   // Set window style bits
-  SetWindowLong (m_hWnd, GWL_STYLE, FullScreen ? 0 : WINDOW_STYLE);
+  SetWindowLong (m_hWnd, GWL_STYLE, FullScreen ? FULLSCREEN_STYLE : WINDOW_STYLE);
   // Call SetWindowPos so that the change takes effect
   SetWindowPos (m_hWnd, HWND_TOP, 0, 0, 0, 0,
     SWP_FRAMECHANGED | SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
 
-  if (FullScreen)
-  {
-    // Set FS video mode
-    hRet = m_lpDD->SetDisplayMode (Width, Height, Depth);
-    if (hRet != DD_OK)
-      return InitFail (hRet, "SetDisplayMode FAILED (Code: %08lx)\n");
-  }
-
   if (!FullScreen || !m_bDoubleBuffer)
   {
     // Create the primary surface
@@ -562,7 +646,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     ddsd.dwSize = sizeof (ddsd);
     ddsd.dwFlags = DDSD_CAPS;
     ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
-    if ((hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsPrimary, NULL)) != DD_OK)
+    if ((hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsPrimary, 0)) != DD_OK)
       return InitFail (hRet, "Cannot create primary surface for DirectDraw (Code: %08lx)\n");
 
     // Create the backbuffer. In fullscreen mode by default we don't
@@ -571,7 +655,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     ddsd.dwWidth = Width;
     ddsd.dwHeight = Height;
     ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
-    if ((hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsBack, NULL)) != DD_OK)
+    if ((hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsBack, 0)) != DD_OK)
     {
       InitFail (hRet, "CreateSurface for backbuffer FAILED (Code: %08lx)\n");
       return false;
@@ -581,7 +665,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     {
       // Create a clipper object since this is for a Windowed render
       LPDIRECTDRAWCLIPPER pClipper;
-      if ((hRet = m_lpDD->CreateClipper (0, &pClipper, NULL)) != DD_OK)
+      if ((hRet = m_lpDD->CreateClipper (0, &pClipper, 0)) != DD_OK)
         return InitFail (hRet, "CreateClipper FAILED (Code: %08lx)\n");
 
       // Associate the clipper with the window
@@ -599,7 +683,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
     ddsd.dwBackBufferCount = 1;
 
-    hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsPrimary, NULL);
+    hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsPrimary, 0);
     if (hRet != DD_OK)
       return InitFail (hRet, "Cannot create primary surface for DirectDraw FAILED (Code: %08lx)\n");
 
@@ -622,32 +706,45 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
   pfmt.BlueMask = ddpf.dwBBitMask;
   Depth = ddpf.dwRGBBitCount;
 
+  if (fontCache) delete fontCache;
   if (Depth == 8)
   {
     pfmt.PalEntries = 256;
     pfmt.PixelBytes = 1;
+
+    fontCache = new csSoftFontCache8 (this);
   }
   else if (Depth == 16)
   {
     _DrawPixel = DrawPixel16;
-    _WriteString = WriteString16;
     _GetPixelAt = GetPixelAt16;
 
     // Set pixel format
     pfmt.PixelBytes = 2;
     pfmt.PalEntries = 0;
+
+    if (ddpf.dwGBitMask == 0x03e0)
+    {
+      fontCache = new csSoftFontCache16_555 (this);
+    }
+    else
+    {
+      fontCache = new csSoftFontCache16_565 (this);
+    }
   }
   else if (Depth == 32)
   {
     _DrawPixel = DrawPixel32;
-    _WriteString = WriteString32;
     _GetPixelAt = GetPixelAt32;
 
     // calculate CS's pixel format structure.
     pfmt.PixelBytes = 4;
     pfmt.PalEntries = 0;
+    
+    fontCache = new csSoftFontCache32 (this);
   }
   pfmt.complete ();
+  fontCache->SetClipRect (ClipX1, ClipY1, ClipX2, ClipY2);
 
   m_lpddsBack->GetSurfaceDesc (&ddsd);
   int i;
@@ -670,7 +767,7 @@ HRESULT csGraphics2DDDraw3::ChangeCoopLevel ()
   int i;
 
   // First of all save the contents of backbuffer since it will be cleared
-  char *oldBuffer = NULL;
+  char *oldBuffer = 0;
   if (BeginDraw ())
   {
     size_t BytesPerLine = Width * ((Depth + 7) / 8);
@@ -697,17 +794,20 @@ HRESULT csGraphics2DDDraw3::ChangeCoopLevel ()
   hRet = InitSurfaces ();
 
   // Now restore the contents of backbuffer
-  int times;
-  for (times = (m_bDoubleBuffer && FullScreen) ? 2 : 1; times; times--)
-    if (BeginDraw ())
-    {
-      size_t BytesPerLine = Width * ((Depth + 7) / 8);
-      for (i = 0; i < Height; i++)
-        memcpy (Memory + LineAddress [i], oldBuffer + i * BytesPerLine, BytesPerLine);
-      FinishDraw ();
-      Print (NULL);
-    }
-  delete [] oldBuffer;
+  if (oldBuffer)
+  {
+    int times;
+    for (times = (m_bDoubleBuffer && FullScreen) ? 2 : 1; times; times--)
+      if (BeginDraw ())
+      {
+	size_t BytesPerLine = Width * ((Depth + 7) / 8);
+	for (i = 0; i < Height; i++)
+	  memcpy (Memory + LineAddress [i], oldBuffer + i * BytesPerLine, BytesPerLine);
+	FinishDraw ();
+	Print (0);
+      }
+    delete [] oldBuffer;
+  }
 
   return hRet;
 }
@@ -747,7 +847,7 @@ void csGraphics2DDDraw3::ClearSystemPalette ()
   Palette->palPalEntry [255].peFlags = 0;
 
   int c;
-  for (c = 0; c < 256; c++)
+  for (c = 1; c < 255; c++)
   {
     Palette->palPalEntry [c].peRed = 0;
     Palette->palPalEntry [c].peGreen = 0;
@@ -755,7 +855,7 @@ void csGraphics2DDDraw3::ClearSystemPalette ()
     Palette->palPalEntry [c].peFlags = 0;
   }
 
-  hdc = GetDC (NULL);
+  hdc = GetDC (0);
 
   BlackPal = CreatePalette (Palette);
   free((void*)Palette);
@@ -765,7 +865,7 @@ void csGraphics2DDDraw3::ClearSystemPalette ()
   SelectPalette (hdc, OldPal, FALSE);
   DeleteObject (BlackPal);
 
-  ReleaseDC (NULL, hdc);
+  ReleaseDC (0, hdc);
 }
 
 bool csGraphics2DDDraw3::CreateIdentityPalette (csRGBpixel *p)
@@ -790,7 +890,7 @@ bool csGraphics2DDDraw3::CreateIdentityPalette (csRGBpixel *p)
   Palette->palPalEntry [255].peBlue = 255;
   Palette->palPalEntry [255].peFlags = 0;
 
-  for (i = 0; i < 256; i++)
+  for (i = 1; i < 255; i++)
   {
     Palette->palPalEntry [i].peRed = p [i].red;
     Palette->palPalEntry [i].peGreen = p [i].green;
@@ -810,21 +910,17 @@ bool csGraphics2DDDraw3::CreateIdentityPalette (csRGBpixel *p)
 LRESULT CALLBACK csGraphics2DDDraw3::WindowProc (HWND hWnd, UINT message,
   WPARAM wParam, LPARAM lParam)
 {
-  csGraphics2DDDraw3 *This = (csGraphics2DDDraw3 *)GetWindowLong (hWnd, GWL_USERDATA);
+  csGraphics2DDDraw3 *This = (csGraphics2DDDraw3 *)GetWindowLongPtrA (hWnd, GWL_USERDATA);
   switch (message)
   {
     case WM_PAINT:
       if (!This->FullScreen || !This->m_bDoubleBuffer)
       {
-        RECT rect;
-        if (GetUpdateRect (hWnd, &rect, FALSE))
-        {
-          PAINTSTRUCT ps;
-          BeginPaint (hWnd, &ps);
-          This->Refresh (rect);
-          EndPaint (hWnd, &ps);
-          return TRUE;
-        }
+        PAINTSTRUCT ps;
+        BeginPaint (hWnd, &ps);
+	This->Refresh (ps.rcPaint);
+        EndPaint (hWnd, &ps);
+        return TRUE;
       }
       break;
     case WM_SYSKEYDOWN:
@@ -841,6 +937,46 @@ LRESULT CALLBACK csGraphics2DDDraw3::WindowProc (HWND hWnd, UINT message,
       if (wParam == SC_KEYMENU)
         return TRUE;
       break;
+    case WM_DESTROY:
+      This->m_hWnd = 0;
+      break;
+  }
+  if (cswinIsWinNT ())
+  {
+    return CallWindowProcW ((WNDPROC)This->m_OldWndProc, hWnd, message, wParam, lParam);
+  }
+  else
+  {
+    return CallWindowProcA ((WNDPROC)This->m_OldWndProc, hWnd, message, wParam, lParam);
+  }
+}
+
+void csGraphics2DDDraw3::SetTitle (const char* title)
+{
+  csGraphics2D::SetTitle (title);
+  if (m_hWnd)
+  {
+    if (cswinIsWinNT ())
+    {
+      SetWindowTextW (m_hWnd, csCtoW (title));
+    }
+    else
+    {
+      SetWindowTextA (m_hWnd, cswinCtoA (title));
+    }
+  }
+}
+
+void csGraphics2DDDraw3::AlertV (int type, const char* title, 
+    const char* okMsg, const char* msg, va_list args)
+{
+  if (FullScreen)
+  {
+    m_lpDD->SetCooperativeLevel (m_hWnd, DDSCL_NORMAL);  
+  }
+  m_piWin32Assistant->AlertV (m_hWnd, type, title, okMsg, msg, args);
+  if (FullScreen)
+  {
+    ChangeCoopLevel();
   }
-  return CallWindowProc (This->m_OldWndProc, hWnd, message, wParam, lParam);
 }
