@@ -7,20 +7,21 @@
  *
  */
 
-// This code must be callable from the ObjC delegate.  Since that uses the standard C compiler, it doesn't like
-// C++ classes, so we create a C API to some functions of this object
+// This code must be callable from the ObjC delegate.  Since that uses the
+// standard C compiler, it doesn't like C++ classes, so we create a C API to
+// some functions of this object
 
-#ifndef __CGDRIVER2D_H
-#define __CGDRIVER2D_H
+#ifndef __CS_CGDRIVER2D_H__
+#define __CS_CGDRIVER2D_H__
 
 
-#include "OSXDriver2D.h"
+#include "plugins/video/canvas/macosx/common/OSXDriver2D.h"
 
 
 #if defined(__cplusplus)
 
-#include "cssys/next/NeXTAssistant.h"
-#include "video/canvas/common/graph2d.h"
+#include "csutil/macosx/OSXAssistant.h"
+#include "plugins/video/canvas/common/graph2d.h"
 
 #include <CoreFoundation/CoreFoundation.h>
 
@@ -47,7 +48,10 @@ public:
     virtual void SetTitle(char *title);
 
     // Flip video page (or dump to framebuffer)
-    virtual void Print(csRect *area = NULL);
+    virtual void Print(csRect const* area = 0);
+
+    // Set mouse position
+    virtual bool SetMousePosition(int x, int y);
 
     // Set the mouse cursor
     virtual bool SetMouseCursor(csMouseCursorID cursor);
@@ -64,6 +68,6 @@ protected:
     virtual void SetupDrawingFunctions();
 };
 
-#endif
+#endif // __cplusplus
 
-#endif
+#endif // __CS_CGDRIVER2D_H__
