@@ -6,14 +6,16 @@
 //  Copyright (c) 2001 Matt Reda. All rights reserved.
 //
 
+#ifndef __CS_OSXDELEGATE2D_OPENGL_H__
+#define __CS_OSXDELEGATE2D_OPENGL_H__
 
 // C interface to a OSXDelegate2D category for creating an OpenGL context
 
 #if defined(__cplusplus)
 
-#include "OSXDelegate2D.h"
+#include "plugins/video/canvas/macosx/common/OSXDelegate2D.h"
 
-#define DEL2D_FUNC(ret, func) __private_extern__ "C" inline ret OSXDelegate2D_##func
+#define DEL2D_FUNC(ret, func) __private_extern__ "C" ret OSXDelegate2D_##func
 
 
 DEL2D_FUNC(CGLContextObj, createOpenGLContext)(OSXDelegate2D delegate, int depth,
@@ -22,4 +24,6 @@ DEL2D_FUNC(void, updateOpenGLContext)(OSXDelegate2D delegate);
 
 #undef DEL2D_FUNC
 
-#endif
+#endif // __cplusplus
+
+#endif // __CS_OSXDELEGATE2D_OPENGL_H__
